@@ -1,6 +1,5 @@
-/* -*- mode: C -*-  */
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2010-2012  Gabor Csardi <csardi.gabor@gmail.com>
    334 Harvard street, Cambridge, MA 02139 USA
 
@@ -53,6 +52,9 @@ int main(void) {
     igraph_vector_t weight;
     igraph_attribute_combination_t comb;
 
+    /* Initialize the library. */
+    igraph_setup();
+
     igraph_set_attribute_table(&igraph_cattribute_table);
 
     igraph_small(&g, 4, IGRAPH_DIRECTED,
