@@ -1,6 +1,5 @@
-/* -*- mode: C -*-  */
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2012  Gabor Csardi <csardi.gabor@gmail.com>
    334 Harvard street, Cambridge, MA 02139 USA
 
@@ -24,7 +23,7 @@
 #include <igraph.h>
 
 void print_maps(igraph_vector_int_t *map, igraph_vector_int_list_t *maps) {
-    igraph_integer_t n, i;
+    igraph_int_t n, i;
     igraph_vector_int_print(map);
     n = igraph_vector_int_list_size(maps);
     for (i = 0; i < n; i++) {
@@ -39,7 +38,7 @@ int main(void) {
     igraph_bool_t iso;
     igraph_vector_int_t map;
     igraph_vector_int_list_t maps;
-    igraph_integer_t i;
+    igraph_int_t i;
     int domainsvec[] = { 0, 2, 8, -1,
                          4, 5, 6, 7, -1,
                          1, 3, 5, 6, 7, 8, -1,
@@ -49,6 +48,9 @@ int main(void) {
     igraph_vector_int_list_t domains;
     igraph_vector_int_t v;
 
+    /* Initialize the library. */
+    igraph_setup();
+
     igraph_small(&target, 9, IGRAPH_UNDIRECTED,
                  0, 1, 0, 4, 0, 6,
                  1, 4, 1, 2,
@@ -70,7 +72,7 @@ int main(void) {
     igraph_vector_int_list_init(&maps, 0);
 
     igraph_subisomorphic_lad(&pattern, &target, /*domains=*/ NULL, &iso, &map,
-                             &maps, /*induced=*/ false, /*time_limit=*/ 0);
+                             &maps, /*induced=*/ false);
 
     if (!iso) {
         return 1;
@@ -80,7 +82,7 @@ int main(void) {
     printf("---------\n");
 
     igraph_subisomorphic_lad(&pattern, &target, /*domains=*/ NULL, &iso, &map,
-                             &maps, /*induced=*/ true, /*time_limit=*/ 0);
+                             &maps, /*induced=*/ true);
 
     if (!iso) {
         return 2;
@@ -106,7 +108,7 @@ int main(void) {
     igraph_vector_int_destroy(&v);
 
     igraph_subisomorphic_lad(&pattern, &target, &domains, &iso, &map, &maps,
-                             /*induced=*/ false, /*time_limit=*/ 0);
+                             /*induced=*/ false);
 
     if (!iso) {
         return 3;
