@@ -1,7 +1,5 @@
-/* -*- mode: C -*-  */
-/* vim:set ts=4 sw=4 sts=4 et: */
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2007-2021  The igraph development team <igraph@igraph.org>
 
    This program is free software; you can redistribute it and/or modify
@@ -14,8 +12,8 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see <https://www.gnu.org/licenses/>.
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "igraph_centrality.h"
@@ -23,6 +21,7 @@
 #include "igraph_adjlist.h"
 #include "igraph_interface.h"
 #include "igraph_random.h"
+#include "igraph_structural.h"
 
 #include "centrality/prpack_internal.h"
 
@@ -36,16 +35,16 @@ static igraph_error_t igraph_i_personalized_pagerank_arpack(const igraph_t *grap
                                                  const igraph_vector_t *weights,
                                                  igraph_arpack_options_t *options);
 
-typedef struct igraph_i_pagerank_data_t {
+typedef struct {
     const igraph_t *graph;
     igraph_adjlist_t *adjlist;
     igraph_real_t damping;
     igraph_vector_t *outdegree;
     igraph_vector_t *tmp;
     igraph_vector_t *reset;
-} igraph_i_pagerank_data_t;
+} pagerank_data_t;
 
-typedef struct igraph_i_pagerank_data2_t {
+typedef struct {
     const igraph_t *graph;
     igraph_inclist_t *inclist;
     const igraph_vector_t *weights;
@@ -53,18 +52,21 @@ typedef struct igraph_i_pagerank_data2_t {
     igraph_vector_t *outdegree;
     igraph_vector_t *tmp;
     igraph_vector_t *reset;
-} igraph_i_pagerank_data2_t;
+} pagerank_data_weighted_t;
 
-static igraph_error_t igraph_i_pagerank(igraph_real_t *to, const igraph_real_t *from,
+/* The two pagerank_operator functions below update the probabilities of a random walker
+ * being in each of the vertices after one step of the walk. */
+
+static igraph_error_t pagerank_operator_unweighted(igraph_real_t *to, const igraph_real_t *from,
                              int n, void *extra) {
 
-    igraph_i_pagerank_data_t *data = extra;
+    pagerank_data_t *data = extra;
     igraph_adjlist_t *adjlist = data->adjlist;
     igraph_vector_t *outdegree = data->outdegree;
     igraph_vector_t *tmp = data->tmp;
     igraph_vector_t *reset = data->reset;
     igraph_vector_int_t *neis;
-    igraph_integer_t i, j, nlen;
+    igraph_int_t i, j, nlen;
     igraph_real_t sumfrom = 0.0;
     igraph_real_t fact = 1 - data->damping;
 
@@ -90,7 +92,7 @@ static igraph_error_t igraph_i_pagerank(igraph_real_t *to, const igraph_real_t *
         nlen = igraph_vector_int_size(neis);
         to[i] = 0.0;
         for (j = 0; j < nlen; j++) {
-            igraph_integer_t nei = VECTOR(*neis)[j];
+            igraph_int_t nei = VECTOR(*neis)[j];
             to[i] += VECTOR(*tmp)[nei];
         }
         to[i] *= data->damping;
@@ -117,17 +119,17 @@ static igraph_error_t igraph_i_pagerank(igraph_real_t *to, const igraph_real_t *
     return IGRAPH_SUCCESS;
 }
 
-static igraph_error_t igraph_i_pagerank2(igraph_real_t *to, const igraph_real_t *from,
+static igraph_error_t pagerank_operator_weighted(igraph_real_t *to, const igraph_real_t *from,
                               int n, void *extra) {
 
-    igraph_i_pagerank_data2_t *data = extra;
+    pagerank_data_weighted_t *data = extra;
     const igraph_t *graph = data->graph;
     igraph_inclist_t *inclist = data->inclist;
     const igraph_vector_t *weights = data->weights;
     igraph_vector_t *outdegree = data->outdegree;
     igraph_vector_t *tmp = data->tmp;
     igraph_vector_t *reset = data->reset;
-    igraph_integer_t i, j, nlen;
+    igraph_int_t i, j, nlen;
     igraph_real_t sumfrom = 0.0;
     igraph_vector_int_t *neis;
     igraph_real_t fact = 1 - data->damping;
@@ -139,8 +141,17 @@ static igraph_error_t igraph_i_pagerank2(igraph_real_t *to, const igraph_real_t
     */
 
     for (i = 0; i < n; i++) {
-        sumfrom += VECTOR(*outdegree)[i] != 0 ? from[i] * fact : from[i];
-        VECTOR(*tmp)[i] = from[i] / VECTOR(*outdegree)[i];
+        if (VECTOR(*outdegree)[i] > 0) {
+            sumfrom += from[i] * fact;
+            VECTOR(*tmp)[i] = from[i] / VECTOR(*outdegree)[i];
+        } else {
+            sumfrom += from[i];
+            /* The following value is used only when all outgoing edges have
+             * weight zero (as opposed to there being no outgoing edges at all).
+             * We set it to zero to avoid a 0.0*inf situation when computing
+             * to[i] below. */
+            VECTOR(*tmp)[i] = 0;
+        }
     }
 
     for (i = 0; i < n; i++) {
@@ -148,8 +159,8 @@ static igraph_error_t igraph_i_pagerank2(igraph_real_t *to, const igraph_real_t
         nlen = igraph_vector_int_size(neis);
         to[i] = 0.0;
         for (j = 0; j < nlen; j++) {
-            igraph_integer_t edge = VECTOR(*neis)[j];
-            igraph_integer_t nei = IGRAPH_OTHER(graph, edge, i);
+            igraph_int_t edge = VECTOR(*neis)[j];
+            igraph_int_t nei = IGRAPH_OTHER(graph, edge, i);
             to[i] += VECTOR(*weights)[edge] * VECTOR(*tmp)[nei];
         }
         to[i] *= data->damping;
@@ -199,10 +210,13 @@ static igraph_error_t igraph_i_pagerank2(igraph_real_t *to, const igraph_real_t
  * </para><para>
  * Starting from version 0.9, igraph has two PageRank implementations,
  * and the user can choose between them. The first implementation is
- * \c IGRAPH_PAGERANK_ALGO_ARPACK, based on the ARPACK library. This
- * was the default before igraph version 0.7. The second and recommended
+ * \c IGRAPH_PAGERANK_ALGO_ARPACK, which phrases the PageRank calculation
+ * as an eigenvalue problem, which is then solved using the ARPACK library.
+ * This was the default before igraph version 0.7. The second and recommended
  * implementation is \c IGRAPH_PAGERANK_ALGO_PRPACK. This is using the
- * PRPACK package, see https://github.com/dgleich/prpack .
+ * PRPACK package, see https://github.com/dgleich/prpack. PRPACK uses an
+ * algebraic method, i.e. solves a linear system to obtain the PageRank
+ * scores.
  *
  * </para><para>
  * Note that the PageRank of a given vertex depends on the PageRank
@@ -218,23 +232,28 @@ static igraph_error_t igraph_i_pagerank2(igraph_real_t *to, const igraph_real_t
  * Sergey Brin and Larry Page: The Anatomy of a Large-Scale Hypertextual
  * Web Search Engine. Proceedings of the 7th World-Wide Web Conference,
  * Brisbane, Australia, April 1998.
+ * https://doi.org/10.1016/S0169-7552(98)00110-X
  *
  * \param graph The graph object.
- * \param algo The PageRank implementation to use. Possible values:
- *    \c IGRAPH_PAGERANK_ALGO_ARPACK, \c IGRAPH_PAGERANK_ALGO_PRPACK.
- * \param vector Pointer to an initialized vector, the result is
- *    stored here. It is resized as needed.
- * \param value Pointer to a real variable, the eigenvalue
- *    corresponding to the PageRank vector is stored here. It should
- *    be always exactly one.
- * \param vids The vertex IDs for which the PageRank is returned.
- * \param directed Boolean, whether to consider the directedness of
- *    the edges. This is ignored for undirected graphs.
- * \param damping The damping factor ("d" in the original paper).
- *    Must be a probability in the range [0, 1]. A commonly used value is 0.85.
  * \param weights Optional edge weights. May be a \c NULL pointer,
  *    meaning unweighted edges, or a vector of non-negative values
  *    of the same length as the number of edges.
+ * \param vector Pointer to an initialized vector, the result is
+ *    stored here. It is resized as needed.
+ * \param value Pointer to a real variable. When using \c IGRAPH_PAGERANK_ALGO_ARPACK,
+ *    the eigenvalue corresponding to the PageRank vector is stored here. It is
+ *    expected to be exactly one. Checking this value can be used to diagnose cases
+ *    when ARPACK failed to converge to the leading eigenvector.
+ *    When using \c IGRAPH_PAGERANK_ALGO_PRPACK, this is always set to 1.0.
+ * \param damping The damping factor ("d" in the original paper).
+ *    Must be a probability in the range [0, 1]. A commonly used value is 0.85.
+ * \param directed Boolean, whether to consider the directedness of
+ *    the edges. This is ignored for undirected graphs.
+ * \param vids The vertex IDs for which the PageRank is returned. This parameter
+ *    is only for convenience. Computing PageRank for fewer than all vertices will
+ *    not speed up the calculation.
+ * \param algo The PageRank implementation to use. Possible values:
+ *    \c IGRAPH_PAGERANK_ALGO_ARPACK, \c IGRAPH_PAGERANK_ALGO_PRPACK.
  * \param options Options for the ARPACK method. See \ref igraph_arpack_options_t
  *    for details. Supply \c NULL here to use the defaults. Note that the function
  *    overwrites the <code>n</code> (number of vertices), <code>nev</code> (1),
@@ -255,15 +274,21 @@ static igraph_error_t igraph_i_pagerank2(igraph_real_t *to, const igraph_real_t
  *
  * \example examples/simple/igraph_pagerank.c
  */
-
-igraph_error_t igraph_pagerank(const igraph_t *graph, igraph_pagerank_algo_t algo,
-                    igraph_vector_t *vector,
-                    igraph_real_t *value, const igraph_vs_t vids,
-                    igraph_bool_t directed, igraph_real_t damping,
-                    const igraph_vector_t *weights, igraph_arpack_options_t *options) {
-    return igraph_personalized_pagerank(graph, algo, vector, value, vids,
-                                        directed, damping, NULL, weights,
-                                        options);
+igraph_error_t igraph_pagerank(
+        const igraph_t *graph, const igraph_vector_t *weights,
+        igraph_vector_t *vector, igraph_real_t *value,
+        igraph_real_t damping, igraph_bool_t directed,
+        igraph_vs_t vids,
+        igraph_pagerank_algo_t algo,
+        igraph_arpack_options_t *options) {
+    return igraph_personalized_pagerank(
+            graph, weights,
+            vector, value,
+            NULL,
+            damping, directed,
+            vids,
+            algo,
+            options);
 }
 
 /**
@@ -281,7 +306,7 @@ igraph_error_t igraph_pagerank(const igraph_t *graph, igraph_pagerank_algo_t alg
  * This simplified interface takes a vertex sequence and resets the random walk to
  * one of the vertices in the specified vertex sequence, chosen uniformly. A typical
  * application of personalized PageRank is when the random walk is reset to the same
- * vertex every time - this can easily be achieved using \ref igraph_vss_1() which
+ * vertex every time: this can easily be achieved using \ref igraph_vss_1() which
  * generates a vertex sequence containing only a single vertex.
  *
  * </para><para>
@@ -290,26 +315,30 @@ igraph_error_t igraph_pagerank(const igraph_t *graph, igraph_pagerank_algo_t alg
  * the personalized PageRank for only some of the vertices, all of them must be
  * calculated. Requesting the personalized PageRank for only some of the vertices
  * does not result in any performance increase at all.
- * </para>
  *
- * <para>
  * \param graph The graph object.
- * \param algo The PageRank implementation to use. Possible values:
- *    \c IGRAPH_PAGERANK_ALGO_ARPACK, \c IGRAPH_PAGERANK_ALGO_PRPACK.
- * \param vector Pointer to an initialized vector, the result is
- *    stored here. It is resized as needed.
- * \param value Pointer to a real variable, the eigenvalue
- *    corresponding to the PageRank vector is stored here. It should
- *    be always exactly one.
- * \param vids The vertex IDs for which the PageRank is returned.
- * \param directed Boolean, whether to consider the directedness of
- *    the edges. This is ignored for undirected graphs.
- * \param damping The damping factor ("d" in the original paper).
- *    Must be a probability in the range [0, 1]. A commonly used value is 0.85.
- * \param reset_vids IDs of the vertices used when resetting the random walk.
  * \param weights Optional edge weights, it is either a null pointer,
  *    then the edges are not weighted, or a vector of the same length
  *    as the number of edges.
+ * \param vector Pointer to an initialized vector, the result is
+ *    stored here. It is resized as needed.
+ * \param value Pointer to a real variable. When using \c IGRAPH_PAGERANK_ALGO_ARPACK,
+ *    the eigenvalue corresponding to the PageRank vector is stored here. It is
+ *    expected to be exactly one. Checking this value can be used to diagnose cases
+ *    when ARPACK failed to converge to the leading eigenvector.
+ *    When using \c IGRAPH_PAGERANK_ALGO_PRPACK, this is always set to 1.0.
+ * \param reset_vids IDs of the vertices used when resetting the random walk.
+ *    The walk will be restarted from a vertex in this set, chosen uniformly at
+ *    random. Duplicate vertices are allowed.
+ * \param damping The damping factor ("d" in the original paper).
+ *    Must be a probability in the range [0, 1]. A commonly used value is 0.85.
+ * \param directed Boolean, whether to consider the directedness of
+ *    the edges. This is ignored for undirected graphs.
+ * \param vids The vertex IDs for which the PageRank is returned. This parameter
+ *    is only for convenience. Computing PageRank for fewer than all vertices will
+ *    not speed up the calculation.
+ * \param algo The PageRank implementation to use. Possible values:
+ *    \c IGRAPH_PAGERANK_ALGO_ARPACK, \c IGRAPH_PAGERANK_ALGO_PRPACK.
  * \param options Options for the ARPACK method. See \ref igraph_arpack_options_t
  *    for details. Supply \c NULL here to use the defaults. Note that the function
  *    overwrites the <code>n</code> (number of vertices), <code>nev</code> (1),
@@ -328,14 +357,15 @@ igraph_error_t igraph_pagerank(const igraph_t *graph, igraph_pagerank_algo_t alg
  *
  * \sa \ref igraph_pagerank() for the non-personalized implementation.
  */
+igraph_error_t igraph_personalized_pagerank_vs(
+        const igraph_t *graph, const igraph_vector_t *weights,
+        igraph_vector_t *vector, igraph_real_t *value,
+        igraph_vs_t reset_vids,
+        igraph_real_t damping, igraph_bool_t directed,
+        igraph_vs_t vids,
+        igraph_pagerank_algo_t algo,
+        igraph_arpack_options_t *options) {
 
-igraph_error_t igraph_personalized_pagerank_vs(const igraph_t *graph,
-                                    igraph_pagerank_algo_t algo, igraph_vector_t *vector,
-                                    igraph_real_t *value, const igraph_vs_t vids,
-                                    igraph_bool_t directed, igraph_real_t damping,
-                                    igraph_vs_t reset_vids,
-                                    const igraph_vector_t *weights,
-                                    igraph_arpack_options_t *options) {
     igraph_vector_t reset;
     igraph_vit_t vit;
 
@@ -344,16 +374,18 @@ igraph_error_t igraph_personalized_pagerank_vs(const igraph_t *graph,
     IGRAPH_FINALLY(igraph_vit_destroy, &vit);
 
     while (!IGRAPH_VIT_END(vit)) {
-        VECTOR(reset)[IGRAPH_VIT_GET(vit)]++;
+        /* Increment by 1 instead of setting to 1 to handle duplicates. */
+        VECTOR(reset)[IGRAPH_VIT_GET(vit)] += 1.0;
         IGRAPH_VIT_NEXT(vit);
     }
     igraph_vit_destroy(&vit);
     IGRAPH_FINALLY_CLEAN(1);
 
-    IGRAPH_CHECK(igraph_personalized_pagerank(graph, algo, vector,
-                 value, vids, directed,
-                 damping, &reset, weights,
-                 options));
+    IGRAPH_CHECK(igraph_personalized_pagerank(
+            graph, weights, vector,
+            value, &reset,
+            damping, directed, vids, algo,
+            options));
 
     igraph_vector_destroy(&reset);
     IGRAPH_FINALLY_CLEAN(1);
@@ -379,29 +411,31 @@ igraph_error_t igraph_personalized_pagerank_vs(const igraph_t *graph,
  * the personalized PageRank for only some of the vertices, all of them must be
  * calculated. Requesting the personalized PageRank for only some of the vertices
  * does not result in any performance increase at all.
- * </para>
  *
- * <para>
  * \param graph The graph object.
- * \param algo The PageRank implementation to use. Possible values:
- *    \c IGRAPH_PAGERANK_ALGO_ARPACK, \c IGRAPH_PAGERANK_ALGO_PRPACK.
+ * \param weights Optional edge weights. May be a \c NULL pointer,
+ *    meaning unweighted edges, or a vector of non-negative values
+ *    of the same length as the number of edges.
  * \param vector Pointer to an initialized vector, the result is
  *    stored here. It is resized as needed.
- * \param value Pointer to a real variable, the eigenvalue
- *    corresponding to the PageRank vector is stored here. It should
- *    be always exactly one.
- * \param vids The vertex IDs for which the PageRank is returned.
- * \param directed Boolean, whether to consider the directedness of
- *    the edges. This is ignored for undirected graphs.
- * \param damping The damping factor ("d" in the original paper).
- *    Must be a probability in the range [0, 1]. A commonly used value is 0.85.
+ * \param value Pointer to a real variable. When using \c IGRAPH_PAGERANK_ALGO_ARPACK,
+ *    the eigenvalue corresponding to the PageRank vector is stored here. It is
+ *    expected to be exactly one. Checking this value can be used to diagnose cases
+ *    when ARPACK failed to converge to the leading eigenvector.
+ *    When using \c IGRAPH_PAGERANK_ALGO_PRPACK, this is always set to 1.0.
  * \param reset The probability distribution over the vertices used when
  *    resetting the random walk. It is either a \c NULL pointer (denoting
  *    a uniform choice that results in the original PageRank measure)
  *    or a vector of the same length as the number of vertices.
- * \param weights Optional edge weights. May be a \c NULL pointer,
- *    meaning unweighted edges, or a vector of non-negative values
- *    of the same length as the number of edges.
+ * \param damping The damping factor ("d" in the original paper).
+ *    Must be a probability in the range [0, 1]. A commonly used value is 0.85.
+ * \param directed Boolean, whether to consider the directedness of
+ *    the edges. This is ignored for undirected graphs.
+ * \param vids The vertex IDs for which the PageRank is returned. This parameter
+ *    is only for convenience. Computing PageRank for fewer than all vertices will
+ *    not speed up the calculation.
+ * \param algo The PageRank implementation to use. Possible values:
+ *    \c IGRAPH_PAGERANK_ALGO_ARPACK, \c IGRAPH_PAGERANK_ALGO_PRPACK.
  * \param options Options for the ARPACK method. See \ref igraph_arpack_options_t
  *    for details. Supply \c NULL here to use the defaults. Note that the function
  *    overwrites the <code>n</code> (number of vertices), <code>nev</code> (1),
@@ -421,13 +455,14 @@ igraph_error_t igraph_personalized_pagerank_vs(const igraph_t *graph,
  * \ref igraph_personalized_pagerank_vs() for a personalized implementation
  * with resetting to specific vertices.
  */
-igraph_error_t igraph_personalized_pagerank(const igraph_t *graph,
-                                 igraph_pagerank_algo_t algo, igraph_vector_t *vector,
-                                 igraph_real_t *value, const igraph_vs_t vids,
-                                 igraph_bool_t directed, igraph_real_t damping,
-                                 const igraph_vector_t *reset,
-                                 const igraph_vector_t *weights,
-                                 igraph_arpack_options_t *options) {
+igraph_error_t igraph_personalized_pagerank(
+        const igraph_t *graph, const igraph_vector_t *weights,
+        igraph_vector_t *vector, igraph_real_t *value,
+        const igraph_vector_t *reset,
+        igraph_real_t damping, igraph_bool_t directed,
+        igraph_vs_t vids,
+        igraph_pagerank_algo_t algo,
+        igraph_arpack_options_t *options) {
 
     if (damping < 0.0 || damping > 1.0) {
         IGRAPH_ERROR("The PageRank damping factor must be in the range [0,1].", IGRAPH_EINVAL);
@@ -466,31 +501,50 @@ static igraph_error_t igraph_i_personalized_pagerank_arpack(const igraph_t *grap
     igraph_vector_t tmp;
     igraph_vector_t normalized_reset;
 
-    igraph_integer_t i;
-    igraph_integer_t no_of_nodes = igraph_vcount(graph);
-    igraph_integer_t no_of_edges = igraph_ecount(graph);
+    igraph_int_t i;
+    igraph_int_t no_of_nodes = igraph_vcount(graph);
+    igraph_int_t no_of_edges = igraph_ecount(graph);
+
+    igraph_real_t reset_sum; /* used only when reset != NULL */
 
     if (no_of_nodes > INT_MAX) {
         IGRAPH_ERROR("Graph has too many vertices for ARPACK.", IGRAPH_EOVERFLOW);
     }
 
+    if (weights && igraph_vector_size(weights) != no_of_edges) {
+        IGRAPH_ERROR("Invalid length of weights vector when calculating PageRank scores.", IGRAPH_EINVAL);
+    }
+
     if (reset && igraph_vector_size(reset) != no_of_nodes) {
         IGRAPH_ERROR("Invalid length of reset vector when calculating personalized PageRank scores.", IGRAPH_EINVAL);
     }
 
+    if (reset) {
+        reset_sum = igraph_vector_sum(reset);
+        if (no_of_nodes > 0 && reset_sum == 0) {
+            IGRAPH_ERROR("The sum of the elements in the reset vector must not be zero.", IGRAPH_EINVAL);
+        }
+
+        igraph_real_t reset_min = igraph_vector_min(reset);
+        if (reset_min < 0) {
+            IGRAPH_ERROR("The reset vector must not contain negative elements.", IGRAPH_EINVAL);
+        }
+        if (!isfinite(reset_sum)) {
+            IGRAPH_ERROR("The reset vector must not contain infinite or NaN values.", IGRAPH_EINVAL);
+        }
+    }
+
     if (no_of_edges == 0) {
         /* Special case: graph with no edges. Result is the same as the personalization vector. */
         if (value) {
             *value = 1.0;
         }
         if (vector) {
-            IGRAPH_CHECK(igraph_vector_resize(vector, no_of_nodes));
             if (reset && no_of_nodes > 0) {
-                for (i=0; i < no_of_nodes; ++i) {
-                    VECTOR(*vector)[i] = VECTOR(*reset)[i];
-                }
-                igraph_vector_scale(vector, 1.0 / igraph_vector_sum(vector));
+                IGRAPH_CHECK(igraph_vector_update(vector, reset));
+                igraph_vector_scale(vector, 1.0 / reset_sum);
             } else {
+                IGRAPH_CHECK(igraph_vector_resize(vector, no_of_nodes));
                 igraph_vector_fill(vector, 1.0 / no_of_nodes);
             }
         }
@@ -508,12 +562,11 @@ static igraph_error_t igraph_i_personalized_pagerank_arpack(const igraph_t *grap
     if (weights) {
         igraph_real_t min, max;
 
-        if (igraph_vector_size(weights) != no_of_edges) {
-            IGRAPH_ERROR("Invalid length of weights vector when calculating PageRank scores.", IGRAPH_EINVAL);
-        }
-
         /* Safe to call minmax, ecount == 0 case was caught earlier */
         igraph_vector_minmax(weights, &min, &max);
+        if (min < 0) {
+            IGRAPH_ERROR("Edge weights must not be negative.", IGRAPH_EINVAL);
+        }
         if (isnan(min)) {
             IGRAPH_ERROR("Weight vector must not contain NaN values.", IGRAPH_EINVAL);
         }
@@ -550,34 +603,34 @@ static igraph_error_t igraph_i_personalized_pagerank_arpack(const igraph_t *grap
     IGRAPH_VECTOR_INIT_FINALLY(&outdegree, options->n);
     IGRAPH_VECTOR_INIT_FINALLY(&tmp, options->n);
 
-    RNG_BEGIN();
-
     if (reset) {
         /* Normalize reset vector so the sum is 1 */
-        igraph_real_t reset_sum, reset_min;
-        reset_min = igraph_vector_min(reset);
-        if (reset_min < 0) {
-            IGRAPH_ERROR("The reset vector must not contain negative elements.", IGRAPH_EINVAL);
-        }
-        if (isnan(reset_min)) {
-            IGRAPH_ERROR("The reset vector must not contain NaN values.", IGRAPH_EINVAL);
-        }
-        reset_sum = igraph_vector_sum(reset);
-        if (reset_sum == 0) {
-            IGRAPH_ERROR("The sum of the elements in the reset vector must not be zero.", IGRAPH_EINVAL);
-        }
-
         IGRAPH_CHECK(igraph_vector_init_copy(&normalized_reset, reset));
         IGRAPH_FINALLY(igraph_vector_destroy, &normalized_reset);
 
         igraph_vector_scale(&normalized_reset, 1.0 / reset_sum);
     }
 
+    IGRAPH_CHECK(igraph_strength(graph, &outdegree, igraph_vss_all(),
+                                 directed ? IGRAPH_OUT : IGRAPH_ALL, IGRAPH_LOOPS, weights));
+    IGRAPH_CHECK(igraph_strength(graph, &indegree, igraph_vss_all(),
+                                 directed ? IGRAPH_IN : IGRAPH_ALL, IGRAPH_LOOPS, weights));
+
+    /* Set up an appropriate starting vector. We start from the (possibly weighted)
+     * in-degrees plus some small random noise to avoid convergence problems. */
+    for (i = 0; i < no_of_nodes; i++) {
+        if (VECTOR(indegree)[i] > 0) {
+            /* Note: Keep random perturbation non-negative. */
+            MATRIX(vectors, i, 0) = VECTOR(indegree)[i] + RNG_UNIF(0, 1e-4);
+        } else {
+            MATRIX(vectors, i, 0) = 1;
+        }
+    }
+
     if (!weights) {
 
         igraph_adjlist_t adjlist;
-        igraph_i_pagerank_data_t data;
-        igraph_vector_int_t degree;
+        pagerank_data_t data;
 
         data.graph = graph;
         data.adjlist = &adjlist;
@@ -586,40 +639,11 @@ static igraph_error_t igraph_i_personalized_pagerank_arpack(const igraph_t *grap
         data.tmp = &tmp;
         data.reset = reset ? &normalized_reset : NULL;
 
-        IGRAPH_VECTOR_INT_INIT_FINALLY(&degree, 0);
-
-        IGRAPH_CHECK(igraph_degree(graph, &degree, igraph_vss_all(),
-                                   directed ? IGRAPH_OUT : IGRAPH_ALL, IGRAPH_LOOPS));
-        for (i = 0; i < no_of_nodes; i++) {
-            VECTOR(outdegree)[i] = VECTOR(degree)[i];
-        }
-
-        IGRAPH_CHECK(igraph_degree(graph, &degree, igraph_vss_all(),
-                                   directed ? IGRAPH_IN : IGRAPH_ALL, IGRAPH_LOOPS));
-        for (i = 0; i < no_of_nodes; i++) {
-            VECTOR(indegree)[i] = VECTOR(degree)[i];
-        }
-
-        igraph_vector_int_destroy(&degree);
-        IGRAPH_FINALLY_CLEAN(1);
-
-        /* Set up an appropriate starting vector. We start from the in-degrees
-         * plus some small random noise to avoid convergence problems */
-        for (i = 0; i < no_of_nodes; i++) {
-            if (VECTOR(indegree)[i]) {
-                MATRIX(vectors, i, 0) = VECTOR(indegree)[i] + RNG_UNIF(-1e-4, 1e-4);
-            } else {
-                MATRIX(vectors, i, 0) = 1;
-            }
-        }
-
-        IGRAPH_CHECK(igraph_adjlist_init(
-            graph, &adjlist, dirmode, IGRAPH_LOOPS, IGRAPH_MULTIPLE
-        ));
+        IGRAPH_CHECK(igraph_adjlist_init(graph, &adjlist, dirmode, IGRAPH_LOOPS, IGRAPH_MULTIPLE));
         IGRAPH_FINALLY(igraph_adjlist_destroy, &adjlist);
 
-        IGRAPH_CHECK(igraph_arpack_rnsolve(igraph_i_pagerank,
-                                           &data, options, 0, &values, &vectors));
+        IGRAPH_CHECK(igraph_arpack_rnsolve(pagerank_operator_unweighted,
+                                           &data, options, NULL, &values, &vectors));
 
         igraph_adjlist_destroy(&adjlist);
         IGRAPH_FINALLY_CLEAN(1);
@@ -627,8 +651,7 @@ static igraph_error_t igraph_i_personalized_pagerank_arpack(const igraph_t *grap
     } else {
 
         igraph_inclist_t inclist;
-        igraph_bool_t negative_weight_warned = false;
-        igraph_i_pagerank_data2_t data;
+        pagerank_data_weighted_t data;
 
         data.graph = graph;
         data.inclist = &inclist;
@@ -641,42 +664,13 @@ static igraph_error_t igraph_i_personalized_pagerank_arpack(const igraph_t *grap
         IGRAPH_CHECK(igraph_inclist_init(graph, &inclist, dirmode, IGRAPH_LOOPS));
         IGRAPH_FINALLY(igraph_inclist_destroy, &inclist);
 
-        /* Weighted degree */
-        for (i = 0; i < no_of_edges; i++) {
-            igraph_integer_t from = IGRAPH_FROM(graph, i);
-            igraph_integer_t to = IGRAPH_TO(graph, i);
-            igraph_real_t weight = VECTOR(*weights)[i];
-            if (weight < 0 && !negative_weight_warned) {
-                IGRAPH_WARNING("Replacing negative weights with zeros during PageRank calculation.");
-                weight = 0;
-                negative_weight_warned = 1;
-            }
-            VECTOR(outdegree)[from] += weight;
-            VECTOR(indegree) [to]   += weight;
-            if (!directed) {
-                VECTOR(outdegree)[to]   += weight;
-                VECTOR(indegree) [from] += weight;
-            }
-        }
-        /* Set up an appropriate starting vector. We start from the in-degrees
-         * plus some small random noise to avoid convergence problems */
-        for (i = 0; i < no_of_nodes; i++) {
-            if (VECTOR(indegree)[i]) {
-                MATRIX(vectors, i, 0) = VECTOR(indegree)[i] + RNG_UNIF(-1e-4, 1e-4);
-            } else {
-                MATRIX(vectors, i, 0) = 1;
-            }
-        }
-
-        IGRAPH_CHECK(igraph_arpack_rnsolve(igraph_i_pagerank2,
-                                           &data, options, 0, &values, &vectors));
+        IGRAPH_CHECK(igraph_arpack_rnsolve(pagerank_operator_weighted,
+                                           &data, options, NULL, &values, &vectors));
 
         igraph_inclist_destroy(&inclist);
         IGRAPH_FINALLY_CLEAN(1);
     }
 
-    RNG_END();
-
     if (reset) {
         igraph_vector_destroy(&normalized_reset);
         IGRAPH_FINALLY_CLEAN(1);
@@ -693,7 +687,7 @@ static igraph_error_t igraph_i_personalized_pagerank_arpack(const igraph_t *grap
 
     if (vector) {
         igraph_vit_t vit;
-        igraph_integer_t nodes_to_calc;
+        igraph_int_t nodes_to_calc;
         igraph_real_t sum = 0;
 
         for (i = 0; i < no_of_nodes; i++) {
