@@ -1,8 +1,6 @@
-/* -*- mode: C -*-  */
-/* vim:set ts=4 sw=4 sts=4 et: */
 /*
-   IGraph library.
-   Copyright (C) 2007-2020 The igraph development team
+   igraph library.
+   Copyright (C) 2007-2024  The igraph development team <igraph@igraph.org>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,76 +13,105 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301 USA
-
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "igraph_community.h"
 #include "igraph_memory.h"
 #include "igraph_sparsemat.h"
 
+#include "community/community_internal.h"
+
 #include <string.h>
 #include <math.h>
 
 /**
- * \function igraph_community_to_membership
- * \brief Creates a membership vector from a community structure dendrogram.
+ * \section about_community
  *
- * This function creates a membership vector from a community
- * structure dendrogram. A membership vector contains for each vertex
- * the id of its graph component, the graph components are numbered
- * from zero, see the same argument of \ref igraph_connected_components()
- * for an example of a membership vector.
+ * <para>
+ * Community detection is concerned with clustering the vertices of networks
+ * into tightly connected subgraphs called "communities". The following
+ * references provide a good introduction to the topic of community detection:
+ * </para>
+ *
+ * <para>
+ * S. Fortunato:
+ * "Community Detection in Graphs".
+ * Physics Reports 486, no. 3–5 (2010): 75–174.
+ * https://doi.org/10.1016/j.physrep.2009.11.002.
+ * </para>
+ *
+ * <para>
+ * S. Fortunato and D. Hric:
+ * "Community Detection in Networks: A User Guide".
+ * Physics Reports 659 (2016): 1–44.
+ * https://doi.org/10.1016/j.physrep.2016.09.002.
+ * </para>
+ */
+
+/**
+ * \function igraph_community_to_membership
+ * \brief Cut a dendrogram after a given number of merges.
+ *
+ * This function creates a membership vector from a dendrogram whose leaves
+ * are individual vertices by cutting it at the specified level. It produces
+ * a membership vector that contains for each vertex its cluster ID, numbered
+ * from zero. This is the same membership vector format that is produced by
+ * \ref igraph_connected_components(), as well as all community detection
+ * functions in igraph.
  *
  * </para><para>
- * Many community detection algorithms return with a \em merges
- * matrix, \ref igraph_community_walktrap() and \ref
- * igraph_community_edge_betweenness() are two examples. The matrix
- * contains the merge operations performed while mapping the
- * hierarchical structure of a network. If the matrix has \c n-1 rows,
- * where \c n is the number of vertices in the graph, then it contains
- * the hierarchical structure of the whole network and it is called a
- * dendrogram.
+ * It takes as input the number of vertices \p n, and a \p merges matrix
+ * encoding the dendrogram, in the format produced by hierarchical clustering
+ * functions such as \ref igraph_community_edge_betweenness(),
+ * \ref igraph_community_walktrap() or \ref igraph_community_fastgreedy().
+ * The matrix must have two columns and up to <code>n - 1</code> rows.
+ * Each row represents merging two dendrogram nodes into their parent node.
+ * The leaf nodes of the dendrogram are indexed from 0 to <code>n - 1</code>
+ * and are identical to the vertices of the graph that is being partitioned
+ * into communities. Row \c i contains the children of dendrogram node
+ * with index <code>n + i</code>.
  *
  * </para><para>
  * This function performs \p steps merge operations as prescribed by
- * the \p merges matrix and returns the current state of the network.
+ * the \p merges matrix and returns the resulting partitioning into
+ * <code>n - steps</code> communities.
  *
  * </para><para>
  * If \p merges is not a complete dendrogram, it is possible to
  * take \p steps steps if \p steps is not bigger than the number
  * lines in \p merges.
- * \param merges The two-column matrix containing the merge
- *    operations. See \ref igraph_community_walktrap() for the
- *    detailed syntax.
+ *
+ * \param merges The two-column matrix containing the merge operations.
  * \param nodes The number of leaf nodes in the dendrogram.
  * \param steps Integer constant, the number of steps to take.
  * \param membership Pointer to an initialized vector, the membership
- *    results will be stored here, if not NULL. The vector will be
+ *    results will be stored here, if not \c NULL. The vector will be
  *    resized as needed.
- * \param csize Pointer to an initialized vector, or NULL. If not NULL
+ * \param csize Pointer to an initialized vector, or \c NULL. If not \c NULL
  *    then the sizes of the components will be stored here, the vector
  *    will be resized as needed.
+ * \return Error code.
  *
  * \sa \ref igraph_community_walktrap(), \ref
  * igraph_community_edge_betweenness(), \ref
  * igraph_community_fastgreedy() for community structure detection
- * algorithms.
+ * algorithms producing merge matrices in this format;
+ * \ref igraph_le_community_to_membership() to perform merges
+ * starting from a given cluster assignment.
  *
  * Time complexity: O(|V|), the number of vertices in the graph.
  */
 igraph_error_t igraph_community_to_membership(const igraph_matrix_int_t *merges,
-                                   igraph_integer_t nodes,
-                                   igraph_integer_t steps,
+                                   igraph_int_t nodes,
+                                   igraph_int_t steps,
                                    igraph_vector_int_t *membership,
                                    igraph_vector_int_t *csize) {
 
-    igraph_integer_t no_of_nodes = nodes;
-    igraph_integer_t components = no_of_nodes - steps;
-    igraph_integer_t i, found = 0;
-    igraph_vector_t tmp;
+    const igraph_int_t no_of_nodes = nodes;
+    const igraph_int_t components = no_of_nodes - steps;
+    igraph_int_t found = 0;
+    igraph_vector_int_t tmp;
     igraph_vector_bool_t already_merged;
     igraph_vector_int_t own_membership;
     igraph_bool_t using_own_membership = false;
@@ -102,11 +129,11 @@ igraph_error_t igraph_community_to_membership(const igraph_matrix_int_t *merges,
         IGRAPH_ERRORF("Number of steps should be non-negative, found %" IGRAPH_PRId ".", IGRAPH_EINVAL, steps);
     }
 
-    if (csize != 0 && membership == 0) {
+    if (csize != NULL && membership == NULL) {
         /* we need a membership vector to calculate 'csize' but the user did
          * not provide one; let's allocate one ourselves */
         IGRAPH_VECTOR_INT_INIT_FINALLY(&own_membership, no_of_nodes);
-        using_own_membership = 1;
+        using_own_membership = true;
         membership = &own_membership;
     }
 
@@ -120,19 +147,19 @@ igraph_error_t igraph_community_to_membership(const igraph_matrix_int_t *merges,
     }
 
     IGRAPH_VECTOR_BOOL_INIT_FINALLY(&already_merged, steps + no_of_nodes);
-    IGRAPH_VECTOR_INIT_FINALLY(&tmp, steps);
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&tmp, steps);
 
-    for (i = steps - 1; i >= 0; i--) {
-        igraph_integer_t c1 = MATRIX(*merges, i, 0);
-        igraph_integer_t c2 = MATRIX(*merges, i, 1);
+    for (igraph_int_t i = steps - 1; i >= 0; i--) {
+        const igraph_int_t c1 = MATRIX(*merges, i, 0);
+        const igraph_int_t c2 = MATRIX(*merges, i, 1);
 
         if (VECTOR(already_merged)[c1] == 0) {
-            VECTOR(already_merged)[c1] = 1;
+            VECTOR(already_merged)[c1] = true;
         } else {
             IGRAPH_ERRORF("Merges matrix contains multiple merges of cluster %" IGRAPH_PRId ".", IGRAPH_EINVAL, c1);
         }
         if (VECTOR(already_merged)[c2] == 0) {
-            VECTOR(already_merged)[c2] = 1;
+            VECTOR(already_merged)[c2] = true;
         } else {
             IGRAPH_ERRORF("Merges matrix contains multiple merges of cluster %" IGRAPH_PRId ".", IGRAPH_EINVAL, c2);
         }
@@ -144,7 +171,7 @@ igraph_error_t igraph_community_to_membership(const igraph_matrix_int_t *merges,
         }
 
         if (c1 < no_of_nodes) {
-            igraph_integer_t cid = VECTOR(tmp)[i] - 1;
+            const igraph_int_t cid = VECTOR(tmp)[i] - 1;
             if (membership) {
                 VECTOR(*membership)[c1] = cid + 1;
             }
@@ -156,7 +183,7 @@ igraph_error_t igraph_community_to_membership(const igraph_matrix_int_t *merges,
         }
 
         if (c2 < no_of_nodes) {
-            igraph_integer_t cid = VECTOR(tmp)[i] - 1;
+            const igraph_int_t cid = VECTOR(tmp)[i] - 1;
             if (membership) {
                 VECTOR(*membership)[c2] = cid + 1;
             }
@@ -170,27 +197,24 @@ igraph_error_t igraph_community_to_membership(const igraph_matrix_int_t *merges,
     }
 
     if (membership || csize) {
-        /* it can never happen that csize != 0 and membership == 0; we have
-         * handled that case above */
-        for (i = 0; i < no_of_nodes; i++) {
-            igraph_integer_t tmp = VECTOR(*membership)[i];
-            if (tmp != 0) {
-                if (membership) {
-                    VECTOR(*membership)[i] = tmp - 1;
-                }
+        /* It can never happen that csize != NULL and membership == NULL; we have
+         * handled that case above. Thus we skip if (membership) checks, which
+         * would confuse static analysers in this context. */
+        for (igraph_int_t i = 0; i < no_of_nodes; i++) {
+            const igraph_int_t c = VECTOR(*membership)[i];
+            if (c != 0) {
+                VECTOR(*membership)[i] = c - 1;
             } else {
                 if (csize) {
                     VECTOR(*csize)[found] += 1;
                 }
-                if (membership) {
-                    VECTOR(*membership)[i] = found;
-                }
+                VECTOR(*membership)[i] = found;
                 found++;
             }
         }
     }
 
-    igraph_vector_destroy(&tmp);
+    igraph_vector_int_destroy(&tmp);
     igraph_vector_bool_destroy(&already_merged);
     IGRAPH_FINALLY_CLEAN(2);
 
@@ -202,15 +226,86 @@ igraph_error_t igraph_community_to_membership(const igraph_matrix_int_t *merges,
     return IGRAPH_SUCCESS;
 }
 
+
+/**
+ * This slower implementation of igraph_reindex_membership() is used when
+ * cluster indices are not within the range 0..vcount-1.
+ *
+ * \param membership Vector encoding the membership of each vertex.
+ *    Elements may be arbitrary integers, however, its length must not be zero.
+ * \param new_to_old If not \c NULL, a mapping from new to old cluster
+ *    indices will be stored here.
+ * \param nb_clusters If not \c NULL, the number of clusters will be stored here.
+ *
+ * Time complexity: O(n log(n)) where n is the membership vector length.
+ */
+igraph_error_t igraph_i_reindex_membership_large(
+        igraph_vector_int_t *membership,
+        igraph_vector_int_t *new_to_old,
+        igraph_int_t *nb_clusters) {
+
+    const igraph_int_t vcount = igraph_vector_int_size(membership);
+    igraph_vector_int_t permutation;
+
+    /* The vcount == 0 case requires special handling because the main
+     * algorithm of this function cannot handle it. */
+    if (vcount == 0) {
+        if (new_to_old) {
+            igraph_vector_int_clear(new_to_old);
+        }
+        if (nb_clusters) {
+            *nb_clusters = 0;
+        }
+        return IGRAPH_SUCCESS;
+    }
+
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&permutation, vcount);
+
+    IGRAPH_CHECK(igraph_vector_int_sort_ind(membership, &permutation, IGRAPH_ASCENDING));
+
+    if (new_to_old) {
+        igraph_vector_int_clear(new_to_old);
+    }
+
+    igraph_int_t j = VECTOR(permutation)[0];
+    igraph_int_t c_old = VECTOR(*membership)[j];
+    igraph_int_t c_old_prev = c_old;
+    igraph_int_t c_new = 0;
+    if (new_to_old) {
+        IGRAPH_CHECK(igraph_vector_int_push_back(new_to_old, c_old));
+    }
+    VECTOR(*membership)[j] = c_new;
+
+    for (igraph_int_t i=1; i < vcount; i++) {
+        j = VECTOR(permutation)[i];
+        c_old = VECTOR(*membership)[j];
+        if (c_old != c_old_prev) {
+            if (new_to_old) {
+                IGRAPH_CHECK(igraph_vector_int_push_back(new_to_old, c_old));
+            }
+            c_new++;
+        }
+        c_old_prev = c_old;
+        VECTOR(*membership)[j] = c_new;
+    }
+
+    if (nb_clusters) {
+        *nb_clusters = c_new+1;
+    }
+
+    igraph_vector_int_destroy(&permutation);
+    IGRAPH_FINALLY_CLEAN(1);
+
+    return IGRAPH_SUCCESS;
+}
+
 /**
  * \function igraph_reindex_membership
  * \brief Makes the IDs in a membership vector contiguous.
  *
- * This function reindexes component IDs in a membership vector
- * in a way that the new IDs start from zero and go up to C-1,
- * where C is the number of unique component IDs in the original
- * vector. The supplied membership is expected to fall in the
- * range 0, ..., n - 1.
+ * This function reindexes component IDs in a membership vector in a way that
+ * the new IDs start from zero and go up to <code>C-1</code>, where \c C is the
+ * number of unique component IDs in the original vector.
  *
  * \param  membership  Numeric vector which gives the type of each
  *                     vertex, i.e. the component to which it belongs.
@@ -223,20 +318,25 @@ igraph_error_t igraph_community_to_membership(const igraph_matrix_int_t *merges,
  *                     distinct clusters. If not \c NULL, this will be
  *                     updated to reflect the number of distinct
  *                     clusters found in membership.
+ * \return Error code.
  *
- * Time complexity: should be O(n) for n elements.
+ * Time complexity: Let n be the length of the membership vector.
+ * O(n) if cluster indices are within 0..n-1, and O(n log(n)) otherwise.
  */
-igraph_error_t igraph_reindex_membership(igraph_vector_int_t *membership,
-                              igraph_vector_int_t *new_to_old,
-                              igraph_integer_t *nb_clusters) {
+igraph_error_t igraph_reindex_membership(
+        igraph_vector_int_t *membership,
+        igraph_vector_int_t *new_to_old,
+        igraph_int_t *nb_clusters) {
 
-    igraph_integer_t i, n = igraph_vector_int_size(membership);
-    igraph_vector_t new_cluster;
-    igraph_integer_t i_nb_clusters;
+    const igraph_int_t vcount = igraph_vector_int_size(membership);
+    igraph_vector_int_t new_cluster;
+    igraph_int_t i_nb_clusters;
 
-    /* We allow original cluster indices in the range 0, ..., n - 1 */
-    IGRAPH_CHECK(igraph_vector_init(&new_cluster, n));
-    IGRAPH_FINALLY(igraph_vector_destroy, &new_cluster);
+    /* First, we assume cluster indices in the range 0..vcount-1,
+     * and attempt to use a performant implementation. If indices
+     * outside of this range are found, we will fall back to a
+     * slower alternative implementation. */
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&new_cluster, vcount);
 
     if (new_to_old) {
         igraph_vector_int_clear(new_to_old);
@@ -245,26 +345,19 @@ igraph_error_t igraph_reindex_membership(igraph_vector_int_t *membership,
     /* Clean clusters. We will store the new cluster + 1 so that membership == 0
      * indicates that no cluster was assigned yet. */
     i_nb_clusters = 1;
-    for (i = 0; i < n; i++) {
-        igraph_integer_t c = VECTOR(*membership)[i];
+    for (igraph_int_t i = 0; i < vcount; i++) {
+        igraph_int_t c = VECTOR(*membership)[i];
 
-        if (c < 0) {
-            IGRAPH_ERRORF("Membership indices should non-negative. "
-            "Found member of cluster %" IGRAPH_PRId ".", IGRAPH_EINVAL, c);
-        }
-
-        if (c < 0) {
-            IGRAPH_ERRORF("Membership indices should be non-negative. "
-            "Found member of cluster %" IGRAPH_PRId ".", IGRAPH_EINVAL, c);
-        }
-
-        if (c >= n) {
-            IGRAPH_ERRORF("Membership indices should be less than total number of vertices. "
-            "Found member of cluster %" IGRAPH_PRId ", but only %" IGRAPH_PRId " vertices.", IGRAPH_EINVAL, c, n);
+        if (c < 0 || c >= vcount) {
+            /* Found cluster indices out of the 0..vcount-1 range.
+             * Use alternative implementation that supports these. */
+            igraph_vector_int_destroy(&new_cluster);
+            IGRAPH_FINALLY_CLEAN(1);
+            return igraph_i_reindex_membership_large(membership, new_to_old, nb_clusters);
         }
 
         if (VECTOR(new_cluster)[c] == 0) {
-            VECTOR(new_cluster)[c] = (igraph_real_t)i_nb_clusters;
+            VECTOR(new_cluster)[c] = i_nb_clusters;
             i_nb_clusters += 1;
             if (new_to_old) {
                 IGRAPH_CHECK(igraph_vector_int_push_back(new_to_old, c));
@@ -273,17 +366,17 @@ igraph_error_t igraph_reindex_membership(igraph_vector_int_t *membership,
     }
 
     /* Assign new membership */
-    for (i = 0; i < n; i++) {
-        igraph_integer_t c = VECTOR(*membership)[i];
+    for (igraph_int_t i = 0; i < vcount; i++) {
+        igraph_int_t c = VECTOR(*membership)[i];
         VECTOR(*membership)[i] = VECTOR(new_cluster)[c] - 1;
     }
+
     if (nb_clusters) {
         /* We used the cluster + 1, so correct */
         *nb_clusters = i_nb_clusters - 1;
     }
 
-    igraph_vector_destroy(&new_cluster);
-
+    igraph_vector_int_destroy(&new_cluster);
     IGRAPH_FINALLY_CLEAN(1);
 
     return IGRAPH_SUCCESS;
@@ -296,8 +389,8 @@ static igraph_error_t igraph_i_compare_communities_nmi(const igraph_vector_int_t
 static igraph_error_t igraph_i_compare_communities_rand(const igraph_vector_int_t *v1,
                                              const igraph_vector_int_t *v2, igraph_real_t* result, igraph_bool_t adjust);
 static igraph_error_t igraph_i_split_join_distance(const igraph_vector_int_t *v1,
-                                        const igraph_vector_int_t *v2, igraph_integer_t* distance12,
-                                        igraph_integer_t* distance21);
+                                        const igraph_vector_int_t *v2, igraph_int_t* distance12,
+                                        igraph_int_t* distance21);
 
 /**
  * \ingroup communities
@@ -317,7 +410,7 @@ static igraph_error_t igraph_i_split_join_distance(const igraph_vector_int_t *v1
  * of cluster \c i. Then the entropy of the clustering is
  *
  * </para><para>
- * <code>H(C) = - \sum_i p_i log p_i</code>
+ * <code>H(C) = - sum_i p_i log p_i</code>
  *
  * </para><para>
  * Similarly, we can define the joint entropy of two clusterings \c C_1 and \c C_2
@@ -325,7 +418,7 @@ static igraph_error_t igraph_i_split_join_distance(const igraph_vector_int_t *v1
  * in the first clustering and cluster \c j in the second one:
  *
  * </para><para>
- * <code>H(C_1, C_2) = - \sum_ii p_ij log p_ij</code>
+ * <code>H(C_1, C_2) = - sum_ii p_ij log p_ij</code>
  *
  * </para><para>
  * The mutual information of \c C_1 and \c C_2 is then
@@ -345,6 +438,8 @@ static igraph_error_t igraph_i_split_join_distance(const igraph_vector_int_t *v1
  * <code>VI(C_1, C_2) = [H(C_1) - MI(C_1, C_2)] + [H(C_2) - MI(C_1, C_2)]</code>.
  * Lower values of the variation of information indicate a smaller difference between
  * the two clusterings, with <code>VI = 0</code> achieved precisely when they coincide.
+ * igraph uses natural units for the variation of information, i.e. it uses the
+ * natural logarithm when computing entropies.
  *
  * </para><para>
  * The Rand index is defined as the probability that the two clusterings agree
@@ -438,8 +533,8 @@ igraph_error_t igraph_compare_communities(const igraph_vector_int_t *comm1,
     IGRAPH_CHECK(igraph_vector_int_init_copy(&c2, comm2));
     IGRAPH_FINALLY(igraph_vector_int_destroy, &c2);
 
-    IGRAPH_CHECK(igraph_reindex_membership(&c1, 0, NULL));
-    IGRAPH_CHECK(igraph_reindex_membership(&c2, 0, NULL));
+    IGRAPH_CHECK(igraph_reindex_membership(&c1, NULL, NULL));
+    IGRAPH_CHECK(igraph_reindex_membership(&c2, NULL, NULL));
 
     switch (method) {
     case IGRAPH_COMMCMP_VI:
@@ -451,7 +546,7 @@ igraph_error_t igraph_compare_communities(const igraph_vector_int_t *comm1,
         break;
 
     case IGRAPH_COMMCMP_SPLIT_JOIN: {
-        igraph_integer_t d12, d21;
+        igraph_int_t d12, d21;
         IGRAPH_CHECK(igraph_i_split_join_distance(&c1, &c2, &d12, &d21));
         *result = d12 + d21;
     }
@@ -511,10 +606,10 @@ igraph_error_t igraph_compare_communities(const igraph_vector_int_t *comm1,
  *
  * \param  comm1       the membership vector of the first community structure
  * \param  comm2       the membership vector of the second community structure
- * \param  distance12  pointer to an \c igraph_integer_t, the projection distance
+ * \param  distance12  pointer to an \c igraph_int_t, the projection distance
  *                     of the first community structure from the second one will be
  *                     returned here.
- * \param  distance21  pointer to an \c igraph_integer_t, the projection distance
+ * \param  distance21  pointer to an \c igraph_int_t, the projection distance
  *                     of the second community structure from the first one will be
  *                     returned here.
  * \return  Error code.
@@ -526,8 +621,8 @@ igraph_error_t igraph_compare_communities(const igraph_vector_int_t *comm1,
  * Time complexity: O(n log(n)).
  */
 igraph_error_t igraph_split_join_distance(const igraph_vector_int_t *comm1,
-                               const igraph_vector_int_t *comm2, igraph_integer_t *distance12,
-                               igraph_integer_t *distance21) {
+                               const igraph_vector_int_t *comm2, igraph_int_t *distance12,
+                               igraph_int_t *distance21) {
     igraph_vector_int_t c1, c2;
 
     if (igraph_vector_int_size(comm1) != igraph_vector_int_size(comm2)) {
@@ -542,8 +637,8 @@ igraph_error_t igraph_split_join_distance(const igraph_vector_int_t *comm1,
     IGRAPH_CHECK(igraph_vector_int_init_copy(&c2, comm2));
     IGRAPH_FINALLY(igraph_vector_int_destroy, &c2);
 
-    IGRAPH_CHECK(igraph_reindex_membership(&c1, 0, NULL));
-    IGRAPH_CHECK(igraph_reindex_membership(&c2, 0, NULL));
+    IGRAPH_CHECK(igraph_reindex_membership(&c1, NULL, NULL));
+    IGRAPH_CHECK(igraph_reindex_membership(&c2, NULL, NULL));
 
     IGRAPH_CHECK(igraph_i_split_join_distance(&c1, &c2, distance12, distance21));
 
@@ -562,10 +657,10 @@ igraph_error_t igraph_split_join_distance(const igraph_vector_int_t *comm1,
  */
 static igraph_error_t igraph_i_entropy_and_mutual_information(const igraph_vector_int_t* v1,
         const igraph_vector_int_t* v2, double* h1, double* h2, double* mut_inf) {
-    igraph_integer_t i, n;
-    igraph_integer_t k1;
-    igraph_integer_t k2;
-    double *p1, *p2;
+    igraph_int_t i, n;
+    igraph_int_t k1;
+    igraph_int_t k2;
+    igraph_real_t *p1, *p2;
     igraph_sparsemat_t m;
     igraph_sparsemat_t mu; /* uncompressed */
     igraph_sparsemat_iterator_t mit;
@@ -579,12 +674,12 @@ static igraph_error_t igraph_i_entropy_and_mutual_information(const igraph_vecto
     }
     k1 = igraph_vector_int_max(v1) + 1;
     k2 = igraph_vector_int_max(v2) + 1;
-    p1 = IGRAPH_CALLOC(k1, double);
+    p1 = IGRAPH_CALLOC(k1, igraph_real_t);
     if (p1 == 0) {
         IGRAPH_ERROR("Insufficient memory for computing community entropy.", IGRAPH_ENOMEM);
     }
     IGRAPH_FINALLY(igraph_free, p1);
-    p2 = IGRAPH_CALLOC(k2, double);
+    p2 = IGRAPH_CALLOC(k2, igraph_real_t);
     if (p2 == 0) {
         IGRAPH_ERROR("Insufficient memory for computing community entropy.", IGRAPH_ENOMEM);
     }
@@ -651,7 +746,7 @@ static igraph_error_t igraph_i_entropy_and_mutual_information(const igraph_vecto
 /**
  * Implementation of the normalized mutual information (NMI) measure of
  * Danon et al. This function assumes that the community membership
- * vectors have already been normalized using igraph_reindex_communities().
+ * vectors have already been normalized using igraph_reindex_membership().
  *
  * </para><para>
  * Reference: Danon L, Diaz-Guilera A, Duch J, Arenas A: Comparing community
@@ -678,7 +773,7 @@ static igraph_error_t igraph_i_compare_communities_nmi(const igraph_vector_int_t
 /**
  * Implementation of the variation of information metric (VI) of
  * Meila et al. This function assumes that the community membership
- * vectors have already been normalized using igraph_reindex_communities().
+ * vectors have already been normalized using igraph_reindex_membership().
  *
  * </para><para>
  * Reference: Meila M: Comparing clusterings by the variation of information.
@@ -705,7 +800,7 @@ static igraph_error_t igraph_i_compare_communities_vi(const igraph_vector_int_t
  *
  * </para><para>
  * This function assumes that the community membership vectors have already
- * been normalized using igraph_reindex_communities().
+ * been normalized using igraph_reindex_membership().
  *
  * </para><para>
  * Time complexity: O(n log(max(k1, k2))), where n is the number of vertices, k1
@@ -713,7 +808,7 @@ static igraph_error_t igraph_i_compare_communities_vi(const igraph_vector_int_t
  */
 static igraph_error_t igraph_i_confusion_matrix(const igraph_vector_int_t *v1, const igraph_vector_int_t *v2,
                               igraph_sparsemat_t *m) {
-    igraph_integer_t k1, k2, i, n;
+    igraph_int_t k1, k2, i, n;
 
     n = igraph_vector_int_size(v1);
     if (n == 0) {
@@ -738,7 +833,7 @@ static igraph_error_t igraph_i_confusion_matrix(const igraph_vector_int_t *v1, c
  *
  * </para><para>
  * This function assumes that the community membership vectors have already
- * been normalized using igraph_reindex_communities().
+ * been normalized using igraph_reindex_membership().
  *
  * </para><para>
  * Reference: van Dongen S: Performance criteria for graph clustering and Markov
@@ -750,8 +845,8 @@ static igraph_error_t igraph_i_confusion_matrix(const igraph_vector_int_t *v1, c
  * and k2 are the number of clusters in each of the clusterings.
  */
 static igraph_error_t igraph_i_split_join_distance(const igraph_vector_int_t *v1, const igraph_vector_int_t *v2,
-                                 igraph_integer_t* distance12, igraph_integer_t* distance21) {
-    igraph_integer_t n = igraph_vector_int_size(v1);
+                                 igraph_int_t* distance12, igraph_int_t* distance21) {
+    igraph_int_t n = igraph_vector_int_size(v1);
     igraph_vector_t rowmax, colmax;
     igraph_sparsemat_t m;
     igraph_sparsemat_t mu; /* uncompressed */
@@ -778,8 +873,8 @@ static igraph_error_t igraph_i_split_join_distance(const igraph_vector_int_t *v1
     IGRAPH_CHECK(igraph_sparsemat_iterator_init(&mit, &m));
     while (!igraph_sparsemat_iterator_end(&mit)) {
         igraph_real_t value = igraph_sparsemat_iterator_get(&mit);
-        igraph_integer_t row = igraph_sparsemat_iterator_row(&mit);
-        igraph_integer_t col = igraph_sparsemat_iterator_col(&mit);
+        igraph_int_t row = igraph_sparsemat_iterator_row(&mit);
+        igraph_int_t col = igraph_sparsemat_iterator_col(&mit);
         if (value > VECTOR(rowmax)[row]) {
             VECTOR(rowmax)[row] = value;
         }
@@ -790,8 +885,8 @@ static igraph_error_t igraph_i_split_join_distance(const igraph_vector_int_t *v1
     }
 
     /* Calculate the distances */
-    *distance12 = (igraph_integer_t) (n - igraph_vector_sum(&rowmax));
-    *distance21 = (igraph_integer_t) (n - igraph_vector_sum(&colmax));
+    *distance12 = (igraph_int_t) (n - igraph_vector_sum(&rowmax));
+    *distance21 = (igraph_int_t) (n - igraph_vector_sum(&colmax));
 
     igraph_vector_destroy(&rowmax);
     igraph_vector_destroy(&colmax);
@@ -807,7 +902,7 @@ static igraph_error_t igraph_i_split_join_distance(const igraph_vector_int_t *v1
  *
  * </para><para>
  * This function assumes that the community membership vectors have already
- * been normalized using igraph_reindex_communities().
+ * been normalized using igraph_reindex_membership().
  *
  * </para><para>
  * References:
@@ -831,9 +926,9 @@ static igraph_error_t igraph_i_compare_communities_rand(
     igraph_sparsemat_t mu; /* uncompressed */
     igraph_sparsemat_iterator_t mit;
     igraph_vector_t rowsums, colsums;
-    igraph_integer_t i, nrow, ncol;
-    double rand, n;
-    double frac_pairs_in_1, frac_pairs_in_2;
+    igraph_int_t i, nrow, ncol;
+    igraph_real_t rand, n;
+    igraph_real_t frac_pairs_in_1, frac_pairs_in_2;
 
     if (igraph_vector_int_size(v1) <= 1) {
         IGRAPH_ERRORF("Rand indices not defined for only zero or one vertices. "
@@ -877,7 +972,7 @@ static igraph_error_t igraph_i_compare_communities_rand(
     /* Calculate row and column sums */
     nrow = igraph_sparsemat_nrow(&mu);
     ncol = igraph_sparsemat_ncol(&mu);
-    n = igraph_vector_int_size(v1) + 0.0;
+    n = igraph_vector_int_size(v1);
     IGRAPH_VECTOR_INIT_FINALLY(&rowsums, nrow);
     IGRAPH_VECTOR_INIT_FINALLY(&colsums, ncol);
     IGRAPH_CHECK(igraph_sparsemat_rowsums(&mu, &rowsums));
