@@ -1,7 +1,5 @@
-/* -*- mode: C -*-  */
-/* vim:set ts=4 sw=4 sts=4 et: */
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2007-2020 The igraph development team
 
    This program is free software; you can redistribute it and/or modify
@@ -29,7 +27,6 @@
 #include "igraph_interface.h"
 #include "igraph_iterators.h"
 #include "igraph_random.h"
-#include "igraph_statusbar.h"
 #include "igraph_structural.h"
 
 #include "core/interruption.h"
@@ -46,13 +43,14 @@
  * eigenvectors of matrices, Phys Rev E 74:036104 (2006).</para>
  *
  * <para>
- * The heart of the method is the definition of the modularity matrix,
- * B, which is B=A-P, A being the adjacency matrix of the (undirected)
- * network, and P contains the probability that certain edges are
- * present according to the <quote>configuration model</quote> In
- * other words, a Pij element of P is the probability that there is an
- * edge between vertices i and j in a random network in which the
- * degrees of all vertices are the same as in the input graph.</para>
+ * The heart of the method is the definition of the modularity matrix
+ * <code>B = A - P</code>, \c A being the adjacency matrix of the (undirected)
+ * network, and \c P contains the probability that certain edges are
+ * present according to the <quote>configuration model</quote>. In
+ * other words, a \c P_ij element of \c P is the probability that there is an
+ * edge between vertices \c i and \c j in a random network in which the
+ * degrees of all vertices are the same as in the input graph. See
+ * \ref igraph_modularity_matrix() for more details.</para>
  *
  * <para>
  * The leading eigenvector method works by calculating the eigenvector
@@ -84,9 +82,9 @@ typedef struct igraph_i_community_leading_eigenvector_data_t {
     igraph_adjlist_t *adjlist;
     igraph_inclist_t *inclist;
     igraph_vector_t *tmp;
-    igraph_integer_t no_of_edges;
+    igraph_int_t no_of_edges;
     igraph_vector_int_t *mymembership;
-    igraph_integer_t comm;
+    igraph_int_t comm;
     const igraph_vector_t *weights;
     const igraph_t *graph;
     igraph_vector_t *strength;
@@ -99,26 +97,26 @@ static igraph_error_t igraph_i_community_leading_eigenvector(
         int n, void *extra) {
 
     igraph_i_community_leading_eigenvector_data_t *data = extra;
-    igraph_integer_t j, k, nlen, size = n;
+    igraph_int_t size = n;
     igraph_vector_int_t *idx = data->idx;
     igraph_vector_int_t *idx2 = data->idx2;
     igraph_vector_t *tmp = data->tmp;
     igraph_adjlist_t *adjlist = data->adjlist;
     igraph_real_t ktx, ktx2;
-    igraph_integer_t no_of_edges = data->no_of_edges;
+    igraph_int_t no_of_edges = data->no_of_edges;
     igraph_vector_int_t *mymembership = data->mymembership;
-    igraph_integer_t comm = data->comm;
+    igraph_int_t comm = data->comm;
 
     /* Ax */
-    for (j = 0; j < size; j++) {
-        igraph_integer_t oldid = VECTOR(*idx)[j];
+    for (igraph_int_t j = 0; j < size; j++) {
+        igraph_int_t oldid = VECTOR(*idx)[j];
         igraph_vector_int_t *neis = igraph_adjlist_get(adjlist, oldid);
-        nlen = igraph_vector_int_size(neis);
+        igraph_int_t nlen = igraph_vector_int_size(neis);
         to[j] = 0.0;
         VECTOR(*tmp)[j] = 0.0;
-        for (k = 0; k < nlen; k++) {
-            igraph_integer_t nei = VECTOR(*neis)[k];
-            igraph_integer_t neimemb = VECTOR(*mymembership)[nei];
+        for (igraph_int_t k = 0; k < nlen; k++) {
+            igraph_int_t nei = VECTOR(*neis)[k];
+            igraph_int_t neimemb = VECTOR(*mymembership)[nei];
             if (neimemb == comm) {
                 to[j] += from[ VECTOR(*idx2)[nei] ];
                 VECTOR(*tmp)[j] += 1;
@@ -128,10 +126,10 @@ static igraph_error_t igraph_i_community_leading_eigenvector(
 
     /* Now calculate k^Tx/2m */
     ktx = 0.0; ktx2 = 0.0;
-    for (j = 0; j < size; j++) {
-        igraph_integer_t oldid = VECTOR(*idx)[j];
+    for (igraph_int_t j = 0; j < size; j++) {
+        igraph_int_t oldid = VECTOR(*idx)[j];
         igraph_vector_int_t *neis = igraph_adjlist_get(adjlist, oldid);
-        igraph_integer_t degree = igraph_vector_int_size(neis);
+        igraph_int_t degree = igraph_vector_int_size(neis);
         ktx += from[j] * degree;
         ktx2 += degree;
     }
@@ -139,8 +137,8 @@ static igraph_error_t igraph_i_community_leading_eigenvector(
     ktx2 = ktx2 / no_of_edges / 2.0;
 
     /* Now calculate Bx */
-    for (j = 0; j < size; j++) {
-        igraph_integer_t oldid = VECTOR(*idx)[j];
+    for (igraph_int_t j = 0; j < size; j++) {
+        igraph_int_t oldid = VECTOR(*idx)[j];
         igraph_vector_int_t *neis = igraph_adjlist_get(adjlist, oldid);
         igraph_real_t degree = igraph_vector_int_size(neis);
         to[j] = to[j] - ktx * degree;
@@ -148,7 +146,7 @@ static igraph_error_t igraph_i_community_leading_eigenvector(
     }
 
     /* -d_ij summa l in G B_il */
-    for (j = 0; j < size; j++) {
+    for (igraph_int_t j = 0; j < size; j++) {
         to[j] -= VECTOR(*tmp)[j] * from[j];
     }
 
@@ -161,31 +159,31 @@ static igraph_error_t igraph_i_community_leading_eigenvector_weighted(
         int n, void *extra) {
 
     igraph_i_community_leading_eigenvector_data_t *data = extra;
-    igraph_integer_t j, k, nlen, size = n;
+    igraph_int_t size = n;
     igraph_vector_int_t *idx = data->idx;
     igraph_vector_int_t *idx2 = data->idx2;
     igraph_vector_t *tmp = data->tmp;
     igraph_inclist_t *inclist = data->inclist;
     igraph_real_t ktx, ktx2;
     igraph_vector_int_t *mymembership = data->mymembership;
-    igraph_integer_t comm = data->comm;
+    igraph_int_t comm = data->comm;
     const igraph_vector_t *weights = data->weights;
     const igraph_t *graph = data->graph;
     igraph_vector_t *strength = data->strength;
     igraph_real_t sw = data->sumweights;
 
     /* Ax */
-    for (j = 0; j < size; j++) {
-        igraph_integer_t oldid = VECTOR(*idx)[j];
+    for (igraph_int_t j = 0; j < size; j++) {
+        igraph_int_t oldid = VECTOR(*idx)[j];
         igraph_vector_int_t *inc = igraph_inclist_get(inclist, oldid);
-        nlen = igraph_vector_int_size(inc);
+        igraph_int_t nlen = igraph_vector_int_size(inc);
         to[j] = 0.0;
         VECTOR(*tmp)[j] = 0.0;
-        for (k = 0; k < nlen; k++) {
-            igraph_integer_t edge = VECTOR(*inc)[k];
+        for (igraph_int_t k = 0; k < nlen; k++) {
+            igraph_int_t edge = VECTOR(*inc)[k];
             igraph_real_t w = VECTOR(*weights)[edge];
-            igraph_integer_t nei = IGRAPH_OTHER(graph, edge, oldid);
-            igraph_integer_t neimemb = VECTOR(*mymembership)[nei];
+            igraph_int_t nei = IGRAPH_OTHER(graph, edge, oldid);
+            igraph_int_t neimemb = VECTOR(*mymembership)[nei];
             if (neimemb == comm) {
                 to[j] += from[ VECTOR(*idx2)[nei] ] * w;
                 VECTOR(*tmp)[j] += w;
@@ -195,8 +193,8 @@ static igraph_error_t igraph_i_community_leading_eigenvector_weighted(
 
     /* k^Tx/2m */
     ktx = 0.0; ktx2 = 0.0;
-    for (j = 0; j < size; j++) {
-        igraph_integer_t oldid = VECTOR(*idx)[j];
+    for (igraph_int_t j = 0; j < size; j++) {
+        igraph_int_t oldid = VECTOR(*idx)[j];
         igraph_real_t str = VECTOR(*strength)[oldid];
         ktx += from[j] * str;
         ktx2 += str;
@@ -205,15 +203,15 @@ static igraph_error_t igraph_i_community_leading_eigenvector_weighted(
     ktx2 = ktx2 / sw / 2.0;
 
     /* Bx */
-    for (j = 0; j < size; j++) {
-        igraph_integer_t oldid = VECTOR(*idx)[j];
+    for (igraph_int_t j = 0; j < size; j++) {
+        igraph_int_t oldid = VECTOR(*idx)[j];
         igraph_real_t str = VECTOR(*strength)[oldid];
         to[j] = to[j] - ktx * str;
         VECTOR(*tmp)[j] = VECTOR(*tmp)[j] - ktx2 * str;
     }
 
     /* -d_ij summa l in G B_il */
-    for (j = 0; j < size; j++) {
+    for (igraph_int_t j = 0; j < size; j++) {
         to[j] -= VECTOR(*tmp)[j] * from[j];
     }
 
@@ -241,22 +239,22 @@ static void igraph_i_error_handler_none(const char *reason, const char *file,
  * regarding the original network, see MEJ Newman: Finding community
  * structure in networks using the eigenvectors of matrices,
  * Phys Rev E 74:036104 (2006).
+ * https://doi.org/10.1103/PhysRevE.74.036104
  *
- * \param graph The undirected input graph.
- * \param weights The weights of the edges, or a null pointer for
- *    unweighted graphs.
+ * \param graph The input graph. Edge directions will be ignored.
+ * \param weights The weights of the edges, or \c NULL for unweighted graphs.
  * \param merges The result of the algorithm, a matrix containing the
  *    information about the splits performed. The matrix is built in
  *    the opposite way however, it is like the result of an
- *    agglomerative algorithm. Unlike with most other hierarchicaly
+ *    agglomerative algorithm. Unlike with most other hierarchical
  *    community detection functions in igraph, the integers in this matrix
  *    represent community indices, not vertex indices. If at the end of
  *    the algorithm (after \p steps steps was done) there are <quote>p</quote>
- *    communities, then these are numbered from zero to <quote>p-1</quote>.
+ *    communities, then these are numbered from zero to <code>p-1</code>.
  *    The first line of the matrix contains the first <quote>merge</quote>
  *    (which is in reality the last split) of two communities into
- *    community <quote>p</quote>, the merge in the second line forms
- *    community <quote>p+1</quote>, etc. The matrix should be
+ *    community <code>p</code>, the merge in the second line forms
+ *    community <code>p+1</code>, etc. The matrix should be
  *    initialized before calling and will be resized as needed.
  *    This argument is ignored if it is \c NULL.
  * \param membership The membership of the vertices after all the
@@ -265,7 +263,7 @@ static void igraph_i_error_handler_none(const char *reason, const char *file,
  *    This argument is ignored if it is \c NULL. This argument can
  *    also be used to supply a starting configuration for the community
  *    finding, in the format of a membership vector. In this case the
- *    \p start argument must be set to 1.
+ *    \p start argument must be set to \c true.
  * \param steps The maximum number of steps to perform. It might
  *    happen that some component (or the whole network) has no
  *    underlying community structure and no further steps can be
@@ -335,28 +333,28 @@ igraph_error_t igraph_community_leading_eigenvector(
         const igraph_vector_t *weights,
         igraph_matrix_int_t *merges,
         igraph_vector_int_t *membership,
-        igraph_integer_t steps,
+        igraph_int_t steps,
         igraph_arpack_options_t *options,
         igraph_real_t *modularity,
         igraph_bool_t start,
         igraph_vector_t *eigenvalues,
         igraph_vector_list_t *eigenvectors,
-        igraph_vector_t *history,
+        igraph_vector_int_t *history,
         igraph_community_leading_eigenvector_callback_t *callback,
         void *callback_extra) {
 
-    igraph_integer_t no_of_nodes = igraph_vcount(graph);
-    igraph_integer_t no_of_edges = igraph_ecount(graph);
+    igraph_int_t no_of_nodes = igraph_vcount(graph);
+    igraph_int_t no_of_edges = igraph_ecount(graph);
     igraph_dqueue_int_t tosplit;
     igraph_vector_int_t idx, idx2;
     igraph_vector_t mymerges;
     igraph_vector_t strength, tmp;
     igraph_vector_t start_vec;
-    igraph_integer_t staken = 0;
+    igraph_int_t staken = 0;
     igraph_adjlist_t adjlist;
     igraph_inclist_t inclist;
-    igraph_integer_t i, j, k, l;
-    igraph_integer_t communities;
+    igraph_int_t i, j, k, l;
+    igraph_int_t communities;
     igraph_vector_int_t vmembership, *mymembership = membership;
     igraph_i_community_leading_eigenvector_data_t extra;
     igraph_arpack_storage_t storage;
@@ -367,30 +365,29 @@ igraph_error_t igraph_community_leading_eigenvector(
     igraph_real_t sumweights = 0.0;
 
     if (no_of_nodes > INT_MAX) {
-        IGRAPH_ERROR("Graph too large for ARPACK", IGRAPH_EOVERFLOW);
+        IGRAPH_ERROR("Graph too large for ARPACK.", IGRAPH_EOVERFLOW);
     }
 
     if (weights && no_of_edges != igraph_vector_size(weights)) {
-        IGRAPH_ERROR("Invalid weight vector length", IGRAPH_EINVAL);
+        IGRAPH_ERROR("Weight vector length does not match number of edges.", IGRAPH_EINVAL);
     }
 
     if (start && !membership) {
-        IGRAPH_ERROR("Cannot start from given configuration if memberships "
-                     "missing", IGRAPH_EINVAL);
+        IGRAPH_ERROR("Cannot start from given configuration if memberships missing.", IGRAPH_EINVAL);
     }
 
     if (start && membership &&
         igraph_vector_int_size(membership) != no_of_nodes) {
-        IGRAPH_ERROR("Wrong length for vector of predefined memberships",
+        IGRAPH_ERROR("Supplied membership vector length does not match number of vertices.",
                      IGRAPH_EINVAL);
     }
 
     if (start && membership && igraph_vector_int_max(membership) >= no_of_nodes) {
-        IGRAPH_WARNING("Too many communities in membership start vector");
+        IGRAPH_WARNING("Too many communities in membership start vector.");
     }
 
     if (igraph_is_directed(graph)) {
-        IGRAPH_WARNING("This method was developed for undirected graphs");
+        IGRAPH_WARNING("Directed graph supplied, edge directions will be ignored.");
     }
 
     if (steps < 0 || steps > no_of_nodes - 1) {
@@ -415,24 +412,24 @@ igraph_error_t igraph_community_leading_eigenvector(
     if (!start) {
         /* Calculate the weakly connected components in the graph and use them as
          * an initial split */
-        IGRAPH_CHECK(igraph_connected_components(graph, mymembership, &idx, 0, IGRAPH_WEAK));
+        IGRAPH_CHECK(igraph_connected_components(graph, mymembership, &idx, NULL, IGRAPH_WEAK));
         communities = igraph_vector_int_size(&idx);
         if (history) {
-            IGRAPH_CHECK(igraph_vector_push_back(history,
+            IGRAPH_CHECK(igraph_vector_int_push_back(history,
                                                  IGRAPH_LEVC_HIST_START_FULL));
         }
     } else {
         /* Just create the idx vector for the given membership vector */
         communities = igraph_vector_int_max(mymembership) + 1;
         if (history) {
-            IGRAPH_CHECK(igraph_vector_push_back(history,
+            IGRAPH_CHECK(igraph_vector_int_push_back(history,
                                                  IGRAPH_LEVC_HIST_START_GIVEN));
-            IGRAPH_CHECK(igraph_vector_push_back(history, communities));
+            IGRAPH_CHECK(igraph_vector_int_push_back(history, communities));
         }
         IGRAPH_CHECK(igraph_vector_int_resize(&idx, communities));
         igraph_vector_int_null(&idx);
         for (i = 0; i < no_of_nodes; i++) {
-            igraph_integer_t t = VECTOR(*mymembership)[i];
+            igraph_int_t t = VECTOR(*mymembership)[i];
             VECTOR(idx)[t] += 1;
         }
     }
@@ -457,8 +454,8 @@ igraph_error_t igraph_community_leading_eigenvector(
             IGRAPH_CHECK(igraph_vector_list_push_back_new(eigenvectors, NULL));
         }
         if (history) {
-            IGRAPH_CHECK(igraph_vector_push_back(history, IGRAPH_LEVC_HIST_SPLIT));
-            IGRAPH_CHECK(igraph_vector_push_back(history, i - 1));
+            IGRAPH_CHECK(igraph_vector_int_push_back(history, IGRAPH_LEVC_HIST_SPLIT));
+            IGRAPH_CHECK(igraph_vector_int_push_back(history, i - 1));
         }
     }
     staken = communities - 1;
@@ -479,7 +476,7 @@ igraph_error_t igraph_community_leading_eigenvector(
         sumweights = igraph_vector_sum(weights);
     }
 
-    if (options == 0) {
+    if (options == NULL) {
         options = igraph_arpack_options_get_default();
     }
 
@@ -505,9 +502,9 @@ igraph_error_t igraph_community_leading_eigenvector(
     extra.mymembership = mymembership;
 
     while (!igraph_dqueue_int_empty(&tosplit) && staken < steps) {
-        igraph_integer_t comm = igraph_dqueue_int_pop_back(&tosplit);
+        igraph_int_t comm = igraph_dqueue_int_pop_back(&tosplit);
         /* depth first search */
-        igraph_integer_t size = 0;
+        igraph_int_t size = 0;
 
         IGRAPH_ALLOW_INTERRUPTION();
 
@@ -527,9 +524,9 @@ igraph_error_t igraph_community_leading_eigenvector(
         options->info = 0;
         options->nev = 1;
         options->ldv = 0;
-        options->ncv = 0;   /* 0 means "automatic" in igraph_arpack_rssolve */
+        options->ncv = 0; /* 0 means "automatic" in igraph_arpack_rssolve */
         options->nconv = 0;
-        options->lworkl = 0;        /* we surely have enough space */
+        options->lworkl = 0; /* we surely have enough space */
         extra.comm = comm;
 
         /* Use a random start vector, but don't let ARPACK generate the
@@ -541,27 +538,33 @@ igraph_error_t igraph_community_leading_eigenvector(
          * convergence in most cases. */
         options->start = 1;
         options->mxiter = options->mxiter > 10000 ? options->mxiter : 10000;  /* use more iterations, we've had convergence problems with 3000 */
-        RNG_BEGIN();
+
         for (i = 0; i < options->n; i++) {
             storage.resid[i] = (i % 2 ? 1 : -1) + RNG_UNIF(-0.1, 0.1);
         }
-        RNG_END();
-        igraph_vector_view(&start_vec, storage.resid, options->n);
-        IGRAPH_CHECK(igraph_vector_shuffle(&start_vec));
+
+        start_vec = igraph_vector_view(storage.resid, options->n);
+        igraph_vector_shuffle(&start_vec);
 
         {
             igraph_error_t retval;
             igraph_error_handler_t *errh =
                     igraph_set_error_handler(igraph_i_error_handler_none);
-            retval = igraph_arpack_rssolve(arpcb1, &extra, options, &storage, /*values=*/ 0, /*vectors=*/ 0);
+            retval = igraph_arpack_rssolve(arpcb1, &extra, options, &storage, /*values=*/ NULL, /*vectors=*/ NULL);
             igraph_set_error_handler(errh);
-            if (retval != IGRAPH_SUCCESS && retval != IGRAPH_ARPACK_MAXIT && retval != IGRAPH_ARPACK_NOSHIFT) {
-                IGRAPH_ERROR("ARPACK call failed", retval);
+            if (retval == IGRAPH_EARPACK) {
+                /* TODO(ntamas): get last ARPACK error code. Some errors are OK. */
+                igraph_arpack_error_t arpack_error = igraph_arpack_get_last_error();
+                if (arpack_error != IGRAPH_ARPACK_MAXIT && arpack_error != IGRAPH_ARPACK_NOSHIFT) {
+                    IGRAPH_ERROR(igraph_arpack_error_to_string(arpack_error), IGRAPH_EARPACK);
+                }
+            } else if (retval != IGRAPH_SUCCESS) {
+                IGRAPH_ERROR("Leading eigenvector calculation failed.", retval);
             }
         }
 
         if (options->nconv < 1) {
-            IGRAPH_ERROR("ARPACK did not converge", IGRAPH_ARPACK_FAILED);
+            IGRAPH_ERROR(igraph_arpack_error_to_string(IGRAPH_ARPACK_FAILED), IGRAPH_EARPACK);
         }
 
         /* Ok, we have the leading eigenvector of the modularity matrix */
@@ -596,10 +599,9 @@ igraph_error_t igraph_community_leading_eigenvector(
         /* ---------------------------------------------------------------*/
 
         if (callback) {
-            igraph_vector_t vv;
+            const igraph_vector_t vv = igraph_vector_view(storage.v, size);;
             igraph_error_t ret;
 
-            igraph_vector_view(&vv, storage.v, size);
             IGRAPH_CHECK_CALLBACK(
                         callback(
                             mymembership, comm, storage.d[0], &vv, arpcb1,
@@ -628,9 +630,9 @@ igraph_error_t igraph_community_leading_eigenvector(
 
         if (storage.d[0] <= 0) {
             if (history) {
-                IGRAPH_CHECK(igraph_vector_push_back(history,
+                IGRAPH_CHECK(igraph_vector_int_push_back(history,
                                                      IGRAPH_LEVC_HIST_FAILED));
-                IGRAPH_CHECK(igraph_vector_push_back(history, comm));
+                IGRAPH_CHECK(igraph_vector_int_push_back(history, comm));
             }
             continue;
         }
@@ -647,9 +649,9 @@ igraph_error_t igraph_community_leading_eigenvector(
         }
         if (l == 0 || l == size) {
             if (history) {
-                IGRAPH_CHECK(igraph_vector_push_back(history,
+                IGRAPH_CHECK(igraph_vector_int_push_back(history,
                                                      IGRAPH_LEVC_HIST_FAILED));
-                IGRAPH_CHECK(igraph_vector_push_back(history, comm));
+                IGRAPH_CHECK(igraph_vector_int_push_back(history, comm));
             }
             continue;
         }
@@ -662,9 +664,9 @@ igraph_error_t igraph_community_leading_eigenvector(
         }
         if (mod <= 1e-8) {
             if (history) {
-                IGRAPH_CHECK(igraph_vector_push_back(history,
+                IGRAPH_CHECK(igraph_vector_int_push_back(history,
                                                      IGRAPH_LEVC_HIST_FAILED));
-                IGRAPH_CHECK(igraph_vector_push_back(history, comm));
+                IGRAPH_CHECK(igraph_vector_int_push_back(history, comm));
             }
             continue;
         }
@@ -674,7 +676,7 @@ igraph_error_t igraph_community_leading_eigenvector(
         /* Rewrite the mymembership vector */
         for (j = 0; j < size; j++) {
             if (storage.v[j] < 0) {
-                igraph_integer_t oldid = VECTOR(idx)[j];
+                igraph_int_t oldid = VECTOR(idx)[j];
                 VECTOR(*mymembership)[oldid] = communities - 1;
             }
         }
@@ -683,8 +685,8 @@ igraph_error_t igraph_community_leading_eigenvector(
         IGRAPH_CHECK(igraph_vector_push_back(&mymerges, comm));
         IGRAPH_CHECK(igraph_vector_push_back(&mymerges, communities - 1));
         if (history) {
-            IGRAPH_CHECK(igraph_vector_push_back(history, IGRAPH_LEVC_HIST_SPLIT));
-            IGRAPH_CHECK(igraph_vector_push_back(history, comm));
+            IGRAPH_CHECK(igraph_vector_int_push_back(history, IGRAPH_LEVC_HIST_SPLIT));
+            IGRAPH_CHECK(igraph_vector_int_push_back(history, comm));
         }
 
         /* Store the resulting communities in the queue if needed */
@@ -720,8 +722,8 @@ igraph_error_t igraph_community_leading_eigenvector(
         j = 0;
         IGRAPH_CHECK(igraph_matrix_int_resize(merges, l / 2, 2));
         for (i = l; i > 0; i -= 2) {
-            igraph_integer_t from = VECTOR(mymerges)[i - 1];
-            igraph_integer_t to = VECTOR(mymerges)[i - 2];
+            igraph_int_t from = VECTOR(mymerges)[i - 1];
+            igraph_int_t to = VECTOR(mymerges)[i - 2];
             MATRIX(*merges, j, 0) = VECTOR(mymerges)[i - 2];
             MATRIX(*merges, j, 1) = VECTOR(mymerges)[i - 1];
             if (VECTOR(idx)[from] != 0) {
@@ -742,7 +744,7 @@ igraph_error_t igraph_community_leading_eigenvector(
     if (modularity) {
         IGRAPH_CHECK(igraph_modularity(graph, mymembership, weights,
                                        /* resolution */ 1,
-                                       /* only undirected */ 0, modularity));
+                                       IGRAPH_UNDIRECTED, modularity));
     }
 
     if (!membership) {
@@ -755,12 +757,32 @@ igraph_error_t igraph_community_leading_eigenvector(
 
 /**
  * \function igraph_le_community_to_membership
- * \brief Vertex membership from the leading eigenvector community structure.
+ * \brief Cut an incomplete dendrogram after a given number of merges, starting with an initial cluster assignment.
  *
- * This function creates a membership vector from the
- * result of \ref igraph_community_leading_eigenvector().
- * It takes \c membership and performs \c steps merges,
- * according to the supplied \c merges matrix.
+ * This function takes a dendrogram whose leaves are cluster IDs given in an
+ * initial cluster assignment provided in \p membership. Then it updates
+ * the cluster assignment by performing the specified number of mergers,
+ * as given by the dendrogram encoded in \p merges. It is a more general
+ * version of \ref igraph_community_to_membership(), which assumes that
+ * the dendrogram leaves are singleton clusters corresponding to individual
+ * vertices.
+ *
+ * </para><para>
+ * This dendrogram format is suitable for divise hierarchical community
+ * detection algorithms that stop before dividing the graph into individual
+ * vertices, such as \ref igraph_community_leading_eigenvector().
+ *
+ * </para><para>
+ * Initially, \p membership is expected to contain \c m contiguous cluster
+ * indices, numbered from zero. These correspond to the leaf nodes of the
+ * dendrogram. Row \c i of the two-column \p merges matrix contains the IDs of
+ * clusters that are merged together into dendrogram node <code>m + i</code>.
+ * It may have up to <code>m - 1</code> rows.
+ *
+ * </para><para>
+ * This function performs \p steps merge operations as prescribed by the
+ * \p merges matrix and updates \p membership to the resulting partitioning
+ * into <code>m - steps</code> communities.
  *
  * \param merges The two-column matrix containing the merge operations.
  *    See \ref igraph_community_leading_eigenvector() for the
@@ -773,16 +795,19 @@ igraph_error_t igraph_community_leading_eigenvector(
  *     if this is not a null pointer, but an initialized vector.
  * \return Error code.
  *
+ * \sa \ref igraph_community_to_membership() for a simpler interface that
+ * starts by merging individual vertices.
+ *
  * Time complexity: O(|V|), the number of vertices.
  */
 igraph_error_t igraph_le_community_to_membership(const igraph_matrix_int_t *merges,
-                                                 igraph_integer_t steps,
+                                                 igraph_int_t steps,
                                                  igraph_vector_int_t *membership,
                                                  igraph_vector_int_t *csize) {
 
-    igraph_integer_t no_of_nodes = igraph_vector_int_size(membership);
+    igraph_int_t no_of_nodes = igraph_vector_int_size(membership);
     igraph_vector_int_t fake_memb;
-    igraph_integer_t components, i;
+    igraph_int_t components, i;
 
     if (no_of_nodes > 0) {
         components = igraph_vector_int_max(membership) + 1;
@@ -817,7 +842,7 @@ igraph_error_t igraph_le_community_to_membership(const igraph_matrix_int_t *merg
         }
     }
 
-    IGRAPH_CHECK(igraph_community_to_membership(merges, components, steps, &fake_memb, 0));
+    IGRAPH_CHECK(igraph_community_to_membership(merges, components, steps, &fake_memb, NULL));
 
     /* Ok, now we have the membership of the initial components,
        rewrite the original membership vector. */
