@@ -1,6 +1,5 @@
-/* -*- mode: C -*-  */
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2006-2012  Gabor Csardi <csardi.gabor@gmail.com>
    334 Harvard street, Cambridge, MA 02139 USA
 
@@ -61,7 +60,7 @@ static igraph_error_t igraph_i_community_spinglass_orig(
         igraph_real_t *temperature,
         igraph_vector_int_t *membership,
         igraph_vector_int_t *csize,
-        igraph_integer_t spins,
+        igraph_int_t spins,
         igraph_bool_t parupdate,
         igraph_real_t starttemp,
         igraph_real_t stoptemp,
@@ -76,16 +75,13 @@ static igraph_error_t igraph_i_community_spinglass_negative(
         igraph_real_t *temperature,
         igraph_vector_int_t *membership,
         igraph_vector_int_t *csize,
-        igraph_integer_t spins,
+        igraph_int_t spins,
         igraph_bool_t parupdate,
         igraph_real_t starttemp,
         igraph_real_t stoptemp,
         igraph_real_t coolfact,
         igraph_spincomm_update_t update_rule,
         igraph_real_t gamma,
-        /* igraph_matrix_t *adhesion, */
-        /* igraph_matrix_t *normalised_adhesion, */
-        /* igraph_real_t *polarization, */
         igraph_real_t gamma_minus);
 
 /**
@@ -111,9 +107,8 @@ static igraph_error_t igraph_i_community_spinglass_negative(
  *     implementation.
  * \param modularity Pointer to a real number, if not \c NULL then the
  *     modularity score of the solution will be stored here. This is the
- *     gereralized modularity that simplifies to the one defined in
- *     M. E. J. Newman and M. Girvan, Phys. Rev. E 69, 026113 (2004),
- *     if the gamma parameter is one.
+ *     gereralized modularity, taking into account the resolution parameter
+ *     \p gamma. See \ref igraph_modularity() for details.
  * \param temperature Pointer to a real number, if not \c NULL then
  *     the temperature at the end of the algorithm will be stored
  *     here.
@@ -128,7 +123,7 @@ static igraph_error_t igraph_i_community_spinglass_negative(
  * \param spins Integer giving the number of spins, i.e. the maximum
  *     number of clusters. Even if the number of spins is high the number of
  *     clusters in the result might be small.
- * \param parupdate A logical constant, whether to update all spins in
+ * \param parupdate A Boolean constant, whether to update all spins in
  *     parallel. It is not implemented in the \c IGRAPH_SPINCOMM_INP_NEG
  *     implementation.
  * \param starttemp Real number, the temperature at the start. A reasonable
@@ -162,7 +157,7 @@ static igraph_error_t igraph_i_community_spinglass_negative(
  *     weights, using the number of spins as the number of colors.
  * \return Error code.
  *
- * \sa igraph_community_spinglass_single() for calculating the community
+ * \sa \ref igraph_community_spinglass_single() for calculating the community
  * of a single vertex.
  *
  * Time complexity: TODO.
@@ -175,7 +170,7 @@ igraph_error_t igraph_community_spinglass(const igraph_t *graph,
                                igraph_real_t *temperature,
                                igraph_vector_int_t *membership,
                                igraph_vector_int_t *csize,
-                               igraph_integer_t spins,
+                               igraph_int_t spins,
                                igraph_bool_t parupdate,
                                igraph_real_t starttemp,
                                igraph_real_t stoptemp,
@@ -203,7 +198,7 @@ igraph_error_t igraph_community_spinglass(const igraph_t *graph,
                     gamma_minus);
             break;
         default:
-            IGRAPH_ERROR("Unknown `implementation' in spinglass community finding",
+            IGRAPH_ERROR("Unknown implementation in spinglass community detection.",
                          IGRAPH_EINVAL);
         }
     );
@@ -216,7 +211,7 @@ static igraph_error_t igraph_i_community_spinglass_orig(
         igraph_real_t *temperature,
         igraph_vector_int_t *membership,
         igraph_vector_int_t *csize,
-        igraph_integer_t spins,
+        igraph_int_t spins,
         igraph_bool_t parupdate,
         igraph_real_t starttemp,
         igraph_real_t stoptemp,
@@ -224,8 +219,8 @@ static igraph_error_t igraph_i_community_spinglass_orig(
         igraph_spincomm_update_t update_rule,
         igraph_real_t gamma) {
 
-    igraph_integer_t no_of_nodes = igraph_vcount(graph);
-    unsigned long changes, runs;
+    igraph_int_t no_of_nodes = igraph_vcount(graph);
+    igraph_int_t changes, runs;
     igraph_bool_t use_weights = false;
     bool zeroT;
     double kT, acc, prob;
@@ -233,30 +228,39 @@ static igraph_error_t igraph_i_community_spinglass_orig(
     /* Check arguments */
 
     if (spins < 2) {
-        IGRAPH_ERROR("Number of spins must be at least 2", IGRAPH_EINVAL);
+        IGRAPH_ERROR("Number of spins must be at least 2.", IGRAPH_EINVAL);
     }
     if (update_rule != IGRAPH_SPINCOMM_UPDATE_SIMPLE &&
         update_rule != IGRAPH_SPINCOMM_UPDATE_CONFIG) {
-        IGRAPH_ERROR("Invalid update rule", IGRAPH_EINVAL);
+        IGRAPH_ERROR("Invalid update rule for spinglass community detection.", IGRAPH_EINVAL);
     }
     if (weights) {
         if (igraph_vector_size(weights) != igraph_ecount(graph)) {
-            IGRAPH_ERROR("Invalid weight vector length", IGRAPH_EINVAL);
+            IGRAPH_ERROR("Invalid weight vector length.", IGRAPH_EINVAL);
         }
-        use_weights = 1;
-        if (igraph_vector_min(weights) < 0) {
-            IGRAPH_ERROR("Weights must not be negative when using the original implementation of spinglass communities. Select the implementation meant for negative weights.", IGRAPH_EINVAL);
+        use_weights = true;
+        if (igraph_vector_size(weights) > 0 && igraph_vector_min(weights) < 0) {
+            IGRAPH_ERROR(
+                "Weights must not be negative when using the original implementation of spinglass communities. "
+                "Select the implementation meant for negative weights.",
+                IGRAPH_EINVAL);
         }
     }
     if (coolfact < 0 || coolfact >= 1.0) {
-        IGRAPH_ERROR("Invalid cooling factor", IGRAPH_EINVAL);
+        IGRAPH_ERROR("Cooling factor must be positive and strictly smaller than 1.", IGRAPH_EINVAL);
     }
     if (gamma < 0.0) {
-        IGRAPH_ERROR("Invalid gamma value", IGRAPH_EINVAL);
+        IGRAPH_ERROR("Gamma value must not be negative.", IGRAPH_EINVAL);
     }
-    if (starttemp / stoptemp < 1.0) {
-        IGRAPH_ERROR("starttemp should be larger in absolute value than stoptemp",
-                     IGRAPH_EINVAL);
+    if ( !(starttemp == 0 && stoptemp == 0) ) {
+        if (! (starttemp > 0 && stoptemp > 0)) {
+            IGRAPH_ERROR("Starting and stopping temperatures must be both positive or both zero.",
+                         IGRAPH_EINVAL);
+        }
+        if (starttemp <= stoptemp) {
+            IGRAPH_ERROR("The starting temperature must be larger than the stopping temperature.",
+                         IGRAPH_EINVAL);
+        }
     }
 
     /* The spinglass algorithm does not handle the trivial cases of the
@@ -264,10 +268,10 @@ static igraph_error_t igraph_i_community_spinglass_orig(
     if (no_of_nodes < 2) {
         if (membership) {
             IGRAPH_CHECK(igraph_vector_int_resize(membership, no_of_nodes));
-            igraph_vector_int_fill(membership, 0);
+            igraph_vector_int_null(membership);
         }
         if (modularity) {
-            IGRAPH_CHECK(igraph_modularity(graph, membership, 0, 1, igraph_is_directed(graph), modularity));
+            IGRAPH_CHECK(igraph_modularity(graph, membership, nullptr, 1, igraph_is_directed(graph), modularity));
         }
         if (temperature) {
             *temperature = stoptemp;
@@ -284,22 +288,19 @@ static igraph_error_t igraph_i_community_spinglass_orig(
     igraph_bool_t conn;
     IGRAPH_CHECK(igraph_is_connected(graph, &conn, IGRAPH_WEAK));
     if (!conn) {
-        IGRAPH_ERROR("Cannot work with unconnected graph", IGRAPH_EINVAL);
+        IGRAPH_ERROR("Cannot work with unconnected graph.", IGRAPH_EINVAL);
     }
 
     network net;
 
     /* Transform the igraph_t */
-    IGRAPH_CHECK(igraph_i_read_network(graph, weights,
-                                       &net, use_weights, 0));
+    IGRAPH_CHECK(igraph_i_read_network_spinglass(graph, weights,
+                                       &net, use_weights));
 
-    prob = 2.0 * net.sum_weights / double(net.node_list->Size())
-           / double(net.node_list->Size() - 1);
+    prob = 2.0 * net.sum_weights / double(net.node_list.Size())
+           / double(net.node_list.Size() - 1);
 
-    PottsModel pm(&net, (unsigned int)spins, update_rule);
-
-    /* initialize the random number generator */
-    RNG_BEGIN();
+    PottsModel pm(&net, spins, update_rule);
 
     if ((stoptemp == 0.0) && (starttemp == 0.0)) {
         zeroT = true;
@@ -350,8 +351,6 @@ static igraph_error_t igraph_i_community_spinglass_orig(
 
     pm.WriteClusters(modularity, temperature, csize, membership, kT, gamma);
 
-    RNG_END();
-
     return IGRAPH_SUCCESS;
 }
 
@@ -373,7 +372,7 @@ static igraph_error_t igraph_i_community_spinglass_orig(
  * \param weights Pointer to a vector with the weights of the edges.
  *    Alternatively \c NULL can be supplied to have the same weight
  *    for every edge.
- * \param vertex The vertex ID of the vertex of which ths community is
+ * \param vertex The vertex ID of the vertex of which this community is
  *    calculated.
  * \param community Pointer to an initialized vector, the result, the
  *    IDs of the vertices in the community of the input vertex will be
@@ -382,11 +381,12 @@ static igraph_error_t igraph_i_community_spinglass_orig(
  *     cohesion index of the community will be stored here.
  * \param adhesion Pointer to a real variable, if not \c NULL the
  *     adhesion index of the community will be stored here.
- * \param inner_links Pointer to an integer, if not \c NULL the
- *     number of edges within the community is stored here.
- * \param outer_links Pointer to an integer, if not \c NULL the
+ * \param inner_links Pointer to a real, if not \c NULL the
+ *     number of edges within the community (or the sum of their weights)
+ *     is stored here.
+ * \param outer_links Pointer to a real, if not \c NULL the
  *     number of edges between the community and the rest of the graph
- *     will be stored here.
+ *     (or the sum of their weights) will be stored here.
  * \param spins The number of spins to use, this can be higher than
  *    the actual number of clusters in the network, in which case some
  *    clusters will contain zero vertices.
@@ -417,24 +417,23 @@ static igraph_error_t igraph_i_community_spinglass_orig(
 
 igraph_error_t igraph_community_spinglass_single(const igraph_t *graph,
                                       const igraph_vector_t *weights,
-                                      igraph_integer_t vertex,
+                                      igraph_int_t vertex,
                                       igraph_vector_int_t *community,
                                       igraph_real_t *cohesion,
                                       igraph_real_t *adhesion,
-                                      igraph_integer_t *inner_links,
-                                      igraph_integer_t *outer_links,
-                                      igraph_integer_t spins,
+                                      igraph_real_t *inner_links,
+                                      igraph_real_t *outer_links,
+                                      igraph_int_t spins,
                                       igraph_spincomm_update_t update_rule,
                                       igraph_real_t gamma) {
     IGRAPH_HANDLE_EXCEPTIONS(
         igraph_bool_t use_weights = false;
-        double prob;
-        char startnode[255];
+        char startnode[SPINGLASS_MAX_NAME_LEN];
 
         /* Check arguments */
 
         if (spins < 2) {
-            IGRAPH_ERROR("Number of spins must be at least 2", IGRAPH_EINVAL);
+            IGRAPH_ERROR("Number of spins must be at least 2.", IGRAPH_EINVAL);
         }
         if (update_rule != IGRAPH_SPINCOMM_UPDATE_SIMPLE &&
             update_rule != IGRAPH_SPINCOMM_UPDATE_CONFIG) {
@@ -442,47 +441,39 @@ igraph_error_t igraph_community_spinglass_single(const igraph_t *graph,
         }
         if (weights) {
             if (igraph_vector_size(weights) != igraph_ecount(graph)) {
-                IGRAPH_ERROR("Invalid weight vector length", IGRAPH_EINVAL);
+                IGRAPH_ERROR("Invalid edge weight vector length.", IGRAPH_EINVAL);
             }
             use_weights = 1;
         }
         if (gamma < 0.0) {
-            IGRAPH_ERROR("Invalid gamme value", IGRAPH_EINVAL);
+            IGRAPH_ERROR("Invalid gamma value.", IGRAPH_EINVAL);
         }
         if (vertex < 0 || vertex > igraph_vcount(graph)) {
-            IGRAPH_ERROR("Invalid vertex ID", IGRAPH_EINVAL);
+            IGRAPH_ERROR("Invalid vertex ID.", IGRAPH_EINVAL);
         }
 
         /* Check whether we have a single component */
         igraph_bool_t conn;
         IGRAPH_CHECK(igraph_is_connected(graph, &conn, IGRAPH_WEAK));
         if (!conn) {
-            IGRAPH_ERROR("Cannot work with unconnected graph", IGRAPH_EINVAL);
+            IGRAPH_ERROR("Cannot work with disconnected graph.", IGRAPH_EINVAL);
         }
 
         network net;
 
         /* Transform the igraph_t */
-        IGRAPH_CHECK(igraph_i_read_network(graph, weights,
-                                           &net, use_weights, 0));
+        IGRAPH_CHECK(igraph_i_read_network_spinglass(graph, weights,
+                                           &net, use_weights));
 
-        prob = 2.0 * net.sum_weights / double(net.node_list->Size())
-               / double(net.node_list->Size() - 1);
+        PottsModel pm(&net, spins, update_rule);
 
-        PottsModel pm(&net, (unsigned int)spins, update_rule);
-
-        /* initialize the random number generator */
-        RNG_BEGIN();
-
-        /* to be exected, if we want to find the community around a particular node*/
+        /* to be expected, if we want to find the community around a particular node*/
         /* the initial conf is needed, because otherwise,
            the degree of the nodes is not in the weight property, stupid!!! */
         pm.assign_initial_conf(-1);
-        snprintf(startnode, 255, "%" IGRAPH_PRId "", vertex + 1);
-        pm.FindCommunityFromStart(gamma, prob, startnode, community,
+        snprintf(startnode, sizeof(startnode) / sizeof(startnode[0]), "%" IGRAPH_PRId "", vertex + 1);
+        pm.FindCommunityFromStart(gamma, startnode, community,
                                    cohesion, adhesion, inner_links, outer_links);
-
-        RNG_END();
     );
 
     return IGRAPH_SUCCESS;
@@ -495,20 +486,17 @@ static igraph_error_t igraph_i_community_spinglass_negative(
         igraph_real_t *temperature,
         igraph_vector_int_t *membership,
         igraph_vector_int_t *csize,
-        igraph_integer_t spins,
+        igraph_int_t spins,
         igraph_bool_t parupdate,
         igraph_real_t starttemp,
         igraph_real_t stoptemp,
         igraph_real_t coolfact,
         igraph_spincomm_update_t update_rule,
         igraph_real_t gamma,
-        /* igraph_matrix_t *adhesion, */
-        /* igraph_matrix_t *normalised_adhesion, */
-        /* igraph_real_t *polarization, */
         igraph_real_t gamma_minus) {
 
-    igraph_integer_t no_of_nodes = igraph_vcount(graph);
-    unsigned long changes, runs;
+    igraph_int_t no_of_nodes = igraph_vcount(graph);
+    igraph_int_t runs;
     igraph_bool_t use_weights = false;
     bool zeroT;
     double kT, acc;
@@ -518,32 +506,38 @@ static igraph_error_t igraph_i_community_spinglass_negative(
     /* Check arguments */
 
     if (parupdate) {
-        IGRAPH_ERROR("Parallel spin update not implemented with "
-                     "negative gamma", IGRAPH_UNIMPLEMENTED);
+        IGRAPH_ERROR("Parallel spin update not implemented with negative weights.",
+                     IGRAPH_UNIMPLEMENTED);
     }
 
     if (spins < 2) {
-        IGRAPH_ERROR("Number of spins must be at least 2", IGRAPH_EINVAL);
+        IGRAPH_ERROR("Number of spins must be at least 2.", IGRAPH_EINVAL);
     }
     if (update_rule != IGRAPH_SPINCOMM_UPDATE_SIMPLE &&
         update_rule != IGRAPH_SPINCOMM_UPDATE_CONFIG) {
-        IGRAPH_ERROR("Invalid update rule", IGRAPH_EINVAL);
+        IGRAPH_ERROR("Invalid update rule for spinglass community detection.", IGRAPH_EINVAL);
     }
     if (weights) {
         if (igraph_vector_size(weights) != igraph_ecount(graph)) {
-            IGRAPH_ERROR("Invalid weight vector length", IGRAPH_EINVAL);
+            IGRAPH_ERROR("Invalid weight vector length.", IGRAPH_EINVAL);
         }
-        use_weights = 1;
+        use_weights = true;
     }
     if (coolfact < 0 || coolfact >= 1.0) {
-        IGRAPH_ERROR("Invalid cooling factor", IGRAPH_EINVAL);
+        IGRAPH_ERROR("Cooling factor must be positive and strictly smaller than 1.", IGRAPH_EINVAL);
     }
     if (gamma < 0.0) {
-        IGRAPH_ERROR("Invalid gamma value", IGRAPH_EINVAL);
+        IGRAPH_ERROR("Gamma value must not be negative.", IGRAPH_EINVAL);
     }
-    if (starttemp / stoptemp < 1.0) {
-        IGRAPH_ERROR("starttemp should be larger in absolute value than stoptemp",
-                     IGRAPH_EINVAL);
+    if ( !(starttemp == 0 && stoptemp == 0) ) {
+        if (! (starttemp > 0 && stoptemp > 0)) {
+            IGRAPH_ERROR("Starting and stopping temperatures must be both positive or both zero.",
+                         IGRAPH_EINVAL);
+        }
+        if (starttemp <= stoptemp) {
+            IGRAPH_ERROR("The starting temperature must be larger than the stopping temperature.",
+                         IGRAPH_EINVAL);
+        }
     }
 
     /* The spinglass algorithm does not handle the trivial cases of the
@@ -551,10 +545,10 @@ static igraph_error_t igraph_i_community_spinglass_negative(
     if (no_of_nodes < 2) {
         if (membership) {
             IGRAPH_CHECK(igraph_vector_int_resize(membership, no_of_nodes));
-            igraph_vector_int_fill(membership, 0);
+            igraph_vector_int_null(membership);
         }
         if (modularity) {
-            IGRAPH_CHECK(igraph_modularity(graph, membership, 0, 1, igraph_is_directed(graph), modularity));
+            IGRAPH_CHECK(igraph_modularity(graph, membership, nullptr, 1, igraph_is_directed(graph), modularity));
         }
         if (temperature) {
             *temperature = stoptemp;
@@ -571,7 +565,7 @@ static igraph_error_t igraph_i_community_spinglass_negative(
     igraph_bool_t conn;
     IGRAPH_CHECK(igraph_is_connected(graph, &conn, IGRAPH_WEAK));
     if (!conn) {
-        IGRAPH_ERROR("Cannot work with unconnected graph", IGRAPH_EINVAL);
+        IGRAPH_ERROR("Cannot work with unconnected graph.", IGRAPH_EINVAL);
     }
 
     if (weights && igraph_vector_size(weights) > 0) {
@@ -591,15 +585,12 @@ static igraph_error_t igraph_i_community_spinglass_negative(
     network net;
 
     /* Transform the igraph_t */
-    IGRAPH_CHECK(igraph_i_read_network(graph, weights,
-                                       &net, use_weights, 0));
+    IGRAPH_CHECK(igraph_i_read_network_spinglass(graph, weights,
+                                       &net, use_weights));
 
     bool directed = igraph_is_directed(graph);
 
-    PottsModelN pm(&net, (unsigned int)spins, directed);
-
-    /* initialize the random number generator */
-    RNG_BEGIN();
+    PottsModelN pm(&net, spins, directed);
 
     if ((stoptemp == 0.0) && (starttemp == 0.0)) {
         zeroT = true;
@@ -614,21 +605,15 @@ static igraph_error_t igraph_i_community_spinglass_negative(
     pm.assign_initial_conf(true);
 
     runs = 0;
-    changes = 1;
-    acc = 0;
-    while (changes > 0 && (kT / stoptemp > 1.0 || (zeroT && runs < 150))) {
-
+    while (kT / stoptemp > 1.0 || (zeroT && runs < 150)) {
         IGRAPH_ALLOW_INTERRUPTION();
 
         runs++;
         kT = kT * coolfact;
         acc = pm.HeatBathLookup(gamma, gamma_minus, kT, 50);
         if (acc < (1.0 - 1.0 / double(spins)) * 0.001) {
-            changes = 0;
-        } else {
-            changes = 1;
+            break;
         }
-
     } /* while loop */
 
     /* These are needed, otherwise 'modularity' is not calculated */
@@ -638,12 +623,10 @@ static igraph_error_t igraph_i_community_spinglass_negative(
     IGRAPH_MATRIX_INIT_FINALLY(&normalized_adhesion, 0, 0);
     pm.WriteClusters(modularity, temperature, csize, membership,
                       &adhesion, &normalized_adhesion, &polarization,
-                      kT, d_p, d_n, gamma, gamma_minus);
+                      kT, d_p, d_n);
     igraph_matrix_destroy(&normalized_adhesion);
     igraph_matrix_destroy(&adhesion);
     IGRAPH_FINALLY_CLEAN(2);
 
-    RNG_END();
-
     return IGRAPH_SUCCESS;
 }
