@@ -228,8 +228,8 @@ igraph_error_t igraph_static_fitness_game(igraph_t *graph, igraph_int_t no_of_ed
                 continue;
             }
 
-            igraph_vector_int_push_back(&edges, from);
-            igraph_vector_int_push_back(&edges, to);
+            igraph_vector_int_push_back(&edges, from); /* reserved */
+            igraph_vector_int_push_back(&edges, to); /* reserved */
 
             no_of_edges--;
         }
