@@ -1,6 +1,5 @@
-/* -*- mode: C -*-  */
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2005-2012  Gabor Csardi <csardi.gabor@gmail.com>
    334 Harvard street, Cambridge, MA 02139 USA
 
@@ -145,17 +144,32 @@ igraph_vs_t igraph_vss_all(void) {
  *          from/to \c vid. That is, all the neighbors of \c vid considered
  *          as if the graph is undirected.
  *        \endclist
+ * \param loops Whether to include the vertex itself in the neighborhood if the
+ *        vertex has a loop edge. If \c IGRAPH_NO_LOOPS, loop edges are
+ *        excluded. If \c IGRAPH_LOOPS_ONCE, the vertex is included in its own
+ *        neighborhood once for every loop edge that it has. If
+ *        \c IGRAPH_LOOPS_TWICE, the vertex is included twice in its own
+ *        neighborhood for every loop edge that it has, but only if the graph is
+ *        undirected or \p mode is set to \c IGRAPH_ALL.
+ * \param multiple Whether to include multiple edges. If \c IGRAPH_NO_MULTIPLE,
+ *        multiple edges are not included in the neighborhood. If
+ *        \c IGRAPH_MULTIPLE, multiple edges are included in the neighborhood.
+ *
  * \return Error code.
  * \sa \ref igraph_vs_destroy()
  *
  * Time complexity: O(1).
  */
 
-igraph_error_t igraph_vs_adj(igraph_vs_t *vs,
-                  igraph_integer_t vid, igraph_neimode_t mode) {
+igraph_error_t igraph_vs_adj(
+    igraph_vs_t *vs, igraph_int_t vid, igraph_neimode_t mode,
+    igraph_loops_t loops, igraph_bool_t multiple
+) {
     vs->type = IGRAPH_VS_ADJ;
     vs->data.adj.vid = vid;
     vs->data.adj.mode = mode;
+    vs->data.adj.loops = loops;
+    vs->data.adj.multiple = multiple;
     return IGRAPH_SUCCESS;
 }
 
@@ -196,11 +210,13 @@ igraph_error_t igraph_vs_adj(igraph_vs_t *vs,
  * \example examples/simple/igraph_vs_nonadj.c
  */
 
-igraph_error_t igraph_vs_nonadj(igraph_vs_t *vs, igraph_integer_t vid,
+igraph_error_t igraph_vs_nonadj(igraph_vs_t *vs, igraph_int_t vid,
                      igraph_neimode_t mode) {
     vs->type = IGRAPH_VS_NONADJ;
     vs->data.adj.vid = vid;
     vs->data.adj.mode = mode;
+    vs->data.adj.loops = IGRAPH_LOOPS;
+    vs->data.adj.multiple = IGRAPH_MULTIPLE;
     return IGRAPH_SUCCESS;
 }
 
@@ -254,7 +270,7 @@ igraph_vs_t igraph_vss_none(void) {
  * Time complexity: O(1).
  */
 
-igraph_error_t igraph_vs_1(igraph_vs_t *vs, igraph_integer_t vid) {
+igraph_error_t igraph_vs_1(igraph_vs_t *vs, igraph_int_t vid) {
     vs->type = IGRAPH_VS_1;
     vs->data.vid = vid;
     return IGRAPH_SUCCESS;
@@ -273,7 +289,7 @@ igraph_error_t igraph_vs_1(igraph_vs_t *vs, igraph_integer_t vid) {
  * Time complexity: O(1).
  */
 
-igraph_vs_t igraph_vss_1(igraph_integer_t vid) {
+igraph_vs_t igraph_vss_1(igraph_int_t vid) {
     igraph_vs_t onevs;
     onevs.type = IGRAPH_VS_1;
     onevs.data.vid = vid;
@@ -358,7 +374,7 @@ igraph_vs_t igraph_vss_vector(const igraph_vector_int_t *v) {
 
 igraph_error_t igraph_vs_vector_small(igraph_vs_t *vs, ...) {
     va_list ap;
-    igraph_integer_t i, n = 0;
+    igraph_int_t i, n = 0;
     igraph_vector_int_t* vec;
 
     vec = IGRAPH_CALLOC(1, igraph_vector_int_t);
@@ -444,10 +460,10 @@ igraph_error_t igraph_vs_vector_copy(igraph_vs_t *vs, const igraph_vector_int_t
  *
  * Time complexity: O(1).
  *
- * \example examples/simple/igraph_vs_seq.c
+ * \example examples/simple/igraph_vs_range.c
  */
 
-igraph_error_t igraph_vs_range(igraph_vs_t *vs, igraph_integer_t start, igraph_integer_t end) {
+igraph_error_t igraph_vs_range(igraph_vs_t *vs, igraph_int_t start, igraph_int_t end) {
     *vs = igraph_vss_range(start, end);
     return IGRAPH_SUCCESS;
 }
@@ -466,7 +482,7 @@ igraph_error_t igraph_vs_range(igraph_vs_t *vs, igraph_integer_t start, igraph_i
  * Time complexity: O(1).
  */
 
-igraph_vs_t igraph_vss_range(igraph_integer_t start, igraph_integer_t end) {
+igraph_vs_t igraph_vss_range(igraph_int_t start, igraph_int_t end) {
     igraph_vs_t vs;
     vs.type = IGRAPH_VS_RANGE;
     vs.data.range.start = start;
@@ -474,52 +490,6 @@ igraph_vs_t igraph_vss_range(igraph_integer_t start, igraph_integer_t end) {
     return vs;
 }
 
-/**
- * \function igraph_vs_seq
- * \brief Vertex set, an interval of vertices with inclusive endpoints (deprecated).
- *
- * Creates a vertex selector containing all vertices with vertex ID
- * equal to or bigger than \p from and equal to or smaller than \p to.
- * Note that both endpoints are inclusive, contrary to C conventions.
- *
- * \deprecated-by igraph_vs_range 0.10.0
- *
- * \param vs Pointer to an uninitialized vertex selector object.
- * \param from The first vertex ID to be included in the vertex selector.
- * \param to The last vertex ID to be included in the vertex selector.
- * \return Error code.
- * \sa \ref igraph_vs_range(), \ref igraph_vss_seq(), \ref igraph_vs_destroy()
- *
- * Time complexity: O(1).
- *
- * \example examples/simple/igraph_vs_seq.c
- */
-
-igraph_error_t igraph_vs_seq(igraph_vs_t *vs, igraph_integer_t from, igraph_integer_t to) {
-    *vs = igraph_vss_seq(from, to);
-    return IGRAPH_SUCCESS;
-}
-
-/**
- * \function igraph_vss_seq
- * \brief An interval of vertices with inclusive endpoints (immediate version, deprecated).
- *
- * The immediate version of \ref igraph_vs_seq().
- *
- * \deprecated-by igraph_vss_range 0.10.0
- *
- * \param from The first vertex ID to be included in the vertex selector.
- * \param to The last vertex ID to be included in the vertex selector.
- * \return Error code.
- * \sa \ref igraph_vss_range(), \ref igraph_vs_seq()
- *
- * Time complexity: O(1).
- */
-
-igraph_vs_t igraph_vss_seq(igraph_integer_t from, igraph_integer_t to) {
-    return igraph_vss_range(from, to + 1);
-}
-
 /**
  * \function igraph_vs_destroy
  * \brief Destroy a vertex set.
@@ -592,8 +562,9 @@ igraph_error_t igraph_vs_as_vector(const igraph_t *graph, igraph_vs_t vs,
  * \function igraph_vs_copy
  * \brief Creates a copy of a vertex selector.
  *
- * \param src The selector being copied.
  * \param dest An uninitialized selector that will contain the copy.
+ * \param src The selector being copied.
+ * \return Error code.
  */
 igraph_error_t igraph_vs_copy(igraph_vs_t* dest, const igraph_vs_t* src) {
     igraph_vector_int_t *vec;
@@ -632,14 +603,16 @@ igraph_vs_type_t igraph_vs_type(const igraph_vs_t *vs) {
  * yield when it is iterated over.
  *
  * \param graph The graph over which we will iterate.
+ * \param vs the vertex selector.
  * \param result The result will be returned here.
+ * \return Error code.
  */
 igraph_error_t igraph_vs_size(const igraph_t *graph, const igraph_vs_t *vs,
-                   igraph_integer_t *result) {
+                   igraph_int_t *result) {
     igraph_vector_int_t vec;
     igraph_bool_t *seen;
-    igraph_integer_t i;
-    igraph_integer_t vec_len;
+    igraph_int_t i;
+    igraph_int_t vec_len;
 
     switch (vs->type) {
     case IGRAPH_VS_NONE:
@@ -661,7 +634,10 @@ igraph_error_t igraph_vs_size(const igraph_t *graph, const igraph_vs_t *vs,
 
     case IGRAPH_VS_ADJ:
         IGRAPH_VECTOR_INT_INIT_FINALLY(&vec, 0);
-        IGRAPH_CHECK(igraph_neighbors(graph, &vec, vs->data.adj.vid, vs->data.adj.mode));
+        IGRAPH_CHECK(igraph_neighbors(
+            graph, &vec, vs->data.adj.vid, vs->data.adj.mode,
+            vs->data.adj.loops, vs->data.adj.multiple
+        ));
         *result = igraph_vector_int_size(&vec);
         igraph_vector_int_destroy(&vec);
         IGRAPH_FINALLY_CLEAN(1);
@@ -669,7 +645,10 @@ igraph_error_t igraph_vs_size(const igraph_t *graph, const igraph_vs_t *vs,
 
     case IGRAPH_VS_NONADJ:
         IGRAPH_VECTOR_INT_INIT_FINALLY(&vec, 0);
-        IGRAPH_CHECK(igraph_neighbors(graph, &vec, vs->data.adj.vid, vs->data.adj.mode));
+        IGRAPH_CHECK(igraph_neighbors(
+            graph, &vec, vs->data.adj.vid, vs->data.adj.mode,
+            vs->data.adj.loops, vs->data.adj.multiple
+        ));
         vec_len = igraph_vector_int_size(&vec);
         *result = igraph_vcount(graph);
         seen = IGRAPH_CALLOC(*result, igraph_bool_t);
@@ -678,7 +657,7 @@ igraph_error_t igraph_vs_size(const igraph_t *graph, const igraph_vs_t *vs,
         for (i = 0; i < vec_len; i++) {
             if (!seen[ VECTOR(vec)[i] ]) {
                 (*result)--;
-                seen[ VECTOR(vec)[i] ] = 1;
+                seen[ VECTOR(vec)[i] ] = true;
             }
         }
         igraph_free(seen);
@@ -734,8 +713,8 @@ igraph_error_t igraph_vit_create(const igraph_t *graph, igraph_vs_t vs, igraph_v
     igraph_vector_int_t vec;
     igraph_vector_int_t *vec_int;
     igraph_bool_t *seen;
-    igraph_integer_t i, j, n;
-    igraph_integer_t vec_len;
+    igraph_int_t i, j, n;
+    igraph_int_t vec_len;
 
     switch (vs.type) {
     case IGRAPH_VS_ALL:
@@ -750,7 +729,10 @@ igraph_error_t igraph_vit_create(const igraph_t *graph, igraph_vs_t vs, igraph_v
         IGRAPH_FINALLY(igraph_free, vec_int);
         IGRAPH_VECTOR_INT_INIT_FINALLY(vec_int, 0);
         IGRAPH_VECTOR_INT_INIT_FINALLY(&vec, 0);
-        IGRAPH_CHECK(igraph_neighbors(graph, &vec, vs.data.adj.vid, vs.data.adj.mode));
+        IGRAPH_CHECK(igraph_neighbors(
+            graph, &vec, vs.data.adj.vid, vs.data.adj.mode,
+            vs.data.adj.loops, vs.data.adj.multiple
+        ));
         n = igraph_vector_int_size(&vec);
         IGRAPH_CHECK(igraph_vector_int_resize(vec_int, n));
         for (i = 0; i < n; i++) {
@@ -773,7 +755,10 @@ igraph_error_t igraph_vit_create(const igraph_t *graph, igraph_vs_t vs, igraph_v
         IGRAPH_FINALLY(igraph_free, vec_int);
         IGRAPH_VECTOR_INT_INIT_FINALLY(vec_int, 0);
         IGRAPH_VECTOR_INT_INIT_FINALLY(&vec, 0);
-        IGRAPH_CHECK(igraph_neighbors(graph, &vec, vs.data.adj.vid, vs.data.adj.mode));
+        IGRAPH_CHECK(igraph_neighbors(
+            graph, &vec, vs.data.adj.vid, vs.data.adj.mode,
+            vs.data.adj.loops, vs.data.adj.multiple
+        ));
         vec_len = igraph_vector_int_size(&vec);
         n = igraph_vcount(graph);
         seen = IGRAPH_CALLOC(n, igraph_bool_t);
@@ -782,7 +767,7 @@ igraph_error_t igraph_vit_create(const igraph_t *graph, igraph_vs_t vs, igraph_v
         for (i = 0; i < vec_len; i++) {
             if (! seen [ VECTOR(vec)[i] ] ) {
                 n--;
-                seen[ VECTOR(vec)[i] ] = 1;
+                seen[ VECTOR(vec)[i] ] = true;
             }
         }
         IGRAPH_CHECK(igraph_vector_int_resize(vec_int, n));
@@ -830,7 +815,7 @@ igraph_error_t igraph_vit_create(const igraph_t *graph, igraph_vs_t vs, igraph_v
         break;
     case IGRAPH_VS_RANGE:
         {
-            igraph_integer_t no_of_nodes = igraph_vcount(graph);
+            igraph_int_t no_of_nodes = igraph_vcount(graph);
             if (vs.data.range.start < 0 ||
                 vs.data.range.start > no_of_nodes ||
                 (no_of_nodes > 0 && vs.data.range.start == no_of_nodes)) {
@@ -881,7 +866,7 @@ void igraph_vit_destroy(const igraph_vit_t *vit) {
 }
 
 igraph_error_t igraph_vit_as_vector(const igraph_vit_t *vit, igraph_vector_int_t *v) {
-    igraph_integer_t i;
+    igraph_int_t i;
 
     IGRAPH_CHECK(igraph_vector_int_resize(v, IGRAPH_VIT_SIZE(*vit)));
 
@@ -915,13 +900,16 @@ igraph_error_t igraph_vit_as_vector(const igraph_vit_t *vit, igraph_vector_int_t
  * \param es Pointer to an uninitialized edge selector object.
  * \param order Constant giving the order in which the edges will be
  *        included in the selector. Possible values:
- *        \c IGRAPH_EDGEORDER_ID, edge ID order.
- *        \c IGRAPH_EDGEORDER_FROM, vertex ID order, the id of the
- *           \em source vertex counts for directed graphs. The order
- *           of the incident edges of a given vertex is arbitrary.
- *        \c IGRAPH_EDGEORDER_TO, vertex ID order, the ID of the \em
- *           target vertex counts for directed graphs. The order
- *           of the incident edges of a given vertex is arbitrary.
+ *        \clist
+ *        \cli IGRAPH_EDGEORDER_ID
+ *        Edge ID order; currently performs the fastest.
+ *        \cli IGRAPH_EDGEORDER_FROM
+ *        Vertex ID order, the id of the \em source vertex counts for directed
+ *        graphs. The order of the incident edges of a given vertex is arbitrary.
+ *        \cli IGRAPH_EDGEORDER_TO
+ *        Vertex ID order, the ID of the \em target vertex counts for directed
+ *        graphs. The order of the incident edges of a given vertex is arbitrary.
+ *        \endclist
  *        For undirected graph the latter two is the same.
  * \return Error code.
  * \sa \ref igraph_ess_all(), \ref igraph_es_destroy()
@@ -980,17 +968,25 @@ igraph_es_t igraph_ess_all(igraph_edgeorder_type_t order) {
  *        \c IGRAPH_OUT, outgoing edges;
  *        \c IGRAPH_IN, incoming edges;
  *        \c IGRAPH_ALL, all edges.
+ * \param loops Whether to include loop edges in the result. If
+ *        \c IGRAPH_NO_LOOPS, loop edges are excluded. If \c IGRAPH_LOOPS_ONCE,
+ *        loop edges are included once. If \c IGRAPH_LOOPS_TWICE, loop edges
+ *        are included twice, but only if the graph is undirected or \p mode is
+ *        set to \c IGRAPH_ALL.
  * \return Error code.
  * \sa \ref igraph_es_destroy()
  *
  * Time complexity: O(1).
  */
 
-igraph_error_t igraph_es_incident(igraph_es_t *es,
-                       igraph_integer_t vid, igraph_neimode_t mode) {
+igraph_error_t igraph_es_incident(
+    igraph_es_t *es, igraph_int_t vid, igraph_neimode_t mode,
+    igraph_loops_t loops
+) {
     es->type = IGRAPH_ES_INCIDENT;
     es->data.incident.vid = vid;
     es->data.incident.mode = mode;
+    es->data.incident.loops = loops;
     return IGRAPH_SUCCESS;
 }
 
@@ -1042,7 +1038,7 @@ igraph_es_t igraph_ess_none(void) {
  * Time complexity: O(1).
  */
 
-igraph_error_t igraph_es_1(igraph_es_t *es, igraph_integer_t eid) {
+igraph_error_t igraph_es_1(igraph_es_t *es, igraph_int_t eid) {
     es->type = IGRAPH_ES_1;
     es->data.eid = eid;
     return IGRAPH_SUCCESS;
@@ -1059,7 +1055,7 @@ igraph_error_t igraph_es_1(igraph_es_t *es, igraph_integer_t eid) {
  * Time complexity: O(1).
  */
 
-igraph_es_t igraph_ess_1(igraph_integer_t eid) {
+igraph_es_t igraph_ess_1(igraph_int_t eid) {
     igraph_es_t es;
     es.type = IGRAPH_ES_1;
     es.data.eid = eid;
@@ -1156,7 +1152,7 @@ igraph_es_t igraph_ess_vector(const igraph_vector_int_t *v) {
  * interval is closed from the left and open from the right, following C
  * conventions.
  *
- * \param vs Pointer to an uninitialized edge selector object.
+ * \param es Pointer to an uninitialized edge selector object.
  * \param start The first edge ID to be included in the edge selector.
  * \param end The first edge ID \em not to be included in the edge selector.
  * \return Error code.
@@ -1165,7 +1161,7 @@ igraph_es_t igraph_ess_vector(const igraph_vector_int_t *v) {
  * Time complexity: O(1).
  */
 
-igraph_error_t igraph_es_range(igraph_es_t *es, igraph_integer_t start, igraph_integer_t end) {
+igraph_error_t igraph_es_range(igraph_es_t *es, igraph_int_t start, igraph_int_t end) {
     *es = igraph_ess_range(start, end);
     return IGRAPH_SUCCESS;
 }
@@ -1182,7 +1178,7 @@ igraph_error_t igraph_es_range(igraph_es_t *es, igraph_integer_t start, igraph_i
  * Time complexity: O(1).
  */
 
-igraph_es_t igraph_ess_range(igraph_integer_t start, igraph_integer_t end) {
+igraph_es_t igraph_ess_range(igraph_int_t start, igraph_int_t end) {
     igraph_es_t es;
     es.type = IGRAPH_ES_RANGE;
     es.data.range.start = start;
@@ -1190,47 +1186,6 @@ igraph_es_t igraph_ess_range(igraph_integer_t start, igraph_integer_t end) {
     return es;
 }
 
-/**
- * \function igraph_es_seq
- * \brief Edge selector, a sequence of edge IDs, with inclusive endpoints (deprecated).
- *
- * All edge IDs between \p from and \p to (inclusive) will be
- * included in the edge selection.
- *
- * \deprecated-by igraph_es_range 0.10.0
- *
- * \param es Pointer to an uninitialized edge selector object.
- * \param from The first edge ID to be included.
- * \param to The last edge ID to be included.
- * \return Error code.
- * \sa \ref igraph_ess_seq(), \ref igraph_es_destroy()
- *
- * Time complexity: O(1).
- */
-
-igraph_error_t igraph_es_seq(igraph_es_t *es, igraph_integer_t from, igraph_integer_t to) {
-    *es = igraph_ess_seq(from, to);
-    return IGRAPH_SUCCESS;
-}
-
-/**
- * \function igraph_ess_seq
- * \brief Immediate version of the sequence edge selector, with inclusive endpoints.
- *
- * \deprecated-by igraph_ess_range 0.10.0
- *
- * \param from The first edge ID to include.
- * \param to The last edge ID to include.
- * \return The initialized edge selector.
- * \sa \ref igraph_es_seq()
- *
- * Time complexity: O(1).
- */
-
-igraph_es_t igraph_ess_seq(igraph_integer_t from, igraph_integer_t to) {
-    return igraph_ess_range(from, to + 1);
-}
-
 /**
  * \function igraph_es_pairs
  * \brief Edge selector, multiple edges defined by their endpoints in a vector.
@@ -1300,7 +1255,7 @@ igraph_error_t igraph_es_pairs(igraph_es_t *es, const igraph_vector_int_t *v,
 
 igraph_error_t igraph_es_pairs_small(igraph_es_t *es, igraph_bool_t directed, int first, ...) {
     va_list ap;
-    igraph_integer_t i, n = 0;
+    igraph_int_t i, n = 0;
     igraph_vector_int_t *vec;
     int num;
 
@@ -1373,7 +1328,7 @@ igraph_error_t igraph_es_path(igraph_es_t *es, const igraph_vector_int_t *v,
 
 igraph_error_t igraph_es_path_small(igraph_es_t *es, igraph_bool_t directed, int first, ...) {
     va_list ap;
-    igraph_integer_t i, n = 0;
+    igraph_int_t i, n = 0;
     igraph_vector_int_t *vec;
     int num;
 
@@ -1419,15 +1374,15 @@ igraph_error_t igraph_es_path_small(igraph_es_t *es, igraph_bool_t directed, int
  * \param es Pointer to an uninitialized edge selector object.
  * \param from The ID of the source vertex.
  * \param to The ID of the target vertex.
- * \param direectd If edge directions should be taken into account. This
+ * \param directed If edge directions should be taken into account. This
  *      will be ignored if the graph to select from is undirected.
  * \return Error code.
  * \sa \ref igraph_es_destroy()
  *
  * Time complexity: O(1).
  */
-IGRAPH_EXPORT igraph_error_t igraph_es_all_between(
-    igraph_es_t *es, igraph_integer_t from, igraph_integer_t to,
+igraph_error_t igraph_es_all_between(
+    igraph_es_t *es, igraph_int_t from, igraph_int_t to,
     igraph_bool_t directed
 ) {
     es->type = IGRAPH_ES_ALL_BETWEEN;
@@ -1494,8 +1449,11 @@ igraph_bool_t igraph_es_is_all(const igraph_es_t *es) {
 /**
  * \function igraph_es_copy
  * \brief Creates a copy of an edge selector.
- * \param src The selector being copied.
+ *
  * \param dest An uninitialized selector that will contain the copy.
+ * \param src The selector being copied.
+ * \return Error code.
+ *
  * \sa \ref igraph_es_destroy()
  */
 igraph_error_t igraph_es_copy(igraph_es_t* dest, const igraph_es_t* src) {
@@ -1538,6 +1496,7 @@ igraph_error_t igraph_es_copy(igraph_es_t* dest, const igraph_es_t* src) {
  * \param graph Pointer to a graph to check if the edges in the selector exist.
  * \param es An edge selector object.
  * \param v Pointer to initialized vector. The result will be stored here.
+ * \return Error code.
  *
  * Time complexity: O(n), the number of edges in the selector.
  */
@@ -1563,11 +1522,11 @@ igraph_es_type_t igraph_es_type(const igraph_es_t *es) {
 }
 
 static igraph_error_t igraph_i_es_pairs_size(const igraph_t *graph,
-                                  const igraph_es_t *es, igraph_integer_t *result);
+                                  const igraph_es_t *es, igraph_int_t *result);
 static igraph_error_t igraph_i_es_path_size(const igraph_t *graph,
-                                 const igraph_es_t *es, igraph_integer_t *result);
+                                 const igraph_es_t *es, igraph_int_t *result);
 static igraph_error_t igraph_i_es_all_between_size(const igraph_t *graph,
-                                 const igraph_es_t *es, igraph_integer_t *result);
+                                 const igraph_es_t *es, igraph_int_t *result);
 
 /**
  * \function igraph_es_size
@@ -1577,10 +1536,12 @@ static igraph_error_t igraph_i_es_all_between_size(const igraph_t *graph,
  * yield when it is iterated over.
  *
  * \param graph The graph over which we will iterate.
+ * \param es The edge selector.
  * \param result The result will be returned here.
+ * \return Error code.
  */
 igraph_error_t igraph_es_size(const igraph_t *graph, const igraph_es_t *es,
-                   igraph_integer_t *result) {
+                   igraph_int_t *result) {
     igraph_vector_int_t v;
 
     switch (es->type) {
@@ -1598,8 +1559,10 @@ igraph_error_t igraph_es_size(const igraph_t *graph, const igraph_es_t *es,
 
     case IGRAPH_ES_INCIDENT:
         IGRAPH_VECTOR_INT_INIT_FINALLY(&v, 0);
-        IGRAPH_CHECK(igraph_incident(graph, &v,
-                                     es->data.incident.vid, es->data.incident.mode));
+        IGRAPH_CHECK(igraph_incident(
+            graph, &v, es->data.incident.vid, es->data.incident.mode,
+            es->data.incident.loops
+        ));
         *result = igraph_vector_int_size(&v);
         igraph_vector_int_destroy(&v);
         IGRAPH_FINALLY_CLEAN(1);
@@ -1645,9 +1608,9 @@ igraph_error_t igraph_es_size(const igraph_t *graph, const igraph_es_t *es,
 }
 
 static igraph_error_t igraph_i_es_pairs_size(const igraph_t *graph,
-                                  const igraph_es_t *es, igraph_integer_t *result) {
-    igraph_integer_t i, n = igraph_vector_int_size(es->data.path.ptr);
-    igraph_integer_t no_of_nodes = igraph_vcount(graph);
+                                  const igraph_es_t *es, igraph_int_t *result) {
+    igraph_int_t i, n = igraph_vector_int_size(es->data.path.ptr);
+    igraph_int_t no_of_nodes = igraph_vcount(graph);
 
     if (n % 2 != 0) {
         IGRAPH_ERROR("Cannot calculate edge selector length from odd number of vertices.",
@@ -1660,9 +1623,9 @@ static igraph_error_t igraph_i_es_pairs_size(const igraph_t *graph,
     *result = n / 2;
     /* Check for the existence of all edges */
     for (i = 0; i < *result; i++) {
-        igraph_integer_t from = VECTOR(*es->data.path.ptr)[2 * i];
-        igraph_integer_t to = VECTOR(*es->data.path.ptr)[2 * i + 1];
-        igraph_integer_t eid;
+        igraph_int_t from = VECTOR(*es->data.path.ptr)[2 * i];
+        igraph_int_t to = VECTOR(*es->data.path.ptr)[2 * i + 1];
+        igraph_int_t eid;
         IGRAPH_CHECK(igraph_get_eid(graph, &eid, from, to, es->data.path.mode,
                                     /*error=*/ 1));
     }
@@ -1671,9 +1634,9 @@ static igraph_error_t igraph_i_es_pairs_size(const igraph_t *graph,
 }
 
 static igraph_error_t igraph_i_es_path_size(const igraph_t *graph,
-                                 const igraph_es_t *es, igraph_integer_t *result) {
-    igraph_integer_t i, n = igraph_vector_int_size(es->data.path.ptr);
-    igraph_integer_t no_of_nodes = igraph_vcount(graph);
+                                 const igraph_es_t *es, igraph_int_t *result) {
+    igraph_int_t i, n = igraph_vector_int_size(es->data.path.ptr);
+    igraph_int_t no_of_nodes = igraph_vcount(graph);
 
     if (!igraph_vector_int_isininterval(es->data.path.ptr, 0, no_of_nodes - 1)) {
         IGRAPH_ERROR("Cannot calculate selector length.", IGRAPH_EINVVID);
@@ -1685,9 +1648,9 @@ static igraph_error_t igraph_i_es_path_size(const igraph_t *graph,
         *result = n - 1;
     }
     for (i = 0; i < *result; i++) {
-        igraph_integer_t from = VECTOR(*es->data.path.ptr)[i];
-        igraph_integer_t to = VECTOR(*es->data.path.ptr)[i + 1];
-        igraph_integer_t eid;
+        igraph_int_t from = VECTOR(*es->data.path.ptr)[i];
+        igraph_int_t to = VECTOR(*es->data.path.ptr)[i + 1];
+        igraph_int_t eid;
         IGRAPH_CHECK(igraph_get_eid(graph, &eid, from, to, es->data.path.mode,
                                     /*error=*/ 1));
     }
@@ -1696,10 +1659,10 @@ static igraph_error_t igraph_i_es_path_size(const igraph_t *graph,
 }
 
 static igraph_error_t igraph_i_es_all_between_size(const igraph_t *graph,
-                                 const igraph_es_t *es, igraph_integer_t *result) {
-    igraph_integer_t no_of_nodes = igraph_vcount(graph);
-    igraph_integer_t from = es->data.between.from;
-    igraph_integer_t to = es->data.between.to;
+                                 const igraph_es_t *es, igraph_int_t *result) {
+    igraph_int_t no_of_nodes = igraph_vcount(graph);
+    igraph_int_t from = es->data.between.from;
+    igraph_int_t to = es->data.between.to;
     igraph_bool_t directed = es->data.between.directed;
     igraph_vector_int_t vec;
 
@@ -1732,41 +1695,39 @@ static igraph_error_t igraph_i_eit_create_allfromto(const igraph_t *graph,
                                          igraph_eit_t *eit,
                                          igraph_neimode_t mode) {
     igraph_vector_int_t *vec;
-    igraph_integer_t no_of_nodes = igraph_vcount(graph);
-    igraph_integer_t no_of_edges = igraph_ecount(graph);
-    igraph_integer_t i, j, length;
+    igraph_int_t no_of_nodes = igraph_vcount(graph);
+    igraph_int_t no_of_edges = igraph_ecount(graph);
 
     vec = IGRAPH_CALLOC(1, igraph_vector_int_t);
     IGRAPH_CHECK_OOM(vec, "Cannot create edge iterator.");
     IGRAPH_FINALLY(igraph_free, vec);
+
     IGRAPH_VECTOR_INT_INIT_FINALLY(vec, 0);
     IGRAPH_CHECK(igraph_vector_int_reserve(vec, no_of_edges));
 
     if (igraph_is_directed(graph)) {
         igraph_vector_int_t adj;
         IGRAPH_VECTOR_INT_INIT_FINALLY(&adj, 0);
-        for (i = 0; i < no_of_nodes; i++) {
-            IGRAPH_CHECK(igraph_incident(graph, &adj, i, mode));
+        for (igraph_int_t i = 0; i < no_of_nodes; i++) {
+            IGRAPH_CHECK(igraph_incident(graph, &adj, i, mode, IGRAPH_LOOPS));
             igraph_vector_int_append(vec, &adj);  /* reserved */
         }
         igraph_vector_int_destroy(&adj);
         IGRAPH_FINALLY_CLEAN(1);
-
     } else {
-
         igraph_vector_int_t adj;
         igraph_bool_t *added;
         IGRAPH_VECTOR_INT_INIT_FINALLY(&adj, 0);
         added = IGRAPH_CALLOC(no_of_edges, igraph_bool_t);
         IGRAPH_CHECK_OOM(added, "Cannot create edge iterator.");
         IGRAPH_FINALLY(igraph_free, added);
-        for (i = 0; i < no_of_nodes; i++) {
-            IGRAPH_CHECK(igraph_incident(graph, &adj, i, IGRAPH_ALL));
-            length = igraph_vector_int_size(&adj);
-            for (j = 0; j < length; j++) {
+        for (igraph_int_t i = 0; i < no_of_nodes; i++) {
+            IGRAPH_CHECK(igraph_incident(graph, &adj, i, IGRAPH_ALL, IGRAPH_LOOPS));
+            const igraph_int_t length = igraph_vector_int_size(&adj);
+            for (igraph_int_t j = 0; j < length; j++) {
                 if (!added[ VECTOR(adj)[j] ]) {
                     igraph_vector_int_push_back(vec, VECTOR(adj)[j]);  /* reserved */
-                    added[ VECTOR(adj)[j] ] += 1;
+                    added[ VECTOR(adj)[j] ] = true;
                 }
             }
         }
@@ -1789,10 +1750,13 @@ static igraph_error_t igraph_i_eit_create_incident(const igraph_t* graph,
                               igraph_es_t es, igraph_eit_t *eit) {
     igraph_vector_int_t vec;
     igraph_vector_int_t* vec_int;
-    igraph_integer_t i, n;
+    igraph_int_t i, n;
 
     IGRAPH_VECTOR_INT_INIT_FINALLY(&vec, 0);
-    IGRAPH_CHECK(igraph_incident(graph, &vec, es.data.incident.vid, es.data.incident.mode));
+    IGRAPH_CHECK(igraph_incident(
+        graph, &vec, es.data.incident.vid, es.data.incident.mode,
+        es.data.incident.loops
+    ));
 
     vec_int = IGRAPH_CALLOC(1, igraph_vector_int_t);
     IGRAPH_CHECK_OOM(vec_int, "Cannot create edge iterator.");
@@ -1819,9 +1783,9 @@ static igraph_error_t igraph_i_eit_create_incident(const igraph_t* graph,
 
 static igraph_error_t igraph_i_eit_pairs(const igraph_t *graph,
                               igraph_es_t es, igraph_eit_t *eit) {
-    igraph_integer_t n = igraph_vector_int_size(es.data.path.ptr);
-    igraph_integer_t no_of_nodes = igraph_vcount(graph);
-    igraph_integer_t i;
+    igraph_int_t n = igraph_vector_int_size(es.data.path.ptr);
+    igraph_int_t no_of_nodes = igraph_vcount(graph);
+    igraph_int_t i;
     igraph_vector_int_t* vec;
 
     if (n % 2 != 0) {
@@ -1838,9 +1802,9 @@ static igraph_error_t igraph_i_eit_pairs(const igraph_t *graph,
     IGRAPH_VECTOR_INT_INIT_FINALLY(vec, n / 2);
 
     for (i = 0; i < n / 2; i++) {
-        igraph_integer_t from = VECTOR(*es.data.path.ptr)[2 * i];
-        igraph_integer_t to = VECTOR(*es.data.path.ptr)[2 * i + 1];
-        igraph_integer_t eid;
+        igraph_int_t from = VECTOR(*es.data.path.ptr)[2 * i];
+        igraph_int_t to = VECTOR(*es.data.path.ptr)[2 * i + 1];
+        igraph_int_t eid;
         IGRAPH_CHECK(igraph_get_eid(graph, &eid, from, to, es.data.path.mode,
                                     /*error=*/ 1));
         VECTOR(*vec)[i] = eid;
@@ -1859,9 +1823,9 @@ static igraph_error_t igraph_i_eit_pairs(const igraph_t *graph,
 
 static igraph_error_t igraph_i_eit_path(const igraph_t *graph,
                              igraph_es_t es, igraph_eit_t *eit) {
-    igraph_integer_t n = igraph_vector_int_size(es.data.path.ptr);
-    igraph_integer_t no_of_nodes = igraph_vcount(graph);
-    igraph_integer_t i, len;
+    igraph_int_t n = igraph_vector_int_size(es.data.path.ptr);
+    igraph_int_t no_of_nodes = igraph_vcount(graph);
+    igraph_int_t i, len;
     igraph_vector_int_t* vec;
 
     if (!igraph_vector_int_isininterval(es.data.path.ptr, 0, no_of_nodes - 1)) {
@@ -1881,9 +1845,9 @@ static igraph_error_t igraph_i_eit_path(const igraph_t *graph,
     IGRAPH_VECTOR_INT_INIT_FINALLY(vec, len);
 
     for (i = 0; i < len; i++) {
-        igraph_integer_t from = VECTOR(*es.data.path.ptr)[i];
-        igraph_integer_t to = VECTOR(*es.data.path.ptr)[i + 1];
-        igraph_integer_t eid;
+        igraph_int_t from = VECTOR(*es.data.path.ptr)[i];
+        igraph_int_t to = VECTOR(*es.data.path.ptr)[i + 1];
+        igraph_int_t eid;
         IGRAPH_CHECK(igraph_get_eid(graph, &eid, from, to, es.data.path.mode,
                                     /*error=*/ 1));
         VECTOR(*vec)[i] = eid;
@@ -1903,10 +1867,10 @@ static igraph_error_t igraph_i_eit_path(const igraph_t *graph,
 static igraph_error_t igraph_i_eit_all_between(
     const igraph_t *graph, igraph_es_t es, igraph_eit_t *eit
 ) {
-    igraph_integer_t from = es.data.between.from;
-    igraph_integer_t to = es.data.between.to;
+    igraph_int_t from = es.data.between.from;
+    igraph_int_t to = es.data.between.to;
     igraph_bool_t directed = es.data.between.directed;
-    igraph_integer_t no_of_nodes = igraph_vcount(graph);
+    igraph_int_t no_of_nodes = igraph_vcount(graph);
     igraph_vector_int_t* vec;
 
     if (from < 0 || from >= no_of_nodes || to < 0 || to >= no_of_nodes) {
@@ -1950,7 +1914,7 @@ static igraph_error_t igraph_i_eit_all_between(
  *
  * Time complexity: depends on the type of the edge selector. For edge
  * selectors created by \ref igraph_es_all(), \ref igraph_es_none(),
- * \ref igraph_es_1(), \ref igraph_es_vector(), \ref igraph_es_seq() it is
+ * \ref igraph_es_1(), \ref igraph_es_vector(), \ref igraph_es_range() it is
  * O(1). For \ref igraph_es_incident() it is O(d) where d is the number of
  * incident edges of the vertex.
  */
@@ -1984,7 +1948,7 @@ igraph_error_t igraph_eit_create(const igraph_t *graph, igraph_es_t es, igraph_e
         eit->start = es.data.eid;
         eit->end = es.data.eid + 1;
         if (eit->pos >= igraph_ecount(graph)) {
-            IGRAPH_ERROR("Cannot create iterator, invalid edge ID.", IGRAPH_EINVAL);
+            IGRAPH_ERROR("Cannot create iterator.", IGRAPH_EINVEID);
         }
         break;
     case IGRAPH_ES_VECTOR:
@@ -1995,19 +1959,19 @@ igraph_error_t igraph_eit_create(const igraph_t *graph, igraph_es_t es, igraph_e
         eit->vec = es.data.vecptr;
         eit->end = igraph_vector_int_size(eit->vec);
         if (!igraph_vector_int_isininterval(eit->vec, 0, igraph_ecount(graph) - 1)) {
-            IGRAPH_ERROR("Cannot create iterator, invalid edge ID.", IGRAPH_EINVAL);
+            IGRAPH_ERROR("Cannot create iterator.", IGRAPH_EINVEID);
         }
         break;
     case IGRAPH_ES_RANGE:
         {
-            igraph_integer_t no_of_edges = igraph_ecount(graph);
+            igraph_int_t no_of_edges = igraph_ecount(graph);
             if (es.data.range.start < 0 ||
                 es.data.range.start > no_of_edges ||
                 (no_of_edges > 0 && es.data.range.start == no_of_edges)) {
-                IGRAPH_ERROR("Cannot create range iterator, starting edge ID out of range.", IGRAPH_EINVAL);
+                IGRAPH_ERROR("Cannot create range iterator, starting edge ID out of range.", IGRAPH_EINVEID);
             }
             if (es.data.range.end < 0 || es.data.range.end > no_of_edges) {
-                IGRAPH_ERROR("Cannot create range iterator, ending edge ID out of range.", IGRAPH_EINVAL);
+                IGRAPH_ERROR("Cannot create range iterator, ending edge ID out of range.", IGRAPH_EINVEID);
             }
         }
         eit->type = IGRAPH_EIT_RANGE;
@@ -2058,7 +2022,7 @@ void igraph_eit_destroy(const igraph_eit_t *eit) {
 
 igraph_error_t igraph_eit_as_vector(const igraph_eit_t *eit, igraph_vector_int_t *v) {
 
-    igraph_integer_t i;
+    igraph_int_t i;
 
     IGRAPH_CHECK(igraph_vector_int_resize(v, IGRAPH_EIT_SIZE(*eit)));
 
