@@ -1,8 +1,6 @@
-/* -*- mode: C -*-  */
 /*
-   IGraph R package.
-   Copyright (C) 2006-2012  Gabor Csardi <csardi.gabor@gmail.com>
-   334 Harvard street, Cambridge, MA 02139 USA
+   igraph library.
+   Copyright (C) 2006-2023  The igraph development team <igraph@igraph.org>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,22 +13,20 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301 USA
-
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "igraph_visitor.h"
-#include "igraph_memory.h"
+
 #include "igraph_adjlist.h"
+#include "igraph_bitset.h"
 #include "igraph_interface.h"
 #include "igraph_dqueue.h"
 #include "igraph_stack.h"
 
 /**
  * \function igraph_bfs
- * Breadth-first search
+ * \brief Breadth-first search.
  *
  * A simple breadth-first search, with a lot of different results and
  * the possibility to call a callback whenever a vertex is visited.
@@ -60,7 +56,7 @@
  *        \c IGRAPH_IN means the opposite, and
  *        \c IGRAPH_ALL ignores the direction of the edges.
  *        This parameter is ignored for undirected graphs.
- * \param unreachable Logical scalar, whether the search should visit
+ * \param unreachable Boolean, whether the search should visit
  *        the vertices that are unreachable from the given root
  *        node(s). If true, then additional searches are performed
  *        until all vertices are visited.
@@ -106,7 +102,7 @@
  * \example examples/simple/igraph_bfs_callback.c
  */
 igraph_error_t igraph_bfs(const igraph_t *graph,
-               igraph_integer_t root, const igraph_vector_int_t *roots,
+               igraph_int_t root, const igraph_vector_int_t *roots,
                igraph_neimode_t mode, igraph_bool_t unreachable,
                const igraph_vector_int_t *restricted,
                igraph_vector_int_t *order, igraph_vector_int_t *rank,
@@ -115,20 +111,21 @@ igraph_error_t igraph_bfs(const igraph_t *graph,
                igraph_vector_int_t *dist, igraph_bfshandler_t *callback,
                void *extra) {
 
+    const igraph_int_t no_of_nodes = igraph_vcount(graph);
+
     igraph_error_t ret;
 
     igraph_dqueue_int_t Q;
-    igraph_integer_t no_of_nodes = igraph_vcount(graph);
-    igraph_integer_t actroot = 0;
-    igraph_vector_bool_t added;
+    igraph_int_t actroot = 0;
+    igraph_bitset_t added;
 
     igraph_lazy_adjlist_t adjlist;
 
-    igraph_integer_t act_rank = 0;
-    igraph_integer_t pred_vec = -1;
+    igraph_int_t act_rank = 0;
+    igraph_int_t pred_vec = -1;
 
-    igraph_integer_t rootpos = 0;
-    igraph_integer_t noroots = roots ? igraph_vector_int_size(roots) : 1;
+    igraph_int_t rootpos = 0;
+    igraph_int_t noroots = roots ? igraph_vector_int_size(roots) : 1;
 
     if (!roots && (root < 0 || root >= no_of_nodes)) {
         IGRAPH_ERROR("Invalid root vertex in BFS.", IGRAPH_EINVVID);
@@ -151,7 +148,7 @@ igraph_error_t igraph_bfs(const igraph_t *graph,
         mode = IGRAPH_ALL;
     }
 
-    IGRAPH_VECTOR_BOOL_INIT_FINALLY(&added, no_of_nodes);
+    IGRAPH_BITSET_INIT_FINALLY(&added, no_of_nodes);
     IGRAPH_DQUEUE_INT_INIT_FINALLY(&Q, 100);
 
     IGRAPH_CHECK(igraph_lazy_adjlist_init(graph, &adjlist, mode, IGRAPH_LOOPS, IGRAPH_MULTIPLE));
@@ -161,15 +158,15 @@ igraph_error_t igraph_bfs(const igraph_t *graph,
        found. Special care must be taken for vertices that are not in
        the restricted set, but are to be used as 'root' vertices. */
     if (restricted) {
-        igraph_integer_t i, n = igraph_vector_int_size(restricted);
-        igraph_vector_bool_fill(&added, true);
+        igraph_int_t i, n = igraph_vector_int_size(restricted);
+        igraph_bitset_fill(&added, true);
         for (i = 0; i < n; i++) {
-            igraph_integer_t v = VECTOR(*restricted)[i];
-            VECTOR(added)[v] = false;
+            igraph_int_t v = VECTOR(*restricted)[i];
+            IGRAPH_BIT_CLEAR(added, v);
         }
     }
 
-    /* Resize result vectors, and fill them with IGRAPH_NAN */
+    /* Resize result vectors, and fill them with the initial value. */
 
 # define VINIT(v, initial) \
     if (v) { \
@@ -210,12 +207,12 @@ igraph_error_t igraph_bfs(const igraph_t *graph,
         }
 
         /* OK, we have a new root, start BFS */
-        if (VECTOR(added)[actroot]) {
+        if (IGRAPH_BIT_TEST(added, actroot)) {
             continue;
         }
         IGRAPH_CHECK(igraph_dqueue_int_push(&Q, actroot));
         IGRAPH_CHECK(igraph_dqueue_int_push(&Q, 0));
-        VECTOR(added)[actroot] = true;
+        IGRAPH_BIT_SET(added, actroot);
         if (parents) {
             VECTOR(*parents)[actroot] = -1;
         }
@@ -223,14 +220,13 @@ igraph_error_t igraph_bfs(const igraph_t *graph,
         pred_vec = -1;
 
         while (!igraph_dqueue_int_empty(&Q)) {
-            igraph_integer_t actvect = igraph_dqueue_int_pop(&Q);
-            igraph_integer_t actdist = igraph_dqueue_int_pop(&Q);
-            igraph_integer_t succ_vec;
+            igraph_int_t actvect = igraph_dqueue_int_pop(&Q);
+            igraph_int_t actdist = igraph_dqueue_int_pop(&Q);
+            igraph_int_t succ_vec;
             igraph_vector_int_t *neis = igraph_lazy_adjlist_get(&adjlist, actvect);
-            igraph_integer_t i, n;
 
             IGRAPH_CHECK_OOM(neis, "Failed to query neighbors.");
-            n = igraph_vector_int_size(neis);
+            const igraph_int_t n = igraph_vector_int_size(neis);
 
             if (pred) {
                 VECTOR(*pred)[actvect] = pred_vec;
@@ -245,10 +241,10 @@ igraph_error_t igraph_bfs(const igraph_t *graph,
                 VECTOR(*dist)[actvect] = actdist;
             }
 
-            for (i = 0; i < n; i++) {
-                igraph_integer_t nei = VECTOR(*neis)[i];
-                if (! VECTOR(added)[nei]) {
-                    VECTOR(added)[nei] = true;
+            for (igraph_int_t i = 0; i < n; i++) {
+                igraph_int_t nei = VECTOR(*neis)[i];
+                if (! IGRAPH_BIT_TEST(added, nei)) {
+                    IGRAPH_BIT_SET(added, nei);
                     IGRAPH_CHECK(igraph_dqueue_int_push(&Q, nei));
                     IGRAPH_CHECK(igraph_dqueue_int_push(&Q, actdist + 1));
                     if (parents) {
@@ -257,8 +253,9 @@ igraph_error_t igraph_bfs(const igraph_t *graph,
                 }
             }
 
-            succ_vec = igraph_dqueue_int_empty(&Q) ? -1L :
-                       igraph_dqueue_int_head(&Q);
+            succ_vec = igraph_dqueue_int_empty(&Q)
+                           ? -1
+                           : igraph_dqueue_int_head(&Q);
             if (callback) {
                 IGRAPH_CHECK_CALLBACK(
                     callback(graph, actvect, pred_vec, succ_vec, act_rank - 1, actdist, extra),
@@ -283,7 +280,7 @@ cleanup:
 
     igraph_lazy_adjlist_destroy(&adjlist);
     igraph_dqueue_int_destroy(&Q);
-    igraph_vector_bool_destroy(&added);
+    igraph_bitset_destroy(&added);
     IGRAPH_FINALLY_CLEAN(3);
 
     return IGRAPH_SUCCESS;
@@ -329,18 +326,17 @@ cleanup:
  * \example examples/simple/igraph_bfs_simple.c
  */
 igraph_error_t igraph_bfs_simple(
-    const igraph_t *graph, igraph_integer_t root, igraph_neimode_t mode,
+    const igraph_t *graph, igraph_int_t root, igraph_neimode_t mode,
     igraph_vector_int_t *order, igraph_vector_int_t *layers,
     igraph_vector_int_t *parents
 ) {
 
+    const igraph_int_t no_of_nodes = igraph_vcount(graph);
     igraph_dqueue_int_t q;
-    igraph_integer_t num_visited = 0;
+    igraph_int_t num_visited = 0;
     igraph_vector_int_t neis;
-    igraph_integer_t no_of_nodes = igraph_vcount(graph);
-    igraph_integer_t i;
-    bool *added;
-    igraph_integer_t lastlayer = -1;
+    igraph_bitset_t added;
+    igraph_int_t lastlayer = -1;
 
     if (!igraph_is_directed(graph)) {
         mode = IGRAPH_ALL;
@@ -348,15 +344,12 @@ igraph_error_t igraph_bfs_simple(
 
     if (mode != IGRAPH_OUT && mode != IGRAPH_IN &&
         mode != IGRAPH_ALL) {
-        IGRAPH_ERROR("Invalid mode argument", IGRAPH_EINVMODE);
+        IGRAPH_ERROR("Invalid mode argument.", IGRAPH_EINVMODE);
     }
 
     /* temporary storage */
-    added = IGRAPH_CALLOC(no_of_nodes, bool);
-    if (added == 0) {
-        IGRAPH_ERROR("Cannot calculate BFS", IGRAPH_ENOMEM); /* LCOV_EXCL_LINE */
-    }
-    IGRAPH_FINALLY(igraph_free, added);
+
+    IGRAPH_BITSET_INIT_FINALLY(&added, no_of_nodes);
     IGRAPH_VECTOR_INT_INIT_FINALLY(&neis, 0);
     IGRAPH_CHECK(igraph_dqueue_int_init(&q, 100));
     IGRAPH_FINALLY(igraph_dqueue_int_destroy, &q);
@@ -386,18 +379,19 @@ igraph_error_t igraph_bfs_simple(
         VECTOR(*parents)[root] = -1;
     }
     num_visited++;
-    added[root] = true;
+    IGRAPH_BIT_SET(added, root);
 
     while (!igraph_dqueue_int_empty(&q)) {
-        igraph_integer_t actvect = igraph_dqueue_int_pop(&q);
-        igraph_integer_t actdist = igraph_dqueue_int_pop(&q);
-        IGRAPH_CHECK(igraph_neighbors(graph, &neis, actvect,
-                                      mode));
-        igraph_integer_t nei_count = igraph_vector_int_size(&neis);
-        for (i = 0; i < nei_count; i++) {
-            igraph_integer_t neighbor = VECTOR(neis)[i];
-            if (! added[neighbor]) {
-                added[neighbor] = true;
+        igraph_int_t actvect = igraph_dqueue_int_pop(&q);
+        igraph_int_t actdist = igraph_dqueue_int_pop(&q);
+        IGRAPH_CHECK(igraph_neighbors(
+            graph, &neis, actvect, mode, IGRAPH_LOOPS, IGRAPH_MULTIPLE
+        ));
+        igraph_int_t nei_count = igraph_vector_int_size(&neis);
+        for (igraph_int_t i = 0; i < nei_count; i++) {
+            const igraph_int_t neighbor = VECTOR(neis)[i];
+            if (! IGRAPH_BIT_TEST(added, neighbor)) {
+                IGRAPH_BIT_SET(added, neighbor);
                 if (parents) {
                     VECTOR(*parents)[neighbor] = actvect;
                 }
@@ -421,7 +415,7 @@ igraph_error_t igraph_bfs_simple(
 
     igraph_vector_int_destroy(&neis);
     igraph_dqueue_int_destroy(&q);
-    IGRAPH_FREE(added);
+    igraph_bitset_destroy(&added);
     IGRAPH_FINALLY_CLEAN(3);
 
     return IGRAPH_SUCCESS;
@@ -429,7 +423,7 @@ igraph_error_t igraph_bfs_simple(
 
 /**
  * \function igraph_dfs
- * Depth-first search
+ * \brief Depth-first search.
  *
  * A simple depth-first search, with
  * the possibility to call a callback whenever a vertex is discovered
@@ -449,7 +443,7 @@ igraph_error_t igraph_bfs_simple(
  *        \c IGRAPH_IN means the opposite, and
  *        \c IGRAPH_ALL ignores the direction of the edges.
  *        This parameter is ignored for undirected graphs.
- * \param unreachable Logical scalar, whether the search should visit
+ * \param unreachable Boolean, whether the search should visit
  *        the vertices that are unreachable from the given root
  *        node(s). If true, then additional searches are performed
  *        until all vertices are visited.
@@ -482,7 +476,7 @@ igraph_error_t igraph_bfs_simple(
  * edges.
  */
 
-igraph_error_t igraph_dfs(const igraph_t *graph, igraph_integer_t root,
+igraph_error_t igraph_dfs(const igraph_t *graph, igraph_int_t root,
                igraph_neimode_t mode, igraph_bool_t unreachable,
                igraph_vector_int_t *order,
                igraph_vector_int_t *order_out, igraph_vector_int_t *parents,
@@ -490,50 +484,48 @@ igraph_error_t igraph_dfs(const igraph_t *graph, igraph_integer_t root,
                igraph_dfshandler_t *out_callback,
                void *extra) {
 
-    igraph_integer_t no_of_nodes = igraph_vcount(graph);
+    const igraph_int_t no_of_nodes = igraph_vcount(graph);
     igraph_lazy_adjlist_t adjlist;
     igraph_stack_int_t stack;
-    igraph_vector_char_t added;
+    igraph_bitset_t added;
     igraph_vector_int_t nptr;
     igraph_error_t ret;
-    igraph_integer_t actroot;
-    igraph_integer_t act_rank = 0;
-    igraph_integer_t rank_out = 0;
-    igraph_integer_t act_dist = 0;
+    igraph_int_t act_rank = 0;
+    igraph_int_t rank_out = 0;
+    igraph_int_t act_dist = 0;
 
     if (root < 0 || root >= no_of_nodes) {
-        IGRAPH_ERROR("Invalid root vertex for DFS", IGRAPH_EINVAL);
+        IGRAPH_ERROR("Invalid root vertex for DFS.", IGRAPH_EINVAL);
     }
 
     if (mode != IGRAPH_OUT && mode != IGRAPH_IN &&
         mode != IGRAPH_ALL) {
-        IGRAPH_ERROR("Invalid mode argument", IGRAPH_EINVMODE);
+        IGRAPH_ERROR("Invalid mode argument.", IGRAPH_EINVMODE);
     }
 
     if (!igraph_is_directed(graph)) {
         mode = IGRAPH_ALL;
     }
 
-    IGRAPH_CHECK(igraph_vector_char_init(&added, no_of_nodes));
-    IGRAPH_FINALLY(igraph_vector_char_destroy, &added);
-    IGRAPH_CHECK(igraph_stack_int_init(&stack, 100));
-    IGRAPH_FINALLY(igraph_stack_int_destroy, &stack);
+    IGRAPH_BITSET_INIT_FINALLY(&added, no_of_nodes);
+    IGRAPH_STACK_INT_INIT_FINALLY(&stack, 100);
+
     IGRAPH_CHECK(igraph_lazy_adjlist_init(graph, &adjlist, mode, IGRAPH_LOOPS, IGRAPH_MULTIPLE));
     IGRAPH_FINALLY(igraph_lazy_adjlist_destroy, &adjlist);
-    IGRAPH_CHECK(igraph_vector_int_init(&nptr, no_of_nodes));
-    IGRAPH_FINALLY(igraph_vector_int_destroy, &nptr);
 
-# define FREE_ALL() do {            \
-        igraph_vector_int_destroy(&nptr);            \
-        igraph_lazy_adjlist_destroy(&adjlist);        \
-        igraph_stack_int_destroy(&stack);                 \
-        igraph_vector_char_destroy(&added);           \
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&nptr, no_of_nodes);
+
+# define FREE_ALL() do { \
+        igraph_vector_int_destroy(&nptr); \
+        igraph_lazy_adjlist_destroy(&adjlist); \
+        igraph_stack_int_destroy(&stack); \
+        igraph_bitset_destroy(&added); \
         IGRAPH_FINALLY_CLEAN(4); } while (0)
 
     /* Resize result vectors and fill them with the initial value */
 
-# define VINIT(v, initial) if (v) {             \
-        IGRAPH_CHECK(igraph_vector_int_resize(v, no_of_nodes));       \
+# define VINIT(v, initial) if (v) { \
+        IGRAPH_CHECK(igraph_vector_int_resize(v, no_of_nodes)); \
         igraph_vector_int_fill(v, initial); }
 
     VINIT(order, -1);
@@ -544,7 +536,7 @@ igraph_error_t igraph_dfs(const igraph_t *graph, igraph_integer_t root,
 # undef VINIT
 
     IGRAPH_CHECK(igraph_stack_int_push(&stack, root));
-    VECTOR(added)[root] = 1;
+    IGRAPH_BIT_SET(added, root);
     if (parents) {
         VECTOR(*parents)[root] = -1;
     }
@@ -562,19 +554,19 @@ igraph_error_t igraph_dfs(const igraph_t *graph, igraph_integer_t root,
         }
     }
 
-    for (actroot = 0; actroot < no_of_nodes; ) {
+    for (igraph_int_t actroot = 0; actroot < no_of_nodes; ) {
 
         /* 'root' first, then all other vertices */
         if (igraph_stack_int_empty(&stack)) {
             if (!unreachable) {
                 break;
             }
-            if (VECTOR(added)[actroot]) {
+            if (IGRAPH_BIT_TEST(added, actroot)) {
                 actroot++;
                 continue;
             }
             IGRAPH_CHECK(igraph_stack_int_push(&stack, actroot));
-            VECTOR(added)[actroot] = 1;
+            IGRAPH_BIT_SET(added, actroot);
             if (parents) {
                 VECTOR(*parents)[actroot] = -1;
             }
@@ -597,25 +589,26 @@ igraph_error_t igraph_dfs(const igraph_t *graph, igraph_integer_t root,
         }
 
         while (!igraph_stack_int_empty(&stack)) {
-            igraph_integer_t actvect = igraph_stack_int_top(&stack);
-            igraph_vector_int_t *neis = igraph_lazy_adjlist_get(&adjlist, actvect);
-            igraph_integer_t n = igraph_vector_int_size(neis);
-            igraph_integer_t *ptr = igraph_vector_int_get_ptr(&nptr, actvect);
+            igraph_int_t actvect = igraph_stack_int_top(&stack);
+            igraph_int_t *ptr = igraph_vector_int_get_ptr(&nptr, actvect);
 
+            igraph_vector_int_t *neis = igraph_lazy_adjlist_get(&adjlist, actvect);
             IGRAPH_CHECK_OOM(neis, "Failed to query neighbors.");
 
+            const igraph_int_t n = igraph_vector_int_size(neis);
+
             /* Search for a neighbor that was not yet visited */
             igraph_bool_t any = false;
-            igraph_integer_t nei = 0;
+            igraph_int_t nei = 0;
             while (!any && (*ptr) < n) {
                 nei = VECTOR(*neis)[(*ptr)];
-                any = !VECTOR(added)[nei];
+                any = !IGRAPH_BIT_TEST(added, nei);
                 (*ptr) ++;
             }
             if (any) {
                 /* There is such a neighbor, add it */
                 IGRAPH_CHECK(igraph_stack_int_push(&stack, nei));
-                VECTOR(added)[nei] = 1;
+                IGRAPH_BIT_SET(added, nei);
                 if (parents) {
                     VECTOR(*parents)[ nei ] = actvect;
                 }
