@@ -1,5 +1,5 @@
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2011-2012  Gabor Csardi <csardi@rmki.kfki.hu>
    334 Harvard street, Cambridge MA, 02139 USA
 
@@ -27,12 +27,11 @@
 
 typedef struct {
     void *scanner;
-    int eof;
     char errmsg[300];
     igraph_error_t igraph_errno;
     igraph_bool_t has_weights;
     igraph_vector_int_t *vector;
     igraph_vector_t *weights;
     igraph_trie_t *trie;
-    igraph_integer_t actvertex;
+    igraph_int_t actvertex;
 } igraph_i_lgl_parsedata_t;
