@@ -1,6 +1,5 @@
-/* -*- mode: C -*-  */
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2005-2020  The igraph development team
 
    This program is free software; you can redistribute it and/or modify
@@ -74,17 +73,18 @@ void igraph_ncol_yylex_destroy_wrapper (void *scanner ) {
  *        the <code>.ncol</code> file. If it is not \c NULL and some unknown
  *        vertex names are found in the <code>.ncol</code> file then new vertex
  *        ids will be assigned to them.
- * \param names Logical value, if \c true the symbolic names of the
+ * \param names Boolean value. If \c true, the symbolic names of the
  *        vertices will be added to the graph as a vertex attribute
  *        called \quote name\endquote.
  * \param weights Whether to add the weights of the edges to the
  *        graph as an edge attribute called \quote weight\endquote.
  *        \c IGRAPH_ADD_WEIGHTS_YES adds the weights (even if they
  *        are not present in the file, in this case they are assumed
- *        to be zero). \c IGRAPH_ADD_WEIGHTS_NO does not add any
+ *        to be 1). \c IGRAPH_ADD_WEIGHTS_NO does not add any
  *        edge attribute. \c IGRAPH_ADD_WEIGHTS_IF_PRESENT adds the
  *        attribute if and only if there is at least one explicit
- *        edge weight in the input file.
+ *        edge weight in the input file, and edges without an explicit
+ *        weight are assumed to have a weight of 1.
  * \param directed Whether to create a directed graph. As this format
  *        was originally used only for undirected graphs there is no
  *        information in the file about the directedness of the graph.
@@ -112,11 +112,11 @@ igraph_error_t igraph_read_graph_ncol(igraph_t *graph, FILE *instream,
     igraph_vector_int_t edges;
     igraph_vector_t ws;
     igraph_trie_t trie = IGRAPH_TRIE_NULL;
-    igraph_integer_t no_of_nodes;
-    igraph_integer_t no_predefined = 0;
-    igraph_vector_ptr_t name, weight;
-    igraph_vector_ptr_t *pname = NULL, *pweight = NULL;
-    igraph_attribute_record_t namerec, weightrec;
+    igraph_int_t no_of_nodes;
+    igraph_int_t no_predefined = 0;
+    igraph_attribute_record_list_t name, weight;
+    igraph_attribute_record_list_t *pname = NULL, *pweight = NULL;
+    igraph_attribute_record_t *namerec, *weightrec;
     const char *namestr = "name", *weightstr = "weight";
     igraph_i_ncol_parsedata_t context;
 
@@ -127,7 +127,7 @@ igraph_error_t igraph_read_graph_ncol(igraph_t *graph, FILE *instream,
 
     /* Add the predefined names, if any */
     if (predefnames != 0) {
-        igraph_integer_t i, id, n;
+        igraph_int_t i, id, n;
         const char *key;
         n = no_predefined = igraph_strvector_size(predefnames);
         for (i = 0; i < n; i++) {
@@ -185,28 +185,32 @@ igraph_error_t igraph_read_graph_ncol(igraph_t *graph, FILE *instream,
         IGRAPH_WARNING("Unknown vertex/vertices found in NCOL file, predefined names extended.");
     }
 
-    /* Prepare attributes, if needed */
-
+    /* Prepare attributes if needed */
     if (names) {
-        IGRAPH_CHECK(igraph_vector_ptr_init(&name, 1));
-        IGRAPH_FINALLY(igraph_vector_ptr_destroy, &name);
+        IGRAPH_CHECK(igraph_attribute_record_list_init(&name, 1));
+        IGRAPH_FINALLY(igraph_attribute_record_list_destroy, &name);
+
+        namerec = igraph_attribute_record_list_get_ptr(&name, 0);
+        IGRAPH_CHECK(igraph_attribute_record_set_name(namerec, namestr));
+        IGRAPH_CHECK(igraph_attribute_record_set_type(namerec, IGRAPH_ATTRIBUTE_STRING));
+        IGRAPH_CHECK(igraph_strvector_update(
+            namerec->value.as_strvector, igraph_i_trie_borrow_keys(context.trie))
+        );
+
         pname = &name;
-        namerec.name = namestr;
-        namerec.type = IGRAPH_ATTRIBUTE_STRING;
-        namerec.value = igraph_i_trie_borrow_keys(&trie);
-        VECTOR(name)[0] = &namerec;
     }
 
     if (weights == IGRAPH_ADD_WEIGHTS_YES ||
         (weights == IGRAPH_ADD_WEIGHTS_IF_PRESENT && context.has_weights)) {
+        IGRAPH_CHECK(igraph_attribute_record_list_init(&weight, 1));
+        IGRAPH_FINALLY(igraph_attribute_record_list_destroy, &weight);
+
+        weightrec = igraph_attribute_record_list_get_ptr(&weight, 0);
+        IGRAPH_CHECK(igraph_attribute_record_set_name(weightrec, weightstr));
+        IGRAPH_CHECK(igraph_attribute_record_set_type(weightrec, IGRAPH_ATTRIBUTE_NUMERIC));
+        igraph_vector_swap(weightrec->value.as_vector, context.weights);
 
-        IGRAPH_CHECK(igraph_vector_ptr_init(&weight, 1));
-        IGRAPH_FINALLY(igraph_vector_ptr_destroy, &weight);
         pweight = &weight;
-        weightrec.name = weightstr;
-        weightrec.type = IGRAPH_ATTRIBUTE_NUMERIC;
-        weightrec.value = &ws;
-        VECTOR(weight)[0] = &weightrec;
     }
 
     if (igraph_vector_int_empty(&edges)) {
@@ -222,11 +226,11 @@ igraph_error_t igraph_read_graph_ncol(igraph_t *graph, FILE *instream,
     IGRAPH_CHECK(igraph_add_edges(graph, &edges, pweight));
 
     if (pname) {
-        igraph_vector_ptr_destroy(pname);
+        igraph_attribute_record_list_destroy(pname);
         IGRAPH_FINALLY_CLEAN(1);
     }
     if (pweight) {
-        igraph_vector_ptr_destroy(pweight);
+        igraph_attribute_record_list_destroy(pweight);
         IGRAPH_FINALLY_CLEAN(1);
     }
     igraph_vector_destroy(&ws);
@@ -309,7 +313,7 @@ igraph_error_t igraph_write_graph_ncol(const igraph_t *graph, FILE *outstream,
         names = NULL;
     }
     if (names) {
-        IGRAPH_CHECK(igraph_i_attribute_gettype(graph, &nametype,
+        IGRAPH_CHECK(igraph_i_attribute_get_type(graph, &nametype,
                                                 IGRAPH_ATTRIBUTE_VERTEX, names));
         if (nametype != IGRAPH_ATTRIBUTE_STRING) {
             IGRAPH_WARNINGF("Ignoring names attribute '%s', "
@@ -324,7 +328,7 @@ igraph_error_t igraph_write_graph_ncol(const igraph_t *graph, FILE *outstream,
         weights = NULL;
     }
     if (weights) {
-        IGRAPH_CHECK(igraph_i_attribute_gettype(graph, &weighttype,
+        IGRAPH_CHECK(igraph_i_attribute_get_type(graph, &weighttype,
                                                 IGRAPH_ATTRIBUTE_EDGE, weights));
         if (weighttype != IGRAPH_ATTRIBUTE_NUMERIC) {
             IGRAPH_WARNINGF("Ignoring weights attribute '%s', "
@@ -336,7 +340,7 @@ igraph_error_t igraph_write_graph_ncol(const igraph_t *graph, FILE *outstream,
     if (names == NULL && weights == NULL) {
         /* No names, no weights */
         while (!IGRAPH_EIT_END(it)) {
-            igraph_integer_t from, to;
+            igraph_int_t from, to;
             int ret;
             igraph_edge(graph, IGRAPH_EIT_GET(it), &from, &to);
             ret = fprintf(outstream, "%" IGRAPH_PRId " %" IGRAPH_PRId "\n", from, to);
@@ -354,8 +358,8 @@ igraph_error_t igraph_write_graph_ncol(const igraph_t *graph, FILE *outstream,
                      igraph_vss_all(),
                      &nvec));
         while (!IGRAPH_EIT_END(it)) {
-            igraph_integer_t edge = IGRAPH_EIT_GET(it);
-            igraph_integer_t from, to;
+            igraph_int_t edge = IGRAPH_EIT_GET(it);
+            igraph_int_t from, to;
             int ret = 0;
             const char *str1, *str2;
             igraph_edge(graph, edge, &from, &to);
@@ -379,8 +383,8 @@ igraph_error_t igraph_write_graph_ncol(const igraph_t *graph, FILE *outstream,
                      igraph_ess_all(IGRAPH_EDGEORDER_ID),
                      &wvec));
         while (!IGRAPH_EIT_END(it)) {
-            igraph_integer_t edge = IGRAPH_EIT_GET(it);
-            igraph_integer_t from, to;
+            igraph_int_t edge = IGRAPH_EIT_GET(it);
+            igraph_int_t from, to;
             int ret1, ret2, ret3;
             igraph_edge(graph, edge, &from, &to);
             ret1 = fprintf(outstream, "%" IGRAPH_PRId " %" IGRAPH_PRId " ", from, to);
@@ -407,8 +411,8 @@ igraph_error_t igraph_write_graph_ncol(const igraph_t *graph, FILE *outstream,
                      igraph_vss_all(),
                      &nvec));
         while (!IGRAPH_EIT_END(it)) {
-            igraph_integer_t edge = IGRAPH_EIT_GET(it);
-            igraph_integer_t from, to;
+            igraph_int_t edge = IGRAPH_EIT_GET(it);
+            igraph_int_t from, to;
             int ret = 0, ret2 = 0;
             const char *str1, *str2;
             igraph_edge(graph, edge, &from, &to);
