@@ -80,7 +80,7 @@
 typedef union YYSTYPE
 
 {
-  igraph_integer_t integer;
+  igraph_int_t integer;
   igraph_real_t real;
 }
 /* Line 1529 of yacc.c.  */
