@@ -94,7 +94,7 @@
 
 
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2006-2012  Gabor Csardi <csardi.gabor@gmail.com>
    334 Harvard st, Cambridge, MA, 02138 USA
 
@@ -157,7 +157,7 @@ int igraph_ncol_yyerror(YYLTYPE* locp,
 typedef union YYSTYPE
 
 {
-  igraph_integer_t edgenum;
+  igraph_int_t edgenum;
   igraph_real_t weightnum;
 }
 /* Line 193 of yacc.c.  */
@@ -403,16 +403,16 @@ union yyalloc
 /* YYFINAL -- State number of the termination state.  */
 #define YYFINAL  2
 /* YYLAST -- Last index in YYTABLE.  */
-#define YYLAST   10
+#define YYLAST   7
 
 /* YYNTOKENS -- Number of terminals.  */
 #define YYNTOKENS  6
 /* YYNNTS -- Number of nonterminals.  */
-#define YYNNTS  5
+#define YYNNTS  6
 /* YYNRULES -- Number of rules.  */
-#define YYNRULES  8
+#define YYNRULES  9
 /* YYNRULES -- Number of states.  */
-#define YYNSTATES  12
+#define YYNSTATES  13
 
 /* YYTRANSLATE(YYLEX) -- Bison symbol number corresponding to YYLEX.  */
 #define YYUNDEFTOK  2
@@ -458,21 +458,21 @@ static const yytype_uint8 yytranslate[] =
    YYRHS.  */
 static const yytype_uint8 yyprhs[] =
 {
-       0,     0,     3,     4,     7,    10,    14,    19,    21
+       0,     0,     3,     4,     7,    10,    13,    17,    20,    22
 };
 
 /* YYRHS -- A `-1'-separated list of the rules' RHS.  */
 static const yytype_int8 yyrhs[] =
 {
        7,     0,    -1,    -1,     7,     4,    -1,     7,     8,    -1,
-       9,     9,     4,    -1,     9,     9,    10,     4,    -1,     3,
-      -1,     3,    -1
+       9,     4,    -1,     9,    11,     4,    -1,    10,    10,    -1,
+       3,    -1,     3,    -1
 };
 
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint8 yyrline[] =
 {
-       0,    92,    92,    93,    94,    97,   102,   110,   120
+       0,    92,    92,    93,    94,    97,   100,   106,   111,   121
 };
 #endif
 
@@ -482,8 +482,8 @@ static const yytype_uint8 yyrline[] =
 static const char *const yytname[] =
 {
   "\"end of file\"", "error", "$undefined", "\"alphanumeric\"",
-  "\"end of line\"", "ERROR", "$accept", "input", "edge", "edgeid",
-  "weight", 0
+  "\"end of line\"", "ERROR", "$accept", "input", "edge", "endpoints",
+  "edgeid", "weight", 0
 };
 #endif
 
@@ -499,13 +499,13 @@ static const yytype_uint16 yytoknum[] =
 /* YYR1[YYN] -- Symbol number of symbol that rule YYN derives.  */
 static const yytype_uint8 yyr1[] =
 {
-       0,     6,     7,     7,     7,     8,     8,     9,    10
+       0,     6,     7,     7,     7,     8,     8,     9,    10,    11
 };
 
 /* YYR2[YYN] -- Number of symbols composing right hand side of rule YYN.  */
 static const yytype_uint8 yyr2[] =
 {
-       0,     2,     0,     2,     2,     3,     4,     1,     1
+       0,     2,     0,     2,     2,     2,     3,     2,     1,     1
 };
 
 /* YYDEFACT[STATE-NAME] -- Default rule to reduce with in state
@@ -513,14 +513,14 @@ static const yytype_uint8 yyr2[] =
    means the default is an error.  */
 static const yytype_uint8 yydefact[] =
 {
-       2,     0,     1,     7,     3,     4,     0,     0,     8,     5,
-       0,     6
+       2,     0,     1,     8,     3,     4,     0,     0,     9,     5,
+       0,     7,     6
 };
 
 /* YYDEFGOTO[NTERM-NUM].  */
 static const yytype_int8 yydefgoto[] =
 {
-      -1,     1,     5,     6,    10
+      -1,     1,     5,     6,     7,    10
 };
 
 /* YYPACT[STATE-NUM] -- Index in YYTABLE of the portion describing
@@ -528,14 +528,14 @@ static const yytype_int8 yydefgoto[] =
 #define YYPACT_NINF -3
 static const yytype_int8 yypact[] =
 {
-      -3,     0,    -3,    -3,    -3,    -3,     2,    -2,    -3,    -3,
-       3,    -3
+      -3,     0,    -3,    -3,    -3,    -3,    -2,     2,    -3,    -3,
+       3,    -3,    -3
 };
 
 /* YYPGOTO[NTERM-NUM].  */
 static const yytype_int8 yypgoto[] =
 {
-      -3,    -3,    -3,     4,    -3
+      -3,    -3,    -3,    -3,    -1,    -3
 };
 
 /* YYTABLE[YYPACT[STATE-NUM]].  What to do in state STATE-NUM.  If
@@ -545,22 +545,20 @@ static const yytype_int8 yypgoto[] =
 #define YYTABLE_NINF -1
 static const yytype_uint8 yytable[] =
 {
-       2,     8,     9,     3,     4,     3,     0,    11,     0,     0,
-       7
+       2,     8,     9,     3,     4,     3,    11,    12
 };
 
-static const yytype_int8 yycheck[] =
+static const yytype_uint8 yycheck[] =
 {
-       0,     3,     4,     3,     4,     3,    -1,     4,    -1,    -1,
-       6
+       0,     3,     4,     3,     4,     3,     7,     4
 };
 
 /* YYSTOS[STATE-NUM] -- The (internal number of the) accessing
    symbol of state STATE-NUM.  */
 static const yytype_uint8 yystos[] =
 {
-       0,     7,     0,     3,     4,     8,     9,     9,     3,     4,
-      10,     4
+       0,     7,     0,     3,     4,     8,     9,    10,     3,     4,
+      11,    10,     4
 };
 
 #define yyerrok		(yyerrstatus = 0)
@@ -1405,26 +1403,30 @@ yyreduce:
         case 5:
 
     {
-           IGRAPH_YY_CHECK(igraph_vector_int_push_back(context->vector, (yyvsp[(1) - (3)].edgenum)));
-           IGRAPH_YY_CHECK(igraph_vector_int_push_back(context->vector, (yyvsp[(2) - (3)].edgenum)));
-           IGRAPH_YY_CHECK(igraph_vector_push_back(context->weights, 0.0));
+           IGRAPH_YY_CHECK(igraph_vector_push_back(context->weights, 1.0));
        ;}
     break;
 
   case 6:
 
     {
-           IGRAPH_YY_CHECK(igraph_vector_int_push_back(context->vector, (yyvsp[(1) - (4)].edgenum)));
-           IGRAPH_YY_CHECK(igraph_vector_int_push_back(context->vector, (yyvsp[(2) - (4)].edgenum)));
-           IGRAPH_YY_CHECK(igraph_vector_push_back(context->weights, (yyvsp[(3) - (4)].weightnum)));
-           context->has_weights = 1;
+           IGRAPH_YY_CHECK(igraph_vector_push_back(context->weights, (yyvsp[(2) - (3)].weightnum)));
+           context->has_weights = true;
        ;}
     break;
 
   case 7:
 
     {
-  igraph_integer_t trie_id;
+  IGRAPH_YY_CHECK(igraph_vector_int_push_back(context->vector, (yyvsp[(1) - (2)].edgenum)));
+  IGRAPH_YY_CHECK(igraph_vector_int_push_back(context->vector, (yyvsp[(2) - (2)].edgenum)));
+;}
+    break;
+
+  case 8:
+
+    {
+  igraph_int_t trie_id;
   IGRAPH_YY_CHECK(igraph_trie_get_len(context->trie,
     igraph_ncol_yyget_text(scanner),
     igraph_ncol_yyget_leng(scanner),
@@ -1434,7 +1436,7 @@ yyreduce:
 ;}
     break;
 
-  case 8:
+  case 9:
 
     {
     igraph_real_t val;
