@@ -182,7 +182,7 @@
 
 
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2006-2012  Gabor Csardi <csardi.gabor@gmail.com>
    334 Harvard st, Cambridge, MA, 02138 USA
 
@@ -237,18 +237,20 @@ static igraph_error_t add_numeric_edge_attribute(const char *name,
                                               igraph_real_t value,
                                               igraph_i_pajek_parsedata_t *context);
 static igraph_error_t add_numeric_attribute(igraph_trie_t *names,
-                                         igraph_vector_ptr_t *attrs,
-                                         igraph_integer_t count,
+                                         igraph_attribute_record_list_t *attrs,
+                                         igraph_int_t count,
                                          const char *attrname,
-                                         igraph_integer_t vid,
+                                         igraph_real_t default_value,
+                                         igraph_int_t vid,
                                          igraph_real_t number);
 static igraph_error_t add_string_attribute(igraph_trie_t *names,
-                                        igraph_vector_ptr_t *attrs,
-                                        igraph_integer_t count,
+                                        igraph_attribute_record_list_t *attrs,
+                                        igraph_int_t count,
                                         const char *attrname,
-                                        igraph_integer_t vid,
+                                        const char *default_value,
+                                        igraph_int_t vid,
                                         const char *str,
-                                        igraph_integer_t str_len);
+                                        igraph_int_t str_len);
 
 static igraph_error_t add_bipartite_type(igraph_i_pajek_parsedata_t *context);
 static igraph_error_t check_bipartite(igraph_i_pajek_parsedata_t *context);
@@ -257,6 +259,10 @@ static igraph_error_t make_dynstr(const char *src, size_t len, char **res);
 static igraph_bool_t is_standard_vattr(const char *attrname);
 static igraph_bool_t is_standard_eattr(const char *attrname);
 static igraph_error_t deconflict_attrname(char **attrname);
+static igraph_real_t get_default_value_for_numeric_vattr(const char *attrname);
+static igraph_real_t get_default_value_for_numeric_eattr(const char *attrname);
+static const char* get_default_value_for_string_vattr(const char *attrname);
+static const char* get_default_value_for_string_eattr(const char *attrname);
 
 #define scanner context->scanner
 
@@ -284,7 +290,7 @@ static igraph_error_t deconflict_attrname(char **attrname);
 typedef union YYSTYPE
 
 {
-  igraph_integer_t intnum;
+  igraph_int_t intnum;
   igraph_real_t    realnum;
   struct {
     char *str;
@@ -648,18 +654,18 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,   188,   188,   199,   199,   201,   201,   203,   205,   216,
-     235,   235,   237,   238,   238,   248,   267,   272,   273,   277,
-     283,   283,   287,   287,   290,   291,   294,   297,   300,   303,
-     306,   309,   312,   315,   318,   323,   326,   329,   332,   335,
-     338,   353,   353,   353,   353,   353,   353,   355,   356,   358,
-     358,   360,   360,   365,   366,   368,   368,   370,   370,   375,
-     375,   379,   379,   382,   383,   386,   389,   392,   395,   398,
-     401,   404,   407,   410,   413,   416,   419,   422,   427,   430,
-     433,   436,   439,   442,   445,   460,   462,   462,   464,   466,
-     466,   468,   470,   475,   477,   477,   479,   481,   481,   483,
-     485,   492,   494,   499,   499,   501,   503,   503,   505,   525,
-     533,   541,   545,   547,   549,   551
+       0,   194,   194,   205,   205,   207,   207,   209,   211,   222,
+     241,   241,   243,   244,   244,   254,   273,   277,   278,   282,
+     288,   288,   292,   292,   295,   296,   299,   302,   305,   308,
+     311,   314,   317,   320,   323,   328,   331,   334,   337,   340,
+     343,   358,   358,   358,   358,   358,   358,   360,   361,   363,
+     363,   365,   365,   370,   371,   373,   373,   375,   375,   380,
+     380,   384,   384,   387,   388,   391,   394,   397,   400,   403,
+     406,   409,   412,   415,   418,   421,   424,   427,   432,   435,
+     438,   441,   444,   447,   450,   465,   467,   467,   469,   471,
+     471,   473,   475,   480,   482,   482,   484,   486,   486,   488,
+     490,   497,   499,   504,   504,   506,   508,   508,   510,   530,
+     538,   546,   550,   552,   554,   556
 };
 #endif
 
@@ -1793,7 +1799,7 @@ yyreduce:
   case 14:
 
     {
-              igraph_integer_t v = (yyvsp[(1) - (7)].intnum)-1; /* zero-based vertex ID */
+              igraph_int_t v = (yyvsp[(1) - (7)].intnum)-1; /* zero-based vertex ID */
               if (IGRAPH_BIT_TEST(*context->seen, v)) {
                 IGRAPH_WARNINGF("Vertex ID %" IGRAPH_PRId " appears twice in Pajek file. Duplicate attributes will be overwritten.", v+1);
               } else {
@@ -1805,7 +1811,7 @@ yyreduce:
   case 15:
 
     {
-  igraph_integer_t v = (yyvsp[(1) - (1)].intnum);
+  igraph_int_t v = (yyvsp[(1) - (1)].intnum);
   /* Per feedback from Pajek's authors, negative signs should be ignored for vertex IDs.
    * See https://nascol.discourse.group/t/pajek-arcslist-edgelist-format/44/2
    * This applies to all of *Edges, *Arcs, *Edgeslist, *Arcslist and *Vertices section.
@@ -1827,7 +1833,6 @@ yyreduce:
   case 16:
 
     {
-  IGRAPH_YY_CHECK(add_string_vertex_attribute("id", (yyvsp[(1) - (1)].string).str, (yyvsp[(1) - (1)].string).len, context));
   IGRAPH_YY_CHECK(add_string_vertex_attribute("name", (yyvsp[(1) - (1)].string).str, (yyvsp[(1) - (1)].string).len, context));
 ;}
     break;
@@ -2259,7 +2264,7 @@ yyreduce:
   case 109:
 
     {
-  igraph_integer_t val;
+  igraph_int_t val;
   IGRAPH_YY_CHECK(igraph_i_parse_integer(igraph_pajek_yyget_text(scanner),
                                          igraph_pajek_yyget_leng(scanner),
                                          &val));
@@ -2545,49 +2550,38 @@ int igraph_pajek_yyerror(YYLTYPE* locp,
 /* TODO: NA's */
 
 static igraph_error_t add_numeric_attribute(igraph_trie_t *names,
-                                            igraph_vector_ptr_t *attrs,
-                                            igraph_integer_t count,
+                                            igraph_attribute_record_list_t *attrs,
+                                            igraph_int_t count,
                                             const char *attrname,
-                                            igraph_integer_t elem_id,
+                                            igraph_real_t default_value,
+                                            igraph_int_t elem_id,
                                             igraph_real_t number) {
-  igraph_integer_t attrsize = igraph_trie_size(names);
-  igraph_integer_t id;
+  igraph_int_t attrsize = igraph_trie_size(names);
+  igraph_int_t id;
   igraph_vector_t *na;
-  igraph_attribute_record_t *rec;
+  igraph_attribute_record_t *prec;
 
   IGRAPH_CHECK(igraph_trie_get(names, attrname, &id));
   if (id == attrsize) {
+    igraph_attribute_record_t rec;
+
     /* add a new attribute */
-    rec = IGRAPH_CALLOC(1, igraph_attribute_record_t);
-    CHECK_OOM_RP(rec);
-    IGRAPH_FINALLY(igraph_free, rec);
+    IGRAPH_CHECK(igraph_attribute_record_init(&rec, attrname, IGRAPH_ATTRIBUTE_NUMERIC));
+    IGRAPH_FINALLY(igraph_attribute_record_destroy, &rec);
 
-    na = IGRAPH_CALLOC(1, igraph_vector_t);
-    CHECK_OOM_RP(na);
-    IGRAPH_FINALLY(igraph_free, na);
-    IGRAPH_VECTOR_INIT_FINALLY(na, count);
+    IGRAPH_CHECK(igraph_attribute_record_set_default_numeric(&rec, default_value));
 
-    rec->name = strdup(attrname);
-    CHECK_OOM_RP(rec->name);
-    IGRAPH_FINALLY(igraph_free, (void *) rec->name);
-
-    rec->type = IGRAPH_ATTRIBUTE_NUMERIC;
-    rec->value = na;
-
-    IGRAPH_CHECK(igraph_vector_ptr_push_back(attrs, rec));
-    IGRAPH_FINALLY_CLEAN(4); /* ownership of rec transferred to attrs */
+    IGRAPH_CHECK(igraph_attribute_record_resize(&rec, count));
+    IGRAPH_CHECK(igraph_attribute_record_list_push_back(attrs, &rec));
+    IGRAPH_FINALLY_CLEAN(1); /* ownership of rec transferred to attrs */
   }
 
-  rec = VECTOR(*attrs)[id];
-  na = (igraph_vector_t *) rec->value;
+  prec = igraph_attribute_record_list_get_ptr(attrs, id);
+  na = prec->value.as_vector;
   if (igraph_vector_size(na) == elem_id) {
     IGRAPH_CHECK(igraph_vector_push_back(na, number));
   } else if (igraph_vector_size(na) < elem_id) {
-    igraph_integer_t origsize=igraph_vector_size(na);
-    IGRAPH_CHECK(igraph_vector_resize(na, elem_id+1));
-    for (;origsize<count; origsize++) {
-      VECTOR(*na)[origsize] = IGRAPH_NAN;
-    }
+    IGRAPH_CHECK(igraph_attribute_record_resize(prec, elem_id+1));
     VECTOR(*na)[elem_id] = number;
   } else {
     VECTOR(*na)[elem_id] = number;
@@ -2596,8 +2590,6 @@ static igraph_error_t add_numeric_attribute(igraph_trie_t *names,
   return IGRAPH_SUCCESS;
 }
 
-/* TODO: NA's */
-
 static igraph_error_t make_dynstr(const char *src, size_t len, char **res) {
   *res = strndup(src, len);
   CHECK_OOM_RP(*res);
@@ -2605,16 +2597,17 @@ static igraph_error_t make_dynstr(const char *src, size_t len, char **res) {
 }
 
 static igraph_error_t add_string_attribute(igraph_trie_t *names,
-                                           igraph_vector_ptr_t *attrs,
-                                           igraph_integer_t count,
+                                           igraph_attribute_record_list_t *attrs,
+                                           igraph_int_t count,
                                            const char *attrname,
-                                           igraph_integer_t elem_id,
+                                           const char *default_value,
+                                           igraph_int_t elem_id,
                                            const char *str,
-                                           igraph_integer_t str_len) {
-  igraph_integer_t attrsize=igraph_trie_size(names);
-  igraph_integer_t id;
+                                           igraph_int_t str_len) {
+  igraph_int_t attrsize=igraph_trie_size(names);
+  igraph_int_t id;
   igraph_strvector_t *na;
-  igraph_attribute_record_t *rec;
+  igraph_attribute_record_t *prec;
 
   if (attrname[0] == '\0') {
     /* This is relevant only for custom attributes, which are always of string type.
@@ -2624,6 +2617,7 @@ static igraph_error_t add_string_attribute(igraph_trie_t *names,
 
   IGRAPH_CHECK(igraph_trie_get(names, attrname, &id));
   if (id == attrsize) {
+    igraph_attribute_record_t rec;
 
 #ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
     /* There are 21 standard vertex attributes and 21 standard edge attributes.
@@ -2633,29 +2627,18 @@ static igraph_error_t add_string_attribute(igraph_trie_t *names,
     }
 #endif
 
-    /* add a new attribute */
-    rec = IGRAPH_CALLOC(1, igraph_attribute_record_t);
-    CHECK_OOM_RP(rec);
-    IGRAPH_FINALLY(igraph_free, rec);
+    IGRAPH_CHECK(igraph_attribute_record_init(&rec, attrname, IGRAPH_ATTRIBUTE_STRING));
+    IGRAPH_FINALLY(igraph_attribute_record_destroy, &rec);
 
-    na = IGRAPH_CALLOC(1, igraph_strvector_t);
-    CHECK_OOM_RP(na);
-    IGRAPH_FINALLY(igraph_free, na);
-    IGRAPH_STRVECTOR_INIT_FINALLY(na, count);
+    IGRAPH_CHECK(igraph_attribute_record_set_default_string(&rec, default_value));
 
-    rec->name = strdup(attrname);
-    CHECK_OOM_RP(rec->name);
-    IGRAPH_FINALLY(igraph_free, (char *) rec->name);
-
-    rec->type = IGRAPH_ATTRIBUTE_STRING;
-    rec->value = na;
-
-    IGRAPH_CHECK(igraph_vector_ptr_push_back(attrs, rec));
-    IGRAPH_FINALLY_CLEAN(4); /* ownership of rec transferred to attrs */
+    IGRAPH_CHECK(igraph_attribute_record_resize(&rec, count));
+    IGRAPH_CHECK(igraph_attribute_record_list_push_back(attrs, &rec));
+    IGRAPH_FINALLY_CLEAN(1); /* ownership of rec transferred to attrs */
   }
 
-  rec = VECTOR(*attrs)[id];
-  na = (igraph_strvector_t *) rec->value;
+  prec = igraph_attribute_record_list_get_ptr(attrs, id);
+  na = prec->value.as_strvector;
   if (igraph_strvector_size(na) <= elem_id) {
     IGRAPH_CHECK(igraph_strvector_resize(na, elem_id+1));
   }
@@ -2672,7 +2655,9 @@ static igraph_error_t add_string_vertex_attribute(const char *name,
   return add_string_attribute(context->vertex_attribute_names,
                               context->vertex_attributes,
                               context->vcount,
-                              name, context->actvertex-1,
+                              name,
+                              get_default_value_for_string_vattr(name),
+                              context->actvertex-1,
                               value, len);
 }
 
@@ -2684,7 +2669,9 @@ static igraph_error_t add_string_edge_attribute(const char *name,
   return add_string_attribute(context->edge_attribute_names,
                               context->edge_attributes,
                               context->actedge,
-                              name, context->actedge-1,
+                              name,
+                              get_default_value_for_string_eattr(name),
+                              context->actedge-1,
                               value, len);
 }
 
@@ -2695,7 +2682,9 @@ static igraph_error_t add_numeric_vertex_attribute(const char *name,
   return add_numeric_attribute(context->vertex_attribute_names,
                                context->vertex_attributes,
                                context->vcount,
-                               name, context->actvertex-1,
+                               name,
+                               get_default_value_for_numeric_vattr(name),
+                               context->actvertex-1,
                                value);
 }
 
@@ -2706,7 +2695,9 @@ static igraph_error_t add_numeric_edge_attribute(const char *name,
   return add_numeric_attribute(context->edge_attribute_names,
                                context->edge_attributes,
                                context->actedge,
-                               name, context->actedge-1,
+                               name,
+                               get_default_value_for_numeric_eattr(name),
+                               context->actedge-1,
                                value);
 }
 
@@ -2714,10 +2705,10 @@ static igraph_error_t add_bipartite_type(igraph_i_pajek_parsedata_t *context) {
 
   const char *attrname="type";
   igraph_trie_t *names=context->vertex_attribute_names;
-  igraph_vector_ptr_t *attrs=context->vertex_attributes;
-  igraph_integer_t n=context->vcount, n1=context->vcount2;
-  igraph_integer_t attrid, attrsize = igraph_trie_size(names);
-  igraph_attribute_record_t *rec;
+  igraph_attribute_record_list_t *attrs=context->vertex_attributes;
+  igraph_int_t n=context->vcount, n1=context->vcount2;
+  igraph_int_t attrid, attrsize = igraph_trie_size(names);
+  igraph_attribute_record_t* rec;
   igraph_vector_bool_t *na;
 
   if (n1 > n) {
@@ -2732,29 +2723,13 @@ static igraph_error_t add_bipartite_type(igraph_i_pajek_parsedata_t *context) {
   IGRAPH_ASSERT(attrid == attrsize);
 
   /* add a new attribute */
-  rec = IGRAPH_CALLOC(1, igraph_attribute_record_t);
-  CHECK_OOM_RP(rec);
-  IGRAPH_FINALLY(igraph_free, rec);
+  IGRAPH_CHECK(igraph_attribute_record_list_push_back_new(attrs, &rec));
+  IGRAPH_CHECK(igraph_attribute_record_set_name(rec, attrname));
+  IGRAPH_CHECK(igraph_attribute_record_set_type(rec, IGRAPH_ATTRIBUTE_BOOLEAN));
+  IGRAPH_CHECK(igraph_attribute_record_resize(rec, n));
 
-  na = IGRAPH_CALLOC(1, igraph_vector_bool_t);
-  CHECK_OOM_RP(na);
-  IGRAPH_FINALLY(igraph_free, na);
-  IGRAPH_VECTOR_BOOL_INIT_FINALLY(na, n);
-
-  rec->name = strdup(attrname);
-  CHECK_OOM_RP(rec->name);
-  IGRAPH_FINALLY(igraph_free, (char *) rec->name);
-
-  rec->type = IGRAPH_ATTRIBUTE_BOOLEAN;
-  rec->value = na;
-
-  IGRAPH_CHECK(igraph_vector_ptr_push_back(attrs, rec));
-  IGRAPH_FINALLY_CLEAN(4); /* ownership of 'rec' transferred to 'attrs' */
-
-  for (igraph_integer_t i=0; i<n1; i++) {
-    VECTOR(*na)[i] = false;
-  }
-  for (igraph_integer_t i=n1; i<n; i++) {
+  na = rec->value.as_vector_bool;
+  for (igraph_int_t i = n1; i < n; i++) {
     VECTOR(*na)[i] = true;
   }
 
@@ -2763,12 +2738,12 @@ static igraph_error_t add_bipartite_type(igraph_i_pajek_parsedata_t *context) {
 
 static igraph_error_t check_bipartite(igraph_i_pajek_parsedata_t *context) {
   const igraph_vector_int_t *edges=context->vector;
-  igraph_integer_t n1=context->vcount2;
-  igraph_integer_t ne=igraph_vector_int_size(edges);
+  igraph_int_t n1=context->vcount2;
+  igraph_int_t ne=igraph_vector_int_size(edges);
 
-  for (igraph_integer_t i=0; i<ne; i+=2) {
-    igraph_integer_t v1 = VECTOR(*edges)[i];
-    igraph_integer_t v2 = VECTOR(*edges)[i+1];
+  for (igraph_int_t i=0; i<ne; i+=2) {
+    igraph_int_t v1 = VECTOR(*edges)[i];
+    igraph_int_t v2 = VECTOR(*edges)[i+1];
     if ( (v1 < n1 && v2 < n1) || (v1 > n1 && v2 > n1) ) {
       IGRAPH_WARNING("Invalid edge in bipartite graph.");
     }
@@ -2794,7 +2769,7 @@ static igraph_bool_t is_standard_vattr(const char *attrname) {
     "font", "url", "color", "framecolor",
     "labelcolor"
   };
-  for (size_t i=0; i < sizeof(names) / sizeof(names[0]); i++) {
+  for (size_t i = 0; i < sizeof(names) / sizeof(names[0]); i++) {
     if (strcmp(attrname, names[i]) == 0) {
       return true;
     }
@@ -2837,3 +2812,108 @@ static igraph_error_t deconflict_attrname(char **attrname) {
   return IGRAPH_SUCCESS;
 }
 
+typedef struct {
+  const char* name;
+  igraph_real_t default_value;
+} attribute_numeric_defaults_t;
+
+typedef struct {
+  const char* name;
+  const char* default_value;
+} attribute_string_defaults_t;
+
+/* The defaults listed below are Pajek's built-in defaults unless the user
+ * overrides them.
+ *
+ * See: https://nascol.discourse.group/t/pajek-file-format-default-values-for-attributes/38/2
+ */
+
+const attribute_numeric_defaults_t vattr_numeric_defaults[] = {
+  { "xfact", 1 },
+  { "yfact", 1 },
+  { "labeldist", 20 },
+  { "labeldegree2", 285 },
+  { "framewidth", 1 },
+  { "fontsize", 15 },
+  { "rotation", 0 },
+  { "radius", 0 },
+  { "diamondratio", 0.01 },
+  { "labeldegree", 0 },
+  { 0 }
+};
+
+const attribute_string_defaults_t vattr_string_defaults[] = {
+  { "color", "LightOrange" },
+  { "framecolor", "Brown" },
+  { "labelcolor", "Maroon" },
+  { 0 }
+};
+
+const attribute_numeric_defaults_t eattr_numeric_defaults[] = {
+  { "arrowsize", 1 },
+  { "edgewidth", 2 },
+  { "hook1", 0 },
+  { "hook2", 0 },
+  { "angle1", 0 },
+  { "angle2", 0 },
+  { "velocity1", 1 },
+  { "velocity2", 1 },
+  { "arrowpos", 0 },
+  { "labelpos", 0.5 },
+  { "labelangle", 10 },
+  { "labelangle2", 90 },
+  { "labeldegree", 0 },
+  { "fontsize", 15 },
+  { 0 }
+};
+
+const attribute_string_defaults_t eattr_string_defaults[] = {
+  { "color", "MidnightBlue" },
+  { "labelcolor", "Black" },
+  { 0 }
+};
+
+static igraph_real_t get_default_value_for_numeric_attr(
+  const char *attrname, const attribute_numeric_defaults_t* table
+) {
+  const attribute_numeric_defaults_t* ptr;
+
+  for (ptr = table; ptr->name != 0; ptr++) {
+    if (!strcmp(attrname, ptr->name)) {
+      return ptr->default_value;
+    }
+  }
+
+  return IGRAPH_NAN;
+}
+
+static const char* get_default_value_for_string_attr(
+  const char *attrname, const attribute_string_defaults_t* table
+) {
+  const attribute_string_defaults_t* ptr;
+
+  for (ptr = table; ptr->name != 0; ptr++) {
+    if (!strcmp(attrname, ptr->name)) {
+      return ptr->default_value;
+    }
+  }
+
+  return "";
+}
+
+static igraph_real_t get_default_value_for_numeric_vattr(const char *attrname) {
+  return get_default_value_for_numeric_attr(attrname, vattr_numeric_defaults);
+}
+
+static igraph_real_t get_default_value_for_numeric_eattr(const char *attrname) {
+  return get_default_value_for_numeric_attr(attrname, eattr_numeric_defaults);
+}
+
+static const char* get_default_value_for_string_vattr(const char *attrname) {
+  return get_default_value_for_string_attr(attrname, vattr_string_defaults);
+}
+
+static const char* get_default_value_for_string_eattr(const char *attrname) {
+  return get_default_value_for_string_attr(attrname, eattr_string_defaults);
+}
+
