@@ -95,11 +95,11 @@ graph::graph(const igraph_t *igraph,
     // scan .int file for node info
     highest_sim = 1.0;
     num_nodes = igraph_vcount(igraph);
-    igraph_integer_t no_of_edges = igraph_ecount(igraph);
-    for (igraph_integer_t i = 0; i < num_nodes; i++) {
+    igraph_int_t no_of_edges = igraph_ecount(igraph);
+    for (igraph_int_t i = 0; i < num_nodes; i++) {
         id_catalog[i] = 1;
     }
-    map< igraph_integer_t, igraph_integer_t>::iterator cat_iter;
+    map< igraph_int_t, igraph_int_t>::iterator cat_iter;
     for ( cat_iter = id_catalog.begin();
           cat_iter != id_catalog.end(); cat_iter++) {
         cat_iter->second = cat_iter->first;
@@ -114,9 +114,9 @@ graph::graph(const igraph_t *igraph,
     }
 
     // read .int file for graph info
-    igraph_integer_t node_1, node_2;
+    igraph_int_t node_1, node_2;
     igraph_real_t weight;
-    for (igraph_integer_t i = 0; i < no_of_edges; i++) {
+    for (igraph_int_t i = 0; i < no_of_edges; i++) {
         node_1 = IGRAPH_FROM(igraph, i);
         node_2 = IGRAPH_TO(igraph, i);
         weight = weights ? VECTOR(*weights)[i] : 1.0 ;
@@ -153,7 +153,7 @@ void graph::init_parms ( int rand_seed, float edge_cut, float real_parm ) {
     cut_rate = ( cut_length_start - cut_length_end ) / 400.0;
 
     // finally set the number of iterations to leave .real coords fixed
-    igraph_integer_t full_comp_iters;
+    igraph_int_t full_comp_iters;
     full_comp_iters = liquid.iterations + expansion.iterations +
                       cooldown.iterations + crunch.iterations + 3;
 
@@ -198,8 +198,8 @@ void graph::init_parms(const igraph_layout_drl_options_t *options) {
 }
 
 int graph::read_real(const igraph_matrix_t *real_mat) {
-    igraph_integer_t n = igraph_matrix_nrow(real_mat);
-    for (igraph_integer_t i = 0; i < n; i++) {
+    igraph_int_t n = igraph_matrix_nrow(real_mat);
+    for (igraph_int_t i = 0; i < n; i++) {
         positions[id_catalog[i]].x = MATRIX(*real_mat, i, 0);
         positions[id_catalog[i]].y = MATRIX(*real_mat, i, 1);
         positions[id_catalog[i]].z = MATRIX(*real_mat, i, 2);
@@ -466,7 +466,7 @@ int graph::ReCompute( ) {
 
 void graph::update_nodes ( ) {
 
-    vector<igraph_integer_t> node_indices;           // node list of nodes currently being updated
+    vector<igraph_int_t> node_indices;           // node list of nodes currently being updated
     float old_positions[2 * MAX_PROCS]; // positions before update
     float new_positions[2 * MAX_PROCS]; // positions after update
 
@@ -479,9 +479,9 @@ void graph::update_nodes ( ) {
 
     // next we calculate the number of nodes there would be if the
     // num_nodes by num_procs schedule grid were perfectly square
-    igraph_integer_t square_num_nodes = (igraph_integer_t)(num_procs + num_procs * floor ((float)(num_nodes - 1) / (float)num_procs ));
+    igraph_int_t square_num_nodes = (igraph_int_t)(num_procs + num_procs * floor ((float)(num_nodes - 1) / (float)num_procs ));
 
-    for ( igraph_integer_t i = myid; i < square_num_nodes; i += num_procs ) {
+    for ( igraph_int_t i = myid; i < square_num_nodes; i += num_procs ) {
 
         // get old positions
         get_positions ( node_indices, old_positions );
@@ -541,7 +541,7 @@ void graph::update_nodes ( ) {
 // The get_positions function takes the node_indices list
 // and returns the corresponding positions in an array.
 
-void graph::get_positions ( vector<igraph_integer_t> &node_indices,
+void graph::get_positions ( vector<igraph_int_t> &node_indices,
                             float return_positions[3 * MAX_PROCS]  ) {
 
     // fill positions
@@ -558,7 +558,7 @@ void graph::get_positions ( vector<igraph_integer_t> &node_indices,
 // of active processes at this level for use by the random number
 // generators.
 
-void graph::update_node_pos ( igraph_integer_t node_ind,
+void graph::update_node_pos ( igraph_int_t node_ind,
                               float old_positions[3 * MAX_PROCS],
                               float new_positions[3 * MAX_PROCS] ) {
 
@@ -636,7 +636,7 @@ void graph::update_node_pos ( igraph_integer_t node_ind,
 // updates the positions by subtracting the old positions and adding the
 // new positions to the density grid.
 
-void graph::update_density ( vector<igraph_integer_t> &node_indices,
+void graph::update_density ( vector<igraph_int_t> &node_indices,
                              float old_positions[3 * MAX_PROCS],
                              float new_positions[3 * MAX_PROCS] ) {
 
@@ -664,13 +664,13 @@ void graph::update_density ( vector<igraph_integer_t> &node_indices,
 * original code by B. Wylie.                *
 *********************************************/
 
-float graph::Compute_Node_Energy( igraph_integer_t node_ind ) {
+float graph::Compute_Node_Energy( igraph_int_t node_ind ) {
 
     /* Want to expand 4th power range of attraction */
     float attraction_factor = attraction * attraction *
                               attraction * attraction * 2e-2;
 
-    map <igraph_integer_t, float>::iterator EI;
+    map <igraph_int_t, float>::iterator EI;
     float x_dis, y_dis, z_dis;
     float energy_distance, weight;
     float node_energy = 0;
@@ -722,10 +722,10 @@ float graph::Compute_Node_Energy( igraph_integer_t node_ind ) {
 * originally written by B. Wylie             *
 *********************************************/
 
-void graph::Solve_Analytic( igraph_integer_t node_ind, float &pos_x, float &pos_y,
+void graph::Solve_Analytic( igraph_int_t node_ind, float &pos_x, float &pos_y,
                             float &pos_z) {
 
-    map <igraph_integer_t, float>::iterator EI;
+    map <igraph_int_t, float>::iterator EI;
     float total_weight = 0;
     float x_dis, y_dis, z_dis, x_cen = 0, y_cen = 0, z_cen = 0;
     float x = 0, y = 0, z = 0, dis;
@@ -766,7 +766,7 @@ void graph::Solve_Analytic( igraph_integer_t node_ind, float &pos_x, float &pos_
     float num_connections = (float)sqrt((float)neighbors[node_ind].size());
     float maxLength = 0;
 
-    map<igraph_integer_t, float>::iterator maxIndex;
+    map<igraph_int_t, float>::iterator maxIndex;
 
     // Go through nodes edges... cutting if necessary
     for (EI = maxIndex = neighbors[node_ind].begin();
