@@ -1,7 +1,5 @@
-/* -*- mode: C -*-  */
-/* vim:set ts=4 sw=4 sts=4 et: */
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2003-2020  The igraph development team
 
    This program is free software; you can redistribute it and/or modify
@@ -33,7 +31,7 @@
 
 static igraph_real_t igraph_i_distance_between(
         const igraph_matrix_t *c,
-        igraph_integer_t a, igraph_integer_t b);
+        igraph_int_t a, igraph_int_t b);
 
 static void igraph_i_determine_electric_axal_forces(
         const igraph_matrix_t *pos,
@@ -41,14 +39,14 @@ static void igraph_i_determine_electric_axal_forces(
         igraph_real_t *y,
         igraph_real_t directed_force,
         igraph_real_t distance,
-        igraph_integer_t other_node,
-        igraph_integer_t this_node);
+        igraph_int_t other_node,
+        igraph_int_t this_node);
 
 static void igraph_i_apply_electrical_force(
         const igraph_matrix_t *pos,
         igraph_vector_t *pending_forces_x,
         igraph_vector_t *pending_forces_y,
-        igraph_integer_t other_node, igraph_integer_t this_node,
+        igraph_int_t other_node, igraph_int_t this_node,
         igraph_real_t node_charge,
         igraph_real_t distance);
 
@@ -58,15 +56,15 @@ static void igraph_i_determine_spring_axal_forces(
         igraph_real_t directed_force,
         igraph_real_t distance,
         igraph_real_t spring_length,
-        igraph_integer_t other_node,
-        igraph_integer_t this_node);
+        igraph_int_t other_node,
+        igraph_int_t this_node);
 
 static void igraph_i_apply_spring_force(
         const igraph_matrix_t *pos,
         igraph_vector_t *pending_forces_x,
         igraph_vector_t *pending_forces_y,
-        igraph_integer_t other_node,
-        igraph_integer_t this_node, igraph_real_t spring_length,
+        igraph_int_t other_node,
+        igraph_int_t this_node, igraph_real_t spring_length,
         igraph_real_t spring_constant);
 
 static void igraph_i_move_nodes(
@@ -78,7 +76,7 @@ static void igraph_i_move_nodes(
 
 static igraph_real_t igraph_i_distance_between(
         const igraph_matrix_t *c,
-        igraph_integer_t a, igraph_integer_t b) {
+        igraph_int_t a, igraph_int_t b) {
     igraph_real_t diffx = MATRIX(*c, a, 0) - MATRIX(*c, b, 0);
     igraph_real_t diffy = MATRIX(*c, a, 1) - MATRIX(*c, b, 1);
     return sqrt(diffx*diffx + diffy*diffy);
@@ -89,8 +87,8 @@ static void igraph_i_determine_electric_axal_forces(const igraph_matrix_t *pos,
         igraph_real_t *y,
         igraph_real_t directed_force,
         igraph_real_t distance,
-        igraph_integer_t other_node,
-        igraph_integer_t this_node) {
+        igraph_int_t other_node,
+        igraph_int_t this_node) {
 
     // We know what the directed force is.  We now need to translate it
     // into the appropriate x and y components.
@@ -139,7 +137,7 @@ static void igraph_i_apply_electrical_force(
         const igraph_matrix_t *pos,
         igraph_vector_t *pending_forces_x,
         igraph_vector_t *pending_forces_y,
-        igraph_integer_t other_node, igraph_integer_t this_node,
+        igraph_int_t other_node, igraph_int_t this_node,
         igraph_real_t node_charge,
         igraph_real_t distance) {
 
@@ -163,7 +161,7 @@ static void igraph_i_determine_spring_axal_forces(
         igraph_real_t directed_force,
         igraph_real_t distance,
         igraph_real_t spring_length,
-        igraph_integer_t other_node, igraph_integer_t this_node) {
+        igraph_int_t other_node, igraph_int_t this_node) {
 
     // if the spring is just the right size, the forces will be 0, so we can
     // skip the computation.
@@ -198,8 +196,8 @@ static void igraph_i_apply_spring_force(
         const igraph_matrix_t *pos,
         igraph_vector_t *pending_forces_x,
         igraph_vector_t *pending_forces_y,
-        igraph_integer_t other_node,
-        igraph_integer_t this_node, igraph_real_t spring_length,
+        igraph_int_t other_node,
+        igraph_int_t this_node, igraph_real_t spring_length,
         igraph_real_t spring_constant) {
 
     // determined using Hooke's Law:
@@ -260,7 +258,7 @@ static void igraph_i_move_nodes(
     //   velocity     = force / mass
     //   displacement = force / mass
 
-    igraph_integer_t this_node, no_of_nodes = igraph_vector_size(pending_forces_x);
+    igraph_int_t this_node, no_of_nodes = igraph_vector_size(pending_forces_x);
 
     for (this_node = 0; this_node < no_of_nodes; this_node++) {
 
@@ -302,7 +300,11 @@ static void igraph_i_move_nodes(
  * anything like that, so a stable fixed point is not guaranteed.)
  *
  * </para><para>
- * See also http://www.schmuhl.org/graphopt/ for the original graphopt.
+ * See also
+ * https://web.archive.org/web/20220611030748/http://www.schmuhl.org/graphopt/
+ * and
+ * https://sourceforge.net/projects/graphopt/
+ * for the original graphopt.
  *
  * \param graph The input graph.
  * \param res Pointer to an initialized matrix, the result will be stored here
@@ -337,23 +339,23 @@ static void igraph_i_move_nodes(
  * of edges. If \p node_charge is zero then it is only O(n|E|).
  */
 igraph_error_t igraph_layout_graphopt(const igraph_t *graph, igraph_matrix_t *res,
-                           igraph_integer_t niter,
+                           igraph_int_t niter,
                            igraph_real_t node_charge, igraph_real_t node_mass,
                            igraph_real_t spring_length,
                            igraph_real_t spring_constant,
                            igraph_real_t max_sa_movement,
                            igraph_bool_t use_seed) {
 
-    igraph_integer_t no_of_nodes = igraph_vcount(graph);
-    igraph_integer_t no_of_edges = igraph_ecount(graph);
+    igraph_int_t no_of_nodes = igraph_vcount(graph);
+    igraph_int_t no_of_edges = igraph_ecount(graph);
     igraph_vector_t pending_forces_x, pending_forces_y;
     /* Set a flag to calculate (or not) the electrical forces that the nodes */
     /* apply on each other based on if both node types' charges are zero. */
     igraph_bool_t apply_electric_charges = (node_charge != 0);
 
-    igraph_integer_t this_node, other_node, edge;
+    igraph_int_t this_node, other_node, edge;
     igraph_real_t distance;
-    igraph_integer_t i;
+    igraph_int_t i;
 
     IGRAPH_VECTOR_INIT_FINALLY(&pending_forces_x, no_of_nodes);
     IGRAPH_VECTOR_INIT_FINALLY(&pending_forces_y, no_of_nodes);
@@ -412,8 +414,8 @@ igraph_error_t igraph_layout_graphopt(const igraph_t *graph, igraph_matrix_t *re
 
         // Apply force from springs
         for (edge = 0; edge < no_of_edges; edge++) {
-            igraph_integer_t tthis_node = IGRAPH_FROM(graph, edge);
-            igraph_integer_t oother_node = IGRAPH_TO(graph, edge);
+            igraph_int_t tthis_node = IGRAPH_FROM(graph, edge);
+            igraph_int_t oother_node = IGRAPH_TO(graph, edge);
             // Apply spring force on both nodes
             igraph_i_apply_spring_force(res, &pending_forces_x, &pending_forces_y,
                                         oother_node, tthis_node, spring_length,
