@@ -1,6 +1,5 @@
-/* -*- mode: C -*-  */
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2010-2012  Gabor Csardi <csardi.gabor@gmail.com>
    334 Harvard street, Cambridge, MA 02139 USA
 
@@ -30,7 +29,7 @@
 #include "igraph_pmt.h"
 #include "igraph_vector_type.h"
 #include "igraph_vector_pmt.h"
-#include "../core/vector.pmt"
+#include "core/vector.pmt"
 #include "igraph_pmt_off.h"
 #undef BASE_FORTRAN_INT
 
@@ -38,11 +37,11 @@
 static igraph_error_t igraph_vector_int_update_from_fortran(
     igraph_vector_int_t* vec, const igraph_vector_fortran_int_t* fortran_vec
 ) {
-    igraph_integer_t size = igraph_vector_fortran_int_size(fortran_vec);
+    igraph_int_t size = igraph_vector_fortran_int_size(fortran_vec);
 
     IGRAPH_CHECK(igraph_vector_int_resize(vec, size));
 
-    for (igraph_integer_t i = 0; i < size; i++) {
+    for (igraph_int_t i = 0; i < size; i++) {
         VECTOR(*vec)[i] = VECTOR(*fortran_vec)[i];
     }
 
@@ -53,7 +52,7 @@ static igraph_error_t igraph_vector_int_update_from_fortran(
 static igraph_error_t igraph_vector_int_copy_to_fortran(
     const igraph_vector_int_t* vec, igraph_vector_fortran_int_t* fortran_vec
 ) {
-    igraph_integer_t i, size = igraph_vector_int_size(vec);
+    igraph_int_t i, size = igraph_vector_int_size(vec);
 
     IGRAPH_CHECK(igraph_vector_fortran_int_resize(fortran_vec, size));
 
@@ -128,25 +127,25 @@ igraph_error_t igraph_lapack_dgetrf(igraph_matrix_t *a, igraph_vector_int_t *ipi
     } else if (*info < 0) {
         switch (*info) {
         case -1:
-            IGRAPH_ERROR("Invalid number of rows.", IGRAPH_ELAPACK);
+            IGRAPH_ERROR("Invalid number of rows.", IGRAPH_FAILURE);
             break;
         case -2:
-            IGRAPH_ERROR("Invalid number of columns.", IGRAPH_ELAPACK);
+            IGRAPH_ERROR("Invalid number of columns.", IGRAPH_FAILURE);
             break;
         case -3:
-            IGRAPH_ERROR("Invalid input matrix.", IGRAPH_ELAPACK);
+            IGRAPH_ERROR("Invalid input matrix.", IGRAPH_FAILURE);
             break;
         case -4:
-            IGRAPH_ERROR("Invalid LDA parameter.", IGRAPH_ELAPACK);
+            IGRAPH_ERROR("Invalid LDA parameter.", IGRAPH_FAILURE);
             break;
         case -5:
-            IGRAPH_ERROR("Invalid pivot vector.", IGRAPH_ELAPACK);
+            IGRAPH_ERROR("Invalid pivot vector.", IGRAPH_FAILURE);
             break;
         case -6:
-            IGRAPH_ERROR("Invalid info argument.", IGRAPH_ELAPACK);
+            IGRAPH_ERROR("Invalid info argument.", IGRAPH_FAILURE);
             break;
         default:
-            IGRAPH_ERROR("Unknown LAPACK error.", IGRAPH_ELAPACK);
+            IGRAPH_ERROR("Unknown LAPACK error.", IGRAPH_FAILURE);
             break;
         }
     }
@@ -208,7 +207,7 @@ igraph_error_t igraph_lapack_dgetrs(igraph_bool_t transpose, const igraph_matrix
     ldb = n > 0 ? n : 1;
 
     if (n != igraph_matrix_ncol(a)) {
-        IGRAPH_ERROR("Cannot LU solve matrix.", IGRAPH_NONSQUARE);
+        IGRAPH_ERROR("Cannot LU solve non-square matrix.", IGRAPH_EINVAL);
     }
     if (n != igraph_matrix_nrow(b)) {
         IGRAPH_ERROR("Cannot LU solve matrix, RHS of wrong size.", IGRAPH_EINVAL);
@@ -233,34 +232,34 @@ igraph_error_t igraph_lapack_dgetrs(igraph_bool_t transpose, const igraph_matrix
     if (info < 0) {
         switch (info) {
         case -1:
-            IGRAPH_ERROR("Invalid transpose argument.", IGRAPH_ELAPACK);
+            IGRAPH_ERROR("Invalid transpose argument.", IGRAPH_FAILURE);
             break;
         case -2:
-            IGRAPH_ERROR("Invalid number of rows/columns.", IGRAPH_ELAPACK);
+            IGRAPH_ERROR("Invalid number of rows/columns.", IGRAPH_FAILURE);
             break;
         case -3:
-            IGRAPH_ERROR("Invalid number of RHS vectors.", IGRAPH_ELAPACK);
+            IGRAPH_ERROR("Invalid number of RHS vectors.", IGRAPH_FAILURE);
             break;
         case -4:
-            IGRAPH_ERROR("Invalid LU matrix.", IGRAPH_ELAPACK);
+            IGRAPH_ERROR("Invalid LU matrix.", IGRAPH_FAILURE);
             break;
         case -5:
-            IGRAPH_ERROR("Invalid LDA parameter.", IGRAPH_ELAPACK);
+            IGRAPH_ERROR("Invalid LDA parameter.", IGRAPH_FAILURE);
             break;
         case -6:
-            IGRAPH_ERROR("Invalid pivot vector.", IGRAPH_ELAPACK);
+            IGRAPH_ERROR("Invalid pivot vector.", IGRAPH_FAILURE);
             break;
         case -7:
-            IGRAPH_ERROR("Invalid RHS matrix.", IGRAPH_ELAPACK);
+            IGRAPH_ERROR("Invalid RHS matrix.", IGRAPH_FAILURE);
             break;
         case -8:
-            IGRAPH_ERROR("Invalid LDB parameter.", IGRAPH_ELAPACK);
+            IGRAPH_ERROR("Invalid LDB parameter.", IGRAPH_FAILURE);
             break;
         case -9:
-            IGRAPH_ERROR("Invalid info argument.", IGRAPH_ELAPACK);
+            IGRAPH_ERROR("Invalid info argument.", IGRAPH_FAILURE);
             break;
         default:
-            IGRAPH_ERROR("Unknown LAPACK error.", IGRAPH_ELAPACK);
+            IGRAPH_ERROR("Unknown LAPACK error.", IGRAPH_FAILURE);
             break;
         }
     }
@@ -320,7 +319,7 @@ igraph_error_t igraph_lapack_dgesv(igraph_matrix_t *a, igraph_vector_int_t *ipiv
     igraph_vector_fortran_int_t vipiv;
 
     if (n != igraph_matrix_ncol(a)) {
-        IGRAPH_ERROR("Cannot LU solve matrix.", IGRAPH_NONSQUARE);
+        IGRAPH_ERROR("Cannot LU solve non-square matrix.", IGRAPH_EINVAL);
     }
     if (n != igraph_matrix_nrow(b)) {
         IGRAPH_ERROR("Cannot LU solve matrix, RHS of wrong size.", IGRAPH_EINVAL);
@@ -337,31 +336,31 @@ igraph_error_t igraph_lapack_dgesv(igraph_matrix_t *a, igraph_vector_int_t *ipiv
     } else if (*info < 0) {
         switch (*info) {
         case -1:
-            IGRAPH_ERROR("Invalid number of rows/column.", IGRAPH_ELAPACK);
+            IGRAPH_ERROR("Invalid number of rows/column.", IGRAPH_FAILURE);
             break;
         case -2:
-            IGRAPH_ERROR("Invalid number of RHS vectors.", IGRAPH_ELAPACK);
+            IGRAPH_ERROR("Invalid number of RHS vectors.", IGRAPH_FAILURE);
             break;
         case -3:
-            IGRAPH_ERROR("Invalid input matrix.", IGRAPH_ELAPACK);
+            IGRAPH_ERROR("Invalid input matrix.", IGRAPH_FAILURE);
             break;
         case -4:
-            IGRAPH_ERROR("Invalid LDA parameter.", IGRAPH_ELAPACK);
+            IGRAPH_ERROR("Invalid LDA parameter.", IGRAPH_FAILURE);
             break;
         case -5:
-            IGRAPH_ERROR("Invalid pivot vector.", IGRAPH_ELAPACK);
+            IGRAPH_ERROR("Invalid pivot vector.", IGRAPH_FAILURE);
             break;
         case -6:
-            IGRAPH_ERROR("Invalid RHS matrix.", IGRAPH_ELAPACK);
+            IGRAPH_ERROR("Invalid RHS matrix.", IGRAPH_FAILURE);
             break;
         case -7:
-            IGRAPH_ERROR("Invalid LDB parameter.", IGRAPH_ELAPACK);
+            IGRAPH_ERROR("Invalid LDB parameter.", IGRAPH_FAILURE);
             break;
         case -8:
-            IGRAPH_ERROR("Invalid info argument.", IGRAPH_ELAPACK);
+            IGRAPH_ERROR("Invalid info argument.", IGRAPH_FAILURE);
             break;
         default:
-            IGRAPH_ERROR("Unknown LAPACK error.", IGRAPH_ELAPACK);
+            IGRAPH_ERROR("Unknown LAPACK error.", IGRAPH_FAILURE);
             break;
         }
     }
@@ -381,7 +380,7 @@ igraph_error_t igraph_lapack_dgesv(igraph_matrix_t *a, igraph_vector_int_t *ipiv
  * \brief Selected eigenvalues and optionally eigenvectors of a symmetric matrix.
  *
  * Calls the DSYEVR LAPACK function to compute selected eigenvalues
- * and, optionally, eigenvectors of a real symmetric matrix A.
+ * and, optionally, eigenvectors of a real symmetric matrix \c A.
  * Eigenvalues and eigenvectors can be selected by specifying either
  * a range of values or a range of indices for the desired eigenvalues.
  *
@@ -395,7 +394,7 @@ igraph_error_t igraph_lapack_dgesv(igraph_matrix_t *a, igraph_vector_int_t *ipiv
  *        the corresponding eigenvectors) to calculate. Possible
  *        values are \c IGRAPH_LAPACK_DSYEV_ALL, all eigenvalues;
  *        \c IGRAPH_LAPACK_DSYEV_INTERVAL, all eigenvalues in the
- *        half-open interval (vl,vu];
+ *        half-open interval <code>(vl, vu]</code>;
  *        \c IGRAPH_LAPACK_DSYEV_SELECT, the il-th through iu-th
  *        eigenvalues.
  * \param vl If \p which is \c IGRAPH_LAPACK_DSYEV_INTERVAL, then
@@ -405,7 +404,7 @@ igraph_error_t igraph_lapack_dgesv(igraph_matrix_t *a, igraph_vector_int_t *ipiv
  *        this is the upper bound of the interval to be searched for
  *        eigenvalues. See also the \p vestimate argument.
  * \param vestimate An upper bound for the number of eigenvalues in
- *        the (vl,vu] interval, if \p which is \c
+ *        the <code>(vl, vu]</code> interval, if \p which is \c
  *        IGRAPH_LAPACK_DSYEV_INTERVAL. Memory is allocated only for
  *        the given number of eigenvalues (and eigenvectors), so this
  *        upper bound must be correct.
@@ -415,14 +414,14 @@ igraph_error_t igraph_lapack_dgesv(igraph_matrix_t *a, igraph_vector_int_t *ipiv
  *        which is \c IGRAPH_LAPACK_DSYEV_SELECT.
  * \param abstol The absolute error tolerance for the eigevalues. An
  *        approximate eigenvalue is accepted as converged when it is
- *        determined to lie in an interval [a,b] of width less than or
- *        equal to abstol + EPS * max(|a|,|b|), where EPS is the
- *        machine precision.
+ *        determined to lie in an interval <code>[a,b]</code> of width
+ *        less than or equal to <code>abstol + EPS * max(|a|,|b|)</code>,
+ *        where \c EPS is the machine precision.
  * \param values An initialized vector, the eigenvalues are stored
  *        here, unless it is a null pointer. It will be resized as
  *        needed.
- * \param vectors An initialized matrix, the eigenvectors are stored
- *        in its columns, unless it is a null pointer. It will be
+ * \param vectors An initialized matrix. A set of orthonormal eigenvectors
+ *        are stored in its columns, unless it is a null pointer. It will be
  *        resized as needed.
  * \param support An integer vector. If not a null pointer, then it
  *        will be resized to (2*max(1,M)) (M is a the total number of
@@ -459,7 +458,7 @@ igraph_error_t igraph_lapack_dsyevr(const igraph_matrix_t *A,
     int lwork = -1, liwork = -1;
 
     if (n != igraph_matrix_ncol(A)) {
-        IGRAPH_ERROR("Cannot find eigenvalues/vectors.", IGRAPH_NONSQUARE);
+        IGRAPH_ERROR("Cannot find eigenvalues/vectors of non-square matrix.", IGRAPH_EINVAL);
     }
     if (which == IGRAPH_LAPACK_DSYEV_INTERVAL &&
         (vestimate < 1 || vestimate > n)) {
@@ -641,7 +640,7 @@ igraph_error_t igraph_lapack_dgeev(const igraph_matrix_t *A,
     int error = *info;
 
     if (igraph_matrix_ncol(A) != n) {
-        IGRAPH_ERROR("Cannot calculate eigenvalues (dgeev).", IGRAPH_NONSQUARE);
+        IGRAPH_ERROR("Cannot calculate eigenvalues of non-square matrix.", IGRAPH_EINVAL);
     }
 
     IGRAPH_CHECK(igraph_matrix_init_copy(&Acopy, A));
@@ -684,10 +683,10 @@ igraph_error_t igraph_lapack_dgeev(const igraph_matrix_t *A,
                  VECTOR(work), &lwork, info);
 
     if (*info < 0) {
-        IGRAPH_ERROR("Cannot calculate eigenvalues (dgeev).", IGRAPH_ELAPACK);
+        IGRAPH_ERROR("Cannot calculate eigenvalues (dgeev).", IGRAPH_FAILURE);
     } else if (*info > 0) {
         if (error) {
-            IGRAPH_ERROR("Cannot calculate eigenvalues (dgeev).", IGRAPH_ELAPACK);
+            IGRAPH_ERROR("Cannot calculate eigenvalues (dgeev).", IGRAPH_FAILURE);
         } else {
             IGRAPH_WARNING("Cannot calculate eigenvalues (dgeev).");
         }
@@ -868,7 +867,7 @@ igraph_error_t igraph_lapack_dgeevx(igraph_lapack_dgeevx_balance_t balance,
         ihi = &ihi_dummy;
     }
     if (igraph_matrix_ncol(A) != n) {
-        IGRAPH_ERROR("Cannot calculate eigenvalues (dgeevx).", IGRAPH_NONSQUARE);
+        IGRAPH_ERROR("Cannot calculate eigenvalues of non-square matrix.", IGRAPH_EINVAL);
     }
 
     switch (balance) {
@@ -953,10 +952,10 @@ igraph_error_t igraph_lapack_dgeevx(igraph_lapack_dgeevx_balance_t balance,
                   VECTOR(work), &lwork, VECTOR(iwork), info);
 
     if (*info < 0) {
-        IGRAPH_ERROR("Cannot calculate eigenvalues (dgeev).", IGRAPH_ELAPACK);
+        IGRAPH_ERROR("Cannot calculate eigenvalues (dgeev).", IGRAPH_FAILURE);
     } else if (*info > 0) {
         if (error) {
-            IGRAPH_ERROR("Cannot calculate eigenvalues (dgeev).", IGRAPH_ELAPACK);
+            IGRAPH_ERROR("Cannot calculate eigenvalues (dgeev).", IGRAPH_FAILURE);
         } else {
             IGRAPH_WARNING("Cannot calculate eigenvalues (dgeev).");
         }
@@ -1003,7 +1002,7 @@ igraph_error_t igraph_lapack_dgehrd(const igraph_matrix_t *A,
     int i;
 
     if (igraph_matrix_ncol(A) != n) {
-        IGRAPH_ERROR("Hessenberg reduction failed.", IGRAPH_NONSQUARE);
+        IGRAPH_ERROR("Hessenberg reduction failed on non-square matrix.", IGRAPH_EINVAL);
     }
 
     if (ilo < 1 || ihi > n || ilo > ihi) {
