@@ -287,7 +287,7 @@ static igraph_error_t igraph_i_neisets_intersect(
  * \param graph The graph object to analyze
  * \param res Pointer to a matrix, the result of the calculation will
  *        be stored here. The number of its rows and columns is the same
- *        as the number of vertex IDs in \p vit_from and \p vit_to, respectively.
+ *        as the number of vertex IDs in \p from and \p to, respectively.
  * \param from The vertex IDs of the first set of vertices of the
  *        pairs for which the calculation will be done.
  * \param to The vertex IDs of the second set of vertices of the
@@ -585,10 +585,10 @@ igraph_error_t igraph_similarity_jaccard_es(const igraph_t *graph, igraph_vector
  * \param graph The graph object to analyze.
  * \param res Pointer to a matrix, the result of the calculation will
  *        be stored here. The number of its rows and columns is the same
- *        as the number of vertex IDs in \p vit_from and \p vit_to, respectively.
- * \param vit_from The vertex IDs of the first vertices of the
+ *        as the number of vertex IDs in \p from and \p to, respectively.
+ * \param from The vertex IDs of the first vertices of the
  *        pairs for which the calculation will be done.
- * \param vit_to The vertex IDs of the second vertices of the
+ * \param to The vertex IDs of the second vertices of the
  *        pairs for which the calculation will be done.
  * \param mode The type of neighbors to be used for the calculation in
  *        directed graphs. Possible values:
@@ -623,10 +623,10 @@ igraph_error_t igraph_similarity_jaccard_es(const igraph_t *graph, igraph_vector
  * \example examples/simple/igraph_similarity.c
  */
 igraph_error_t igraph_similarity_dice(const igraph_t *graph, igraph_matrix_t *res,
-                                      const igraph_vs_t vit_from, const igraph_vs_t vit_to,
+                                      const igraph_vs_t from, const igraph_vs_t to,
                                       igraph_neimode_t mode, igraph_bool_t loops) {
 
-    IGRAPH_CHECK(igraph_similarity_jaccard(graph, res, vit_from, vit_to, mode, loops));
+    IGRAPH_CHECK(igraph_similarity_jaccard(graph, res, from, to, mode, loops));
 
     igraph_int_t nr = igraph_matrix_nrow(res);
     igraph_int_t nc = igraph_matrix_ncol(res);
