@@ -1,7 +1,5 @@
-/* -*- mode: C -*-  */
-/* vim:set ts=4 sw=4 sts=4 et: */
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2005-2012  Gabor Csardi <csardi.gabor@gmail.com>
    334 Harvard street, Cambridge, MA 02139 USA
 
@@ -40,32 +38,102 @@
  * The density of a graph is simply the ratio of the actual number of its
  * edges and the largest possible number of edges it could have.
  * The maximum number of edges depends on interpretation: are vertices
- * allowed to have a connected to themselves? This is controlled by the
+ * allowed to have a connection to themselves? This is controlled by the
  * \p loops parameter.
  *
  * </para><para>
- * Note that density is ill-defined for graphs which have multiple edges
- * between some pairs of vertices. Consider calling \ref igraph_simplify()
- * on such graphs.
+ * The classic definition of the density is formulated for unweighted
+ * graphs without multi-edges. This function allows multigraphs and
+ * weighted graphs as well. In this case, it computes the ratio of the
+ * total edge weight to the largest possible number of adjacent vertex
+ * pairs the graph could have. This value may be larger than 1.
  *
- * \param graph The input graph object.
- * \param res Pointer to a real number, the result will be stored
- *   here.
- * \param loops Logical constant, whether to include self-loops in the
+ * </para><para>
+ * If you need the density concept for simple graphs, make sure to
+ * eliminate any multi-edges appropriately. This can be done using
+ * \ref igraph_simplify().
+ *
+ * \param graph The input graph object. It must not have parallel edges.
+ * \param res Pointer to a real number, the result will be stored here.
+ * \param weights Vector of edge weights. Pass \c NULL to to perform
+ *   an unweighted density calculation.
+ * \param loops Boolean constant, whether to include self-loops in the
  *   calculation. If this constant is \c true then
  *   loop edges are thought to be possible in the graph (this does not
  *   necessarily mean that the graph really contains any loops). If
  *   this is \c false then the result is only correct if the graph does not
- *   contain loops.
+ *   contain loops. This function does not check if loops are actually
+ *   present.
  * \return Error code.
  *
  * Time complexity: O(1).
  */
-igraph_error_t igraph_density(const igraph_t *graph, igraph_real_t *res,
-                   igraph_bool_t loops) {
+igraph_error_t igraph_density(
+        const igraph_t *graph,
+        const igraph_vector_t *weights,
+        igraph_real_t *res,
+        igraph_bool_t loops) {
 
-    igraph_real_t no_of_nodes = (igraph_real_t) igraph_vcount(graph);
-    igraph_real_t no_of_edges = (igraph_real_t) igraph_ecount(graph);
+    const igraph_bool_t directed = igraph_is_directed(graph);
+    const igraph_int_t ecount = igraph_ecount(graph);
+    const igraph_real_t vcount = (igraph_real_t) igraph_vcount(graph); /* note real type */
+    igraph_real_t total_weight;
+
+    if (vcount == 0) {
+        *res = IGRAPH_NAN;
+        return IGRAPH_SUCCESS;
+    }
+
+    if (weights) {
+        if (igraph_vector_size(weights) != ecount) {
+            IGRAPH_ERROR("Weight vector length does not match edge count.", IGRAPH_EINVAL);
+        }
+        total_weight = igraph_vector_sum(weights);
+    } else {
+        total_weight = ecount;
+    }
+
+    if (!loops) {
+        if (vcount == 1) {
+            *res = IGRAPH_NAN;
+        } else if (directed) {
+            *res = total_weight / vcount / (vcount - 1);
+        } else {
+            *res = total_weight / vcount * 2.0 / (vcount - 1);
+        }
+    } else {
+        if (directed) {
+            *res = total_weight / vcount / vcount;
+        } else {
+            *res = total_weight / vcount * 2.0 / (vcount + 1);
+        }
+    }
+
+    return IGRAPH_SUCCESS;
+}
+
+/**
+ * \function igraph_mean_degree
+ * \brief The mean degree of a graph.
+ *
+ * This is a convenience function that computes the average of all vertex
+ * degrees. In directed graphs, the average of out-degrees and in-degrees is
+ * the same; this is the number that is returned. For the null graph, which
+ * has no vertices, NaN is returned.
+ *
+ * \param graph The input graph object.
+ * \param res Pointer to a real number, the result will be stored here.
+ * \param loops Whether to consider self-loops during the calculation.
+ * \return Error code.
+ *
+ * Time complexity: O(1) if self-loops are considered,
+ * O(|E|) where |E| is the number of edges if self-loops are ignored.
+ */
+igraph_error_t igraph_mean_degree(const igraph_t *graph, igraph_real_t *res,
+                                  igraph_bool_t loops) {
+
+    igraph_int_t no_of_nodes = igraph_vcount(graph);
+    igraph_int_t no_of_edges = igraph_ecount(graph);
     igraph_bool_t directed = igraph_is_directed(graph);
 
     if (no_of_nodes == 0) {
@@ -73,22 +141,14 @@ igraph_error_t igraph_density(const igraph_t *graph, igraph_real_t *res,
         return IGRAPH_SUCCESS;
     }
 
-    if (!loops) {
-        if (no_of_nodes == 1) {
-            *res = IGRAPH_NAN;
-        } else if (directed) {
-            *res = no_of_edges / no_of_nodes / (no_of_nodes - 1);
-        } else {
-            *res = no_of_edges / no_of_nodes * 2.0 / (no_of_nodes - 1);
-        }
-    } else {
-        if (directed) {
-            *res = no_of_edges / no_of_nodes / no_of_nodes;
-        } else {
-            *res = no_of_edges / no_of_nodes * 2.0 / (no_of_nodes + 1);
-        }
+    if (! loops) {
+        igraph_int_t loop_count;
+        IGRAPH_CHECK(igraph_count_loops(graph, &loop_count));
+        no_of_edges -= loop_count;
     }
 
+    *res = (directed ? 1.0 : 2.0) * (igraph_real_t) no_of_edges / (igraph_real_t) no_of_nodes;
+
     return IGRAPH_SUCCESS;
 }
 
@@ -102,11 +162,14 @@ igraph_error_t igraph_density(const igraph_t *graph, igraph_real_t *res,
  *
  * </para><para>
  * It is simply the (normalized) Shannon entropy of the
- * incident edges' weights. D(i)=H(i)/log(k[i]), and
- * H(i) = -sum(p[i,j] log(p[i,j]), j=1..k[i]),
- * where p[i,j]=w[i,j]/sum(w[i,l], l=1..k[i]),  k[i] is the (total)
- * degree of vertex i, and w[i,j] is the weight of the edge(s) between
- * vertex i and j. The diversity of isolated vertices will be NaN
+ * incident edges' weights.
+ * <code>D(i) = H(i) / log(k[i])</code>,
+ * and
+ * <code>H(i) = -sum(p[i,j] log(p[i,j]), j=1..k[i])</code>,
+ * where <code>p[i,j] = w[i,j] / sum(w[i,l], l=1..k[i])</code>,
+ * <code>k[i]</code> is the (total) degree of vertex \c i,
+ * and <code>w[i,j]</code> is the weight of the edge(s) between
+ * vertex \c i and \c j. The diversity of isolated vertices will be NaN
  * (not-a-number), while that of vertices with a single connection
  * will be zero.
  *
@@ -130,8 +193,8 @@ igraph_error_t igraph_density(const igraph_t *graph, igraph_real_t *res,
 igraph_error_t igraph_diversity(const igraph_t *graph, const igraph_vector_t *weights,
                      igraph_vector_t *res, const igraph_vs_t vids) {
 
-    igraph_integer_t no_of_edges = igraph_ecount(graph);
-    igraph_integer_t k, i;
+    igraph_int_t no_of_edges = igraph_ecount(graph);
+    igraph_int_t k, i;
     igraph_vector_int_t incident;
     igraph_bool_t has_multiple;
     igraph_vit_t vit;
@@ -172,9 +235,9 @@ igraph_error_t igraph_diversity(const igraph_t *graph, const igraph_vector_t *we
 
     for (IGRAPH_VIT_RESET(vit); !IGRAPH_VIT_END(vit); IGRAPH_VIT_NEXT(vit)) {
         igraph_real_t d;
-        igraph_integer_t v = IGRAPH_VIT_GET(vit);
+        igraph_int_t v = IGRAPH_VIT_GET(vit);
 
-        IGRAPH_CHECK(igraph_incident(graph, &incident, v, /*mode=*/ IGRAPH_ALL));
+        IGRAPH_CHECK(igraph_incident(graph, &incident, v, IGRAPH_ALL, IGRAPH_LOOPS));
         k = igraph_vector_int_size(&incident); /* degree */
 
         /*
@@ -219,31 +282,33 @@ igraph_error_t igraph_diversity(const igraph_t *graph, const igraph_vector_t *we
  * \function igraph_reciprocity
  * \brief Calculates the reciprocity of a directed graph.
  *
- * </para><para>
- * The measure of reciprocity defines the proportion of mutual
- * connections, in a directed graph. It is most commonly defined as
- * the probability that the opposite counterpart of a directed edge is
- * also included in the graph. In adjacency matrix notation:
- * <code>sum(i, j, (A.*A')ij) / sum(i, j, Aij)</code>, where
- * <code>A.*A'</code> is the element-wise product of matrix
- * <code>A</code> and its transpose. This measure is
- * calculated if the \p mode argument is \c
- * IGRAPH_RECIPROCITY_DEFAULT.
+ * In a directed graph, the measure of reciprocity defines the proportion of
+ * mutual connections. It is most commonly defined as the probability that the
+ * opposite counterpart of a randomly chosen directed edge is also included in
+ * the graph. In adjacency matrix notation:
+ * <code>1 - (sum_ij |A_ij - A_ji|) / (2 sum_ij A_ij)</code>.
+ * In multigraphs, each parallel edge between two vertices must have its own
+ * separate reciprocal edge, in accordance with the above formula. This measure
+ * is calculated if the \p mode argument is \c IGRAPH_RECIPROCITY_DEFAULT.
  *
  * </para><para>
- * Prior to igraph version 0.6, another measure was implemented,
- * defined as the probability of mutual connection between a vertex
- * pair if we know that there is a (possibly non-mutual) connection
- * between them. In other words, (unordered) vertex pairs are
- * classified into three groups: (1) disconnected, (2)
- * non-reciprocally connected, (3) reciprocally connected.
+ * For directed graphs with no edges, NaN is returned.
+ * For undirected graphs, 1 is returned unconditionally.
+ *
+ * </para><para>
+ * Prior to igraph version 0.6, another measure was implemented, defined as the
+ * probability of having mutual connections between a vertex pair if we know
+ * that there is a (possibly non-mutual) connection between them. In other
+ * words, (unordered) vertex pairs are classified into three groups:
+ * (1) disconnected, (2) non-reciprocally connected, (3) reciprocally connected.
  * The result is the size of group (3), divided by the sum of group
- * sizes (2)+(3). This measure is calculated if \p mode is \c
- * IGRAPH_RECIPROCITY_RATIO.
+ * sizes (2)+(3). This measure is calculated if \p mode is
+ * \c IGRAPH_RECIPROCITY_RATIO.
  *
  * \param graph The graph object.
  * \param res Pointer to an \c igraph_real_t which will contain the result.
- * \param ignore_loops Whether to ignore loop edges.
+ * \param ignore_loops Whether to ignore self-loops when counting edges.
+ *    Self-loops are considered as a mutual connection.
  * \param mode Type of reciprocity to calculate, possible values are
  *    \c IGRAPH_RECIPROCITY_DEFAULT and \c IGRAPH_RECIPROCITY_RATIO,
  *    please see their description above.
@@ -261,17 +326,16 @@ igraph_error_t igraph_reciprocity(const igraph_t *graph, igraph_real_t *res,
                        igraph_bool_t ignore_loops,
                        igraph_reciprocity_t mode) {
 
-    igraph_integer_t nonrec = 0, rec = 0, loops = 0;
+    igraph_int_t nonrec = 0, rec = 0, loops = 0;
     igraph_vector_int_t inneis, outneis;
-    igraph_integer_t i;
-    igraph_integer_t no_of_nodes = igraph_vcount(graph);
+    igraph_int_t no_of_nodes = igraph_vcount(graph);
 
     if (mode != IGRAPH_RECIPROCITY_DEFAULT &&
         mode != IGRAPH_RECIPROCITY_RATIO) {
         IGRAPH_ERROR("Invalid reciprocity type.", IGRAPH_EINVAL);
     }
 
-    /* THIS IS AN EXIT HERE !!!!!!!!!!!!!! */
+    /* Undirected graphs have reciprocity 1.0 by definition. */
     if (!igraph_is_directed(graph)) {
         *res = 1.0;
         return IGRAPH_SUCCESS;
@@ -280,14 +344,20 @@ igraph_error_t igraph_reciprocity(const igraph_t *graph, igraph_real_t *res,
     IGRAPH_VECTOR_INT_INIT_FINALLY(&inneis, 0);
     IGRAPH_VECTOR_INT_INIT_FINALLY(&outneis, 0);
 
-    for (i = 0; i < no_of_nodes; i++) {
-        igraph_integer_t ip, op;
-        IGRAPH_CHECK(igraph_neighbors(graph, &inneis, i, IGRAPH_IN));
-        IGRAPH_CHECK(igraph_neighbors(graph, &outneis, i, IGRAPH_OUT));
+    for (igraph_int_t i = 0; i < no_of_nodes; i++) {
+        igraph_int_t ip, op, indeg, outdeg;
+        IGRAPH_CHECK(igraph_neighbors(
+            graph, &inneis, i, IGRAPH_IN, IGRAPH_LOOPS_ONCE, IGRAPH_MULTIPLE
+        ));
+        IGRAPH_CHECK(igraph_neighbors(
+            graph, &outneis, i, IGRAPH_OUT, IGRAPH_LOOPS_ONCE, IGRAPH_MULTIPLE
+        ));
+
+        indeg = igraph_vector_int_size(&inneis);
+        outdeg = igraph_vector_int_size(&outneis);
 
         ip = op = 0;
-        while (ip < igraph_vector_int_size(&inneis) &&
-               op < igraph_vector_int_size(&outneis)) {
+        while (ip < indeg && op < outdeg) {
             if (VECTOR(inneis)[ip] < VECTOR(outneis)[op]) {
                 nonrec += 1;
                 ip++;
@@ -310,8 +380,12 @@ igraph_error_t igraph_reciprocity(const igraph_t *graph, igraph_real_t *res,
                 op++;
             }
         }
-        nonrec += (igraph_vector_int_size(&inneis) - ip) +
-                  (igraph_vector_int_size(&outneis) - op);
+        nonrec += (indeg - ip) + (outdeg - op);
+    }
+
+    /* If we found non-loop mutual connections, we can set the cache. */
+    if (rec - (ignore_loops ? 0 : loops) > 0) {
+        igraph_i_property_cache_set_bool_checked(graph, IGRAPH_PROP_HAS_MUTUAL, true);
     }
 
     if (mode == IGRAPH_RECIPROCITY_DEFAULT) {
@@ -327,5 +401,6 @@ igraph_error_t igraph_reciprocity(const igraph_t *graph, igraph_real_t *res,
     igraph_vector_int_destroy(&inneis);
     igraph_vector_int_destroy(&outneis);
     IGRAPH_FINALLY_CLEAN(2);
+
     return IGRAPH_SUCCESS;
 }
