@@ -1,6 +1,5 @@
-/* -*- mode: C -*-  */
 /*
-   IGraph library.
+   igraph library.
    (C) 2006-2021 The igraph development team  Gabor Csardi <csardi.gabor@gmail.com>
 
    This program is free software; you can redistribute it and/or modify
@@ -28,33 +27,33 @@ void check_evecs(const igraph_t *graph, const igraph_vector_int_list_t *vecs,
                  const igraph_vector_int_list_t *evecs) {
 
     igraph_bool_t directed = igraph_is_directed(graph);
-    igraph_integer_t i, n = igraph_vector_int_list_size(vecs);
+    igraph_int_t i, n = igraph_vector_int_list_size(vecs);
 
     IGRAPH_ASSERT(igraph_vector_int_list_size(evecs) == n);
 
     for (i = 0; i < n; i++) {
         igraph_vector_int_t *vvec = igraph_vector_int_list_get_ptr(vecs, i);
         igraph_vector_int_t *evec = igraph_vector_int_list_get_ptr(evecs, i);
-        igraph_integer_t j, n2 = igraph_vector_int_size(evec);
+        igraph_int_t j, n2 = igraph_vector_int_size(evec);
         if (igraph_vector_int_size(vvec) == 0 && n2 == 0) {
             continue;
         }
         IGRAPH_ASSERT(igraph_vector_int_size(vvec) == n2 + 1);
 
         for (j = 0; j < n2; j++) {
-            igraph_integer_t edge = VECTOR(*evec)[j];
-            igraph_integer_t from = VECTOR(*vvec)[j];
-            igraph_integer_t to = VECTOR(*vvec)[j + 1];
+            igraph_int_t edge = VECTOR(*evec)[j];
+            igraph_int_t from = VECTOR(*vvec)[j];
+            igraph_int_t to = VECTOR(*vvec)[j + 1];
             if (directed) {
                 IGRAPH_ASSERT(from == IGRAPH_FROM(graph, edge) &&
                               to == IGRAPH_TO(graph, edge));
             } else {
-                igraph_integer_t from2 = IGRAPH_FROM(graph, edge);
-                igraph_integer_t to2 = IGRAPH_TO(graph, edge);
-                igraph_integer_t min1 = from < to ? from : to;
-                igraph_integer_t max1 = from < to ? to : from;
-                igraph_integer_t min2 = from2 < to2 ? from2 : to2;
-                igraph_integer_t max2 = from2 < to2 ? to2 : from2;
+                igraph_int_t from2 = IGRAPH_FROM(graph, edge);
+                igraph_int_t to2 = IGRAPH_TO(graph, edge);
+                igraph_int_t min1 = from < to ? from : to;
+                igraph_int_t max1 = from < to ? to : from;
+                igraph_int_t min2 = from2 < to2 ? from2 : to2;
+                igraph_int_t max2 = from2 < to2 ? to2 : from2;
                 IGRAPH_ASSERT(min1 == min2 && max1 == max2);
             }
         }
@@ -64,7 +63,7 @@ void check_evecs(const igraph_t *graph, const igraph_vector_int_list_t *vecs,
 void check_parents_inbound(const igraph_t* graph, const igraph_vector_int_t* parents,
                         const igraph_vector_int_t* inbound, int start) {
 
-    igraph_integer_t i, n = igraph_vcount(graph);
+    igraph_int_t i, n = igraph_vcount(graph);
 
     IGRAPH_ASSERT(igraph_vector_int_size(parents) == n);
     IGRAPH_ASSERT(igraph_vector_int_size(inbound) == n);
@@ -81,10 +80,10 @@ void check_parents_inbound(const igraph_t* graph, const igraph_vector_int_t* par
             IGRAPH_ASSERT(VECTOR(*inbound)[i] == -1);
 
         } else {
-            igraph_integer_t eid = VECTOR(*inbound)[i];
-            igraph_integer_t u = IGRAPH_FROM(graph, eid), v = IGRAPH_TO(graph, eid);
+            igraph_int_t eid = VECTOR(*inbound)[i];
+            igraph_int_t u = IGRAPH_FROM(graph, eid), v = IGRAPH_TO(graph, eid);
             if (v != i && !igraph_is_directed(graph)) {
-                igraph_integer_t dummy = u;
+                igraph_int_t dummy = u;
                 u = v;
                 v = dummy;
             }
@@ -103,7 +102,7 @@ int main(void) {
     igraph_real_t weights_data_2[] = { 6, 7, 2, -4, -2, -3, 9, 2, 7 };
     igraph_vector_t weights_vec;
     igraph_vs_t vs;
-    igraph_integer_t vs_size;
+    igraph_int_t vs_size;
 
     igraph_small(&g, 10, IGRAPH_DIRECTED,
                 0, 1, 0, 2, 0, 3,    1, 2, 1, 4, 1, 5,
@@ -125,7 +124,7 @@ int main(void) {
     igraph_vector_int_list_init(&vecs, 0);
     igraph_vector_int_list_init(&evecs, 0);
 
-    igraph_vector_view(&weights_vec, weights_data_0, sizeof(weights_data_0) / sizeof(weights_data_0[0]));
+    weights_vec = igraph_vector_view(weights_data_0, sizeof(weights_data_0) / sizeof(weights_data_0[0]));
     igraph_get_shortest_paths_bellman_ford(&g, /*vertices=*/ &vecs, /*edges=*/ &evecs,
                                            /*from=*/ 0, /*to=*/ vs,
                                            &weights_vec, IGRAPH_OUT,
@@ -178,7 +177,7 @@ int main(void) {
                  0, 1, 0, 3, 1, 3, 1, 4, 2, 1, 3, 2, 3, 4, 4, 0, 4, 2,
                  -1);
 
-    igraph_vector_view(&weights_vec, weights_data_1, sizeof(weights_data_1) / sizeof(weights_data_1[0]));
+    weights_vec = igraph_vector_view(weights_data_1, sizeof(weights_data_1) / sizeof(weights_data_1[0]));
     igraph_get_shortest_paths_bellman_ford(&g, /*vertices=*/ &vecs, /*edges=*/ &evecs,
                                            /*from=*/ 0, /*to=*/ vs,
                                            &weights_vec, IGRAPH_OUT,
@@ -194,13 +193,13 @@ int main(void) {
 
     /* Same graph with negative loop */
     igraph_set_error_handler(igraph_error_handler_ignore);
-    igraph_vector_view(&weights_vec, weights_data_2,
+    weights_vec = igraph_vector_view(weights_data_2,
                        sizeof(weights_data_2) / sizeof(weights_data_2[0]));
     IGRAPH_ASSERT(igraph_get_shortest_paths_bellman_ford(&g, /*vertices=*/ &vecs, /*edges=*/ &evecs,
                                                          /*from=*/ 0, /*to=*/ vs,
                                                          &weights_vec, IGRAPH_OUT,
                                                          &parents,
-                                                         /*inbound_edges=*/ &inbound) == IGRAPH_ENEGLOOP);
+                                                         /*inbound_edges=*/ &inbound) == IGRAPH_ENEGCYCLE);
 
     igraph_vector_int_list_destroy(&vecs);
     igraph_vector_int_list_destroy(&evecs);
