@@ -1,6 +1,6 @@
 /* -*- mode: C++ -*-  */
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2011-2012  Gabor Csardi <csardi.gabor@gmail.com>
    334 Harvard st, Cambridge MA, 02139 USA
 
@@ -30,7 +30,7 @@ int main(void) {
     igraph_t karate;
     igraph_vector_int_t parents;
     igraph_vector_t weights;
-    igraph_integer_t i, n;
+    igraph_int_t i, n;
 
     igraph_rng_seed(igraph_rng_default(), 42);
 
@@ -76,7 +76,7 @@ int main(void) {
             igraph_vector_size(&weights), igraph_vcount(&karate),
             igraph_vector_int_size(&parents)
         );
-        abort();
+        IGRAPH_FATAL("Vector length mismatch.");
     }
 
     n = igraph_vector_int_size(&parents);
@@ -84,7 +84,7 @@ int main(void) {
         if (VECTOR(parents)[i] < -1 || VECTOR(parents)[i] >= igraph_vcount(&karate) + igraph_vector_size(&weights)) {
             fprintf(stderr, "Invalid parents vector:\n");
             igraph_vector_int_fprint(&parents, stderr);
-            abort();
+            IGRAPH_FATAL("Invalid parents vector.");
         }
     }
 
