@@ -1,6 +1,5 @@
-/* -*- mode: C -*-  */
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2006-2012  Gabor Csardi <csardi.gabor@gmail.com>
    334 Harvard st, Cambridge MA, 02139 USA
 
@@ -28,8 +27,8 @@
 
 /* Compare the elements of two vectors for equality, handling NaN values. */
 igraph_bool_t vector_equal(const igraph_vector_t *v1, const igraph_vector_t *v2) {
-    igraph_integer_t n1 = igraph_vector_size(v1), n2 = igraph_vector_size(v2);
-    igraph_integer_t i;
+    igraph_int_t n1 = igraph_vector_size(v1), n2 = igraph_vector_size(v2);
+    igraph_int_t i;
 
     if (n1 != n2) {
         return 0;
@@ -50,8 +49,8 @@ igraph_bool_t vector_equal(const igraph_vector_t *v1, const igraph_vector_t *v2)
 
 /* Compute the average of a vector, ignoring NaN values. */
 igraph_real_t vector_avg(const igraph_vector_t *v) {
-    igraph_integer_t n = igraph_vector_size(v);
-    igraph_integer_t i;
+    igraph_int_t n = igraph_vector_size(v);
+    igraph_int_t i;
     igraph_real_t sum = 0.0, count;
 
     count = 0;
@@ -87,8 +86,7 @@ int main(void) {
 
     /* 100 vertices */
 
-    igraph_erdos_renyi_game(&g, IGRAPH_ERDOS_RENYI_GNP, 100, 0.1,
-                            IGRAPH_UNDIRECTED, IGRAPH_NO_LOOPS);
+    igraph_erdos_renyi_game_gnp(&g, 100, 0.1, IGRAPH_UNDIRECTED, IGRAPH_SIMPLE_SW, IGRAPH_EDGE_UNLABELED);
 
     igraph_vs_range(&vertices, 0, igraph_vcount(&g));
 
@@ -104,8 +102,7 @@ int main(void) {
 
     /* 50 vertices */
 
-    igraph_erdos_renyi_game(&g, IGRAPH_ERDOS_RENYI_GNP, 50, 0.3,
-                            IGRAPH_UNDIRECTED, IGRAPH_NO_LOOPS);
+    igraph_erdos_renyi_game_gnp(&g, 50, 0.3, IGRAPH_UNDIRECTED, IGRAPH_SIMPLE_SW, IGRAPH_EDGE_UNLABELED);
 
     igraph_vs_range(&vertices, 0, igraph_vcount(&g));
 
